package hiit.blackboard;

import hiit.blackboard.protocol.ComponentInfo;

public class BlackboardUtil {
    /**
     * Checks if a ComponentInfo (or CustomerInfo) matches a set of parameters specified. 
     * Each parameter may be Integer.MAX_VALUE to match all alternatives.
     * @param type The type of the component as specified in Properties
     * @param user The user id of the user owning the component
     * @param id The unique component id
     * @param cInfo the ComponentInfo description to compare to
     * @return if the ComponentInfo description matches the parameters.
     */
    public static boolean matches(int type, int user, int id, ComponentInfo cInfo){
        if (cInfo == null) // invalid params
            return false;
        boolean typeMatch = false;
        boolean userMatch = false;
        boolean idMatch = false;
        
        if (type == Integer.MAX_VALUE || type == cInfo.getComponentType())
            typeMatch = true;
        if (user == Integer.MAX_VALUE || user == cInfo.getUserId())
            userMatch = true;
        if (id == Integer.MAX_VALUE || id == cInfo.getUserId())
            idMatch = true;
        return typeMatch && userMatch && idMatch;
    }    
}
