package hiit.blackboard;

import hiit.blackboard.CustomerSkeleton.SenderThread;
import hiit.blackboard.event.BasicEvent;
import hiit.blackboard.event.EventData;
import hiit.blackboard.protocol.Protocol;
import hiit.connection.Connection;
import hiit.logger.Logger;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Date;
import java.util.Hashtable;
import java.util.Vector;

public class CallCustomer extends CustomerSkeleton{
    
    protected String name = "GSMCustomer";
    
    public static final String classId = "[GC]";
    
    public static final int standoffTime = 120000; 
    
    protected TimerThread t = null;
        
    private class TimerThread extends Thread{
        private long lastTime = 0;
        private CallCustomer parent = null;
        
        public TimerThread(CallCustomer p){
            this.parent = p;
        }
        
        
        public synchronized void run(){
            reset();
            while (running){
                try {
                    this.wait(standoffTime);
                } catch (InterruptedException e) {
                    // Intentionally empty
                }
                long now = System.currentTimeMillis();
                if (lastTime + standoffTime < now){ // not received anything for 2 minutes!
                    sendZeroData();
                    lastTime = now;
                }
            }
        }
        public void reset(){
            lastTime = System.currentTimeMillis();
        }
        
        private void sendZeroData(){
            Object[] calldata = new Object[14];
            calldata[0] = "";
            calldata[1] = "";
            calldata[2] = "";
            calldata[3] = "";
            calldata[4] = "";
            calldata[5] = "";
            calldata[6] = "";
            calldata[7] = "";
            calldata[8] = "";
            calldata[9] = "";
            calldata[10] = "";
            calldata[11] = "";
            calldata[12] = "";
            calldata[13] = "";
            
            
            Vector v = new Vector();
            for (int i = 0; i < calldata.length; ++i) {
                v.addElement(calldata[i]);
            }
            BlackboardFactory.getBlackboard(logger).pushData(
                    parent, new CustomerData(new Vector[] { v }, 
                            new Date().getTime(), info), null);
        }
    };
    
    public CallCustomer(Logger logger, int ctype, int user, Connection s,
            InputStream is, OutputStream os) {
        super(logger, ctype, user, s, is, os);
    }
    
    
    public synchronized void run() {
        running = true;
        t = new TimerThread(this);
        t.start();
        while (running) {
            //channelFree = true;
            int command = readCommand();
            if (command == -1)
                return;
            switch (command) {
            case 's': // subscribe
                // readSubRequest(command);
                /* We do not do this with GSM info (server does not want
                 * stuff back, it never subs)*/ 
                break;
            case 'u': // unsubscribe
                //readUnSubRequest(command);
                break;
            case 'p': // data from client
                readPushData();
                break;
            case 'l': // client requested list of components
                readListComponents();
                break;
            case 'r':
                // Return - never sent by client
            case 't':
                // Subscribed - Never sent by client.
            case 'e':
                // Error message from client? Not possible...
            case 'c':
                // New create? Not possible; this should be sent to the BB instead.
            default:
                if (logger != null)
                    logger.logMessage(classId + " Received incorrect command byte: " + (char)command + " (" + (int)command + ")");
            }
        }
    }
    
    /**
     * Read a data message from our client and pass it on to the Blackboard.
     */
    protected void readPushData(){
    	try {
        // Pushdata.
        Object[] stuff = Protocol.readPushData(in);
        int compId = ((Integer) stuff[0]).intValue();
        if (compId == -1)
            compId = Integer.MAX_VALUE;
        
        byte[] data = (byte[]) stuff[1];
        String s = new String(data);
        int tab = -1;
       
        tab = s.indexOf('\t');
        String inboxSize = (s.substring(0, tab));
        s = s.substring(tab + 1);
        tab = s.indexOf('\t');
        String unreadMessages = (s.substring(0, tab));
        s = s.substring(tab + 1);
        tab = s.indexOf('\t');
        String phoneState = (s.substring(0, tab));
        s = s.substring(tab + 1);
        
        String[] callPackages = null;
        String[] smsPackages = null;
        String[] smsReadPackages = null;
        
        //Read number of call packages
        tab = s.indexOf('\t');
        int numberOfCallPackages = Integer.parseInt((s.substring(0, tab)));
        s = s.substring(tab + 1);
        
        callPackages = new String[numberOfCallPackages];
        for(int i=0; i<numberOfCallPackages; i++){
            tab = s.indexOf('\t');
            callPackages[i] = s.substring(0, tab);
            s = s.substring(tab + 1);
        }
        
        //Read number of sms packages
        tab = s.indexOf('\t');
        int numberOfSmsPackages = Integer.parseInt((s.substring(0, tab)));
        s = s.substring(tab + 1);
        
        smsPackages = new String[numberOfSmsPackages];
        for(int i=0; i<numberOfSmsPackages; i++){
            tab = s.indexOf('\t');
            smsPackages[i] = s.substring(0, tab);
            s = s.substring(tab + 1);
        }
        
        //Read number of call packages
        tab = s.indexOf('\t');
        int numberOfSmsReadPackages = Integer.parseInt((s.substring(0, tab)));
        if (numberOfSmsReadPackages > 0)
        	s = s.substring(tab + 1);
        
        smsReadPackages =  new String[numberOfSmsReadPackages];
        for(int i=0; i<numberOfSmsReadPackages; i++){
            tab = s.indexOf('\t');
            smsReadPackages[i] = s.substring(0, tab);
            s = s.substring(tab + 1);
        }
        
        /*
         * Check if we really have any new data or not...
         */
        if((numberOfCallPackages + numberOfSmsPackages + numberOfSmsReadPackages) == 0){
            t.reset();
            return;
        }
        
        Vector[] parsedData = new Vector[14];
        
        Vector inboxSizeVec = new Vector();
        inboxSizeVec.addElement(inboxSize);
        parsedData[0] = inboxSizeVec;
        Vector unreadMessagesVec = new Vector();
        unreadMessagesVec.addElement(unreadMessages);
        parsedData[1] = unreadMessagesVec;
        Vector phoneStateVec = new Vector();
        phoneStateVec.addElement(phoneState);
        parsedData[2] = phoneStateVec;
        
        Vector callConnectedTime = new Vector();
        Vector callDuration = new Vector();
        Vector callDirection = new Vector();
        Vector callNumber = new Vector();
        
        for(int i=0; i<numberOfCallPackages; i++){
            String temp = callPackages[i];
            
            int coma = temp.indexOf(',');
            callConnectedTime.addElement(temp.substring(0,coma));
            temp = temp.substring(coma + 1);
            
            coma = temp.indexOf(',');
            callDuration.addElement(temp.substring(0, coma));
            temp = temp.substring(coma + 1);
            
            coma = temp.indexOf(',');
            callDirection.addElement(temp.substring(0, coma));
            callNumber.addElement(temp.substring(coma + 1));
            
        }
        
        parsedData[3] = callConnectedTime;
        parsedData[4] = callDuration;
        parsedData[5] = callDirection;
        parsedData[6] = callNumber;
        
        Vector smsTime = new Vector();
        Vector smsDirection = new Vector();
        Vector smsNumber = new Vector();
        
        for(int i=0; i<numberOfSmsPackages; i++){
            String temp = smsPackages[i];
            
            int coma = temp.indexOf(',');
            smsTime.addElement(temp.substring(0,coma));
            temp = temp.substring(coma + 1);
            
            coma = temp.indexOf(',');
            smsDirection.addElement(temp.substring(0, coma));
            smsNumber.addElement(temp.substring(coma + 1));
        }
        
        parsedData[7] = smsTime;
        parsedData[8] = smsDirection;
        parsedData[9] = smsNumber;
        
        Vector smsReadTime = new Vector();
        Vector smsReadDirection = new Vector();
        Vector smsReadNumber = new Vector();
        Vector smsReadTimeSpent = new Vector();
        
        for(int i=0; i<numberOfSmsReadPackages; i++){
            String temp = smsReadPackages[i];
            
            int coma = temp.indexOf(',');
            smsReadTime.addElement(temp.substring(0,coma));
            temp = temp.substring(coma + 1);
            
            coma = temp.indexOf(',');
            smsReadDirection.addElement(temp.substring(0, coma));
            temp = temp.substring(coma + 1);
            
            coma = temp.indexOf(',');
            smsReadNumber.addElement(temp.substring(0, coma));
            smsReadTimeSpent.addElement(temp.substring(coma + 1));
            
        }
        
        parsedData[10] = smsReadTime;
        parsedData[11] = smsReadDirection;
        parsedData[12] = smsReadNumber;
        parsedData[13] = smsReadTimeSpent;
        
        CustomerData d = new CustomerData(parsedData, new Date().getTime(), info);
        BlackboardFactory.getBlackboard(logger).pushData(this, d, null);
        t.reset();
    	} catch (Throwable t) {
        	if (this.logger != null) {
        		logger.logMessage("CallCustomer: " + t.toString());
        	}
        }
    }
    
    public void newData(Customer prod, CustomerData pd) {
        // We do not want data from others
    }
    
    public void updateProducerList(Blackboard bb, Vector customers) {
        // subscriptions not used in this class
    }


	

    public void newData(Customer prod, CustomerData pd, EventData eventData){
		// TODO Auto-generated method stub
		
	}
}
