package hiit.blackboard;

import hiit.blackboard.event.EventData;
import hiit.blackboard.event.EventHandler;

import java.util.Vector;

public interface Customer {
    
    /**
     * Returns the CustomerInfo for this Customer.
     * 
     * @return the CustomerInfo for this Customer.
     */
    public CustomerInfo getInfo();
    
    /**
     * Set the CustomerInfo.
     * 
     * @param info
     *            the CustomerInfo to replace the current one with.
     */
    //public void setCustomerInfo(CustomerInfo info);
    
    /**
     * asks the Customer to produce new data.
     */
    public boolean requestData();
    
    /**
     * Sets the data production mode: Streamed (0) or on-request (1)? and
     * frequency in milliseconds.
     * 
     * @param mode
     * @param timeout
     */
    public boolean setProducerMode(int mode, long timeout);
    
    /**
     * Notifies this component about new customers, so that we can choose which
     * ones to subscribe to
     */
    
    public void updateProducerList(Blackboard bb, Vector producers);
    
    public void updateCustomerInfo(Blackboard bb, Customer c, CustomerInfo pi);
    
    public void newData(Customer prod, CustomerData pd);
   // public void newData(Customer prod, CustomerData pd, EventData eventData);
    public void newData(Customer prod, CustomerData pd, EventData eventData);
    
    /**
     * If the Customer supports events and has an EventHandler associated with it,
     * this method can be used to mark the current parameters as set of parameters
     * to which all future parameter groups are compared to.
     */
    
    public void markParameters();
    
}
