package hiit.blackboard;

import hiit.BetelgeuseMIDP;
import hiit.mapper.Properties;
import hiit.blackboard.event.Event;
import hiit.blackboard.event.EventData;
import hiit.blackboard.protocol.Protocol;
import hiit.connection.Connection;
import hiit.connection.MIDPServer;
import hiit.connection.ServerConnection;
import hiit.connectorUnit.BTDeviceConnector;
import hiit.logger.Logger;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

public class CustomerBoard implements Blackboard {
    
    private Vector producers;
    
    private Hashtable producerInfo;
    
    // (Producer -> Vector of Consumers)
    private Hashtable producerSubscribers;
    
    // Integer -> Customer
    private Hashtable customersById;
    
    private Hashtable producerData;
    
    private Vector subscribers;
    
    private Logger logger = null;
    
    private ListenerThread lt = null;
    
    private static Blackboard instance = null;
    
    private BTDeviceConnector connector;
    
    private Vector eventList;
    
   
    
    // Customer -> Vector of Events
    private Hashtable producerEventTable;
    
    
    
    // EventName -> Vector of Listeners/Customers
    private Hashtable eventSubscribers;
    // Producer -> Hashtable of Events -> Vector of Listeners/Customers
    private Hashtable producerEventSubscribers;
    
    
    
    /*Taru:*/
    /*Methods setting and returning the username and bluetoothaddress for capella-users.
     * Implemented here, while HttpServer uses Blackboard...
     */
    private String username;
    private String btaddress;
    private int sessionId;
    
    
    
    
    public String getUsername(){
    	return this.username;
    }
    
    public void setUsername(String username){
    	this.username=username;
    }
    
    public String getBTAddress(){
    	return this.btaddress;
    	
    }
    
    public void setBTAddress(String btaddress){
    	this.btaddress=btaddress;
    }
    
    public int getSessionId(){
    	return this.sessionId;
    	
    }
    
    public void setSessionId(int id){
    	this.sessionId=id;
    }
    /*-->Taru*/
    
    protected static Blackboard getInstance(Logger logger, BTDeviceConnector connector) {
        if (instance == null)
            instance = new CustomerBoard(logger, connector);
        return instance;
    }
    
    private class ListenerThread extends Thread {
        private ServerConnection ss;
        
        private boolean listening = false; 
        
        public ListenerThread() {
            init();
        }
        
        private void init() {
            ss = new MIDPServer(logger);
            ss = ss.open(Properties.LISTEN_PORT); // REMEMBER THIS!
            listening = true;
        }
        
        public synchronized void run() {
            while (listening) {
                Connection s = listenSocket();
                if (BlackboardFactory.DEBUG && logger != null)
                    logger.logMessage("Got a connection.");
                InputStream in = s.getIs();
                parseMessage(in, s);
            }
        }
        
        public void stopListening(){
            this.listening = false;
        }
        
        private void parseMessage(InputStream is, Connection s) {
            try {
                int mtype = is.read();
                if (mtype != 'c') {
                    s.close();
                    return;
                }
                // create message received.
                Object[] params = Protocol.readCreate(is);
                int type = ((Integer) params[0]).intValue();
                int userid = ((Integer) params[1]).intValue();
                String username = new String((byte[]) params[2]);
                
                // TODO: Do something for users
                if (BlackboardFactory.DEBUG && logger != null)
                    logger.logMessage("Got a create message with type=" + type
                            + " userid=" + userid + " username=" + username);
                OutputStream os = s.getOs();
                
                Customer c = addCustomer(type, userid, s, is, os);
            } catch (IOException e) {
                if (logger != null)
                    logger.logMessage("Failed to read from stream: " + e);
            }
        }
        
        public Connection listenSocket() {
            return ss.accept();
        }
    }
    
    private CustomerBoard(Logger logger, BTDeviceConnector connector) {
        this.logger = logger;
        this.producers = new Vector();
        this.subscribers = new Vector();
        this.producerInfo = new Hashtable();
        this.producerSubscribers = new Hashtable();
        this.producerData = new Hashtable();
        this.customersById = new Hashtable();
        this.eventList = new Vector();
        this.producerEventTable = new Hashtable();
        
        this.producerEventSubscribers = new Hashtable();
        this.eventSubscribers = new Hashtable();
        
        this.connector = connector;
        this.lt = new ListenerThread();
        lt.start();
    }
    
    public Customer addCustomer(int type, int userid, Connection s,
            InputStream is, OutputStream os) {
        try {
            if (s == null || is == null || os == null) // invalid parameters
                return null;

            Hashtable mapping = Properties.getServerMapping();
            Object o;
            if((o = mapping.get(new Integer(type))) != null){
                int readerType = ((Integer)o).intValue();        

                ByteArrayOutputStream b = new ByteArrayOutputStream();
                Protocol.writeCreated(b, type, userid, ((Integer)o).intValue()); // formulate "creation
                // "successful" message.
                try {
                    os.write(b.toByteArray());
                    os.flush();
                } catch (IOException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } // kept open for Customer

                return this.connector.createProducerReader(null, null, readerType, is, os, userid);
            }else{


                /*
                 * IF YOU HAVE A SPECIAL CUSTOMER CLASS, ADD IT'S CREATION HERE!
                 * 
                 * IF YOU ARE USING A READER THAT IMPLEMENTS THE CustomerReaderSkeleton,
                 * ADD A MAPPING TO THE PROPERTIES CLASS INSTEAD!
                 */
                //FIXME delete the following lines when the new customer architechture is complete
                if (type == hiit.mapper.Properties.GSM_SERVER) {
                    return addGSMCustomer(type, userid, s, is, os);
                } else {
                    return addGenericCustomer(type, userid, s, is, os);
                }
            }
        } catch (Throwable t) {
            if (this.logger != null) {
                logger.logMessage("Blacboard: " + t.toString());
            }
        }
        return null;

    }
    
    /*
    private Customer addCallCustomer(int type, int userid, Connection s,
            InputStream is, OutputStream os) {
        if (s == null || is == null || os == null) // invalid parameters
            return null;
        CallCustomer c = new CallCustomer(logger, type, userid, s, is, os);
        customersById.put(new Integer(c.getInfo().getId()), c);
        // FIXME: Add this to the Consumer or the Producer side.
        // Need to know if it will produce anything.
        // Need to change protocol for that.
        registerProducer(c, c.getInfo());
        int id = c.getInfo().getId();
        
        ByteArrayOutputStream b = new ByteArrayOutputStream();
        Protocol.writeCreated(b, type, userid, id); // formulate "creation
        // successful" message.
        try {
            os.write(b.toByteArray());
            os.flush();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } // kept open for Customer
        c.start();
        if (logger != null)
            logger.logMessage("Started GSMCustomer " + id);
        return c;
    }
    */
    
    private Customer addGSMCustomer(int type, int userid, Connection s,
            InputStream is, OutputStream os) {
        if (s == null || is == null || os == null) // invalid parameters
            return null;
        GSMCustomer c = new GSMCustomer(logger, type, userid, s, is, os);
        customersById.put(new Integer(c.getInfo().getId()), c);
        // FIXME: Add this to the Consumer or the Producer side.
        // Need to know if it will produce anything.
        // Need to change protocol for that.
        registerProducer(c, c.getInfo(), null);
        int id = c.getInfo().getId();
        
        ByteArrayOutputStream b = new ByteArrayOutputStream();
        Protocol.writeCreated(b, type, userid, id); // formulate "creation
        // successful" message.
        try {
            os.write(b.toByteArray());
            os.flush();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } // kept open for Customer
        c.start();
        if (logger != null)
            logger.logMessage("Started GSMCustomer " + id);
        return c;
    }
    
    private Customer addGenericCustomer(int type, int userid, Connection s,
            InputStream is, OutputStream os){
        if (s == null || is == null || os == null) // invalid parameters
            return null;
        SocketCustomer c = new SocketCustomer(logger, type, userid, s, is, os);
        customersById.put(new Integer(c.getInfo().getId()), c);
        // FIXME: Add this to the Consumer or the Producer side.
        // Need to know if it will produce anything.
        // Need to change protocol for that.
        registerProducer(c, c.getInfo(), null);
        int id = c.getInfo().getId();
        
        ByteArrayOutputStream b = new ByteArrayOutputStream();
        Protocol.writeCreated(b, type, userid, id); // formulate "creation
                                                    // successful" message.
        try {
            os.write(b.toByteArray());
            os.flush();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } // kept open for Customer
        c.start();
        if (logger != null)
            logger.logMessage("Started SocketCustomer " + id);
        return c;
    }
    
    public void announceInfo(Customer prod, CustomerInfo pi) {
        if (prod == null || pi == null)
            return;
        if (this.producerInfo == null || producerSubscribers == null)
            return;
        this.producerInfo.put(prod, pi);
        if (this.producerSubscribers.get(prod) != null) {
            Vector temp = (Vector) this.producerSubscribers.get(prod);
            for (int i = 0; i < temp.size(); i++)
                ((Customer) temp.elementAt(i)).updateCustomerInfo(this, prod,
                        pi);
        }
    }
    
    public CustomerData getData(Customer prod) {
        if (prod == null) // invalid parameters
            return null;
        if (this.producerData == null)
            return null;
        Object o = this.producerData.get(prod);
        if (o != null)
            return (CustomerData) o;
        return null;
    }
    
    
    public CustomerInfo getCustomerInfo(Customer prod) {
        if (prod == null) // invalid parameters
            return null;
        if (this.producerInfo == null)
            return null;
        Object o = this.producerInfo.get(prod);
        if (o != null)
            return (CustomerInfo) o;
        return null;
    }
    
    public Customer[] getProducers() {
        if (this.producers == null){
            return new Customer[0];
        }
        Customer[] temp = new Customer[this.producers.size()];
        for (int i = 0; i < temp.length; i++) {
            temp[i] = (Customer) this.producers.elementAt(i);
        }
        return temp;
    }
    /*
    public void pushData(Customer prod, CustomerData data) {
        if (prod == null || data == null)
            return;
        this.producerData.put(prod, data);
        if (BlackboardFactory.DEBUG && logger != null)
            logger.logMessage("Subscribers for me?: "
                    + (this.producerSubscribers.get(prod) != null));
        
        // raw data
        if (this.producerSubscribers.get(prod) != null) {
            Vector producerListeners = (Vector) this.producerSubscribers.get(prod);
            for (int i = 0; i < producerListeners.size(); i++) {
                ((Customer) producerListeners.elementAt(i)).newData(prod, data);
            }
        }
    }*/
    public void pushData(Customer prod, CustomerData data, EventData[] eventData) {
        if (prod == null || data == null )
            return;

        this.producerData.put(prod, data);
        
        // BetelgeuseMIDP.logger.logMessage("BB: pushData: " + prod + ", " + data + "," + eventData + ", subcribers?: " + this.producerSubscribers.containsKey(prod) );
        
        // producerSubscribers, raw data
        if (this.producerSubscribers.containsKey(prod)){
	        Vector prodSubscribers = (Vector)this.producerSubscribers.get(prod);
	    	
	        for (int i = 0; i < prodSubscribers.size(); i++) {
	    		try {
	    			Customer c = (Customer)prodSubscribers.elementAt(i);
	    			c.newData(prod, data);
	    		} catch (Exception e){
	    			BetelgeuseMIDP.logger.logMessage("BB: pushData RAW ERR: " + e);
				}
	    	}
        }
    	
    	if (eventData != null){
    		
    		for (int i = 0; i < eventData.length; i++) {
    			// producer-event-listeners
    			EventData d = eventData[i];
	    		String eventName = d.getName();
	    		
	    		 // Producer -> Hashtable of Events -> Vector of Listeners/Customers
	    		if (this.producerEventSubscribers.containsKey(prod)){
		    		Hashtable prodEvents = (Hashtable)this.producerEventSubscribers.get(prod);
		    		
		    		if (prodEvents.containsKey(eventName)){
		    			Vector evListeners = (Vector)prodEvents.get(eventName);
		    			//BetelgeuseMIDP.logger.logMessage("BB: evListeners: " + evListeners.size());
		    			
		    			for (int j=0; j<evListeners.size(); j++){
		    				try {
		    					Customer c = (Customer)evListeners.elementAt(j);
		    					c.newData(prod, data, d);
		    					// if (logger != null) logger.logMessage("BB: push: " + prod + ", ev: " + eventName + ", to: " + c);
		    					//System.out.println("BB: push: " + prod + ", ev: " + eventName + ", to: " + c);
		    					
 		    				} catch (Exception e){
 		    					BetelgeuseMIDP.logger.logMessage("BB pushData EV ERR: " + e);
 		    					e.printStackTrace();
		    				}
		    			}
		    		}
	    		}
	    	}
    	}
    }

    public void kill(){
        if (this.lt == null)
            return;
        this.lt.stopListening();
    }
    
    public void registerProducer(Customer prod, CustomerInfo info, String[] events) {    	
        if (prod == null || info == null) // invalid parameters
            return; 
        
        if (!this.producers.contains(prod)) {
            this.producers.addElement(prod);
            this.producerInfo.put(prod, info);
        }

        if (events != null ){
            Vector ev = new Vector();
            for (int i=0; i<events.length; i++){
            	ev.addElement(events[i]);
            }
            this.producerEventTable.put(prod, ev);
        }
        
        
        for (int i = 0; i < subscribers.size(); i++) {
            Customer cons = (Customer) subscribers.elementAt(i);
            cons.updateProducerList(this, this.producers);
        }
    }
    
    public void removeProducer(Customer prod) {
        if (prod == null) // invalid parameters
            return; 
        this.producers.removeElement(prod);
        this.producerInfo.remove(prod);
        for (int i = 0; i < subscribers.size(); i++) {
            Customer cons = (Customer) subscribers.elementAt(i);
            cons.updateProducerList(this, this.producers);
        }
    }
    
    public boolean requestNewData(Customer prod) {
        if (prod == null) // invalid parameters
            return false; 
        return prod.requestData();
    }
    
    public boolean setProducerMode(Customer prod, int mode, long timeout) {
        if (prod == null) // invalid parameters
            return false; 
        return prod.setProducerMode(mode, timeout);
    }
    
    
    public boolean subscribeToProducers(Customer cons, int ctype, int user,
            int custid) {
        if (cons == null) // invalid parameters
            return false;
        // FIXME: look through all and choose the ones with the chosen userid,
        // customer type and custid
        if (custid == Integer.MAX_VALUE) {
            Customer[] prod = getProducers();
            for (int i = 0; i < prod.length; ++i) {
                if (prod[i] == cons)
                    continue;
                subscribeToProducer(cons, prod[i]);
            }
            return true;
        } else {
            Customer c = (Customer) customersById.get(new Integer(custid));
            subscribeToProducer(cons, c);
            return true;
        }
    }
    
    public void subscribeToProducer(Customer cons, Customer prod) {
    	
    	// subscribe to producer's raw data
        if (cons == null || prod == null) // invalid parameters
            return;
        
        //System.out.println("BB: subscribetoprod, cons: " + cons + " to " + prod);
        //if (logger != null) logger.logMessage("BB: subscribetoprod, cons: " + cons + " to " + prod);
        
        if (this.producerSubscribers.get(prod) == null) {
            Vector temp = new Vector();
            temp.addElement(cons);
            this.producerSubscribers.put(prod, temp);
        } else {
            Vector temp = (Vector) this.producerSubscribers.get(prod);
            if (!temp.contains(cons))
                temp.addElement(cons);
        }
    }
    public void subscribeToEvent(Customer cons, Customer prod, String eventName) {
    	
    	// producer-event-listeners
		if (!this.eventList.contains(eventName)) {
			this.eventList.addElement(eventName);
		}

		if (logger != null) logger.logMessage("BB: subsc to EV: " + cons + ", " + prod + ", " + eventName);
		
		 // Producer -> Hashtable of Events -> Vector of Listeners/Customers
		if (!this.producerEventSubscribers.containsKey(prod)){
			this.producerEventSubscribers.put(prod, new Hashtable());
		}
		Hashtable producerEvents = (Hashtable)this.producerEventSubscribers.get(prod);
		if (!producerEvents.containsKey(eventName)){
			producerEvents.put(eventName, new Vector());
		}
		Vector evListeners = (Vector)producerEvents.get(eventName);
		evListeners.addElement(cons);
		//
		
		if (this.producerEventSubscribers.containsKey(prod)){
    		Hashtable prodEvents = (Hashtable)this.producerEventSubscribers.get(prod);
    		if (prodEvents.containsKey(eventName)){
    			Vector evListeners2 = (Vector)prodEvents.get(eventName);

    			for (int j=0; j<evListeners2.size(); j++){
    				Customer c = (Customer)evListeners2.elementAt(j);
    			}
    		}
		}
		
		
		
    }
    
    public void registerEvent(String eventName) {
		if (!this.eventSubscribers.contains(eventName)) {
			this.eventSubscribers.put(eventName, new Vector());
		}
	}
	public void unregisterEvent(String eventName) {
		this.eventSubscribers.remove(eventName);
	}
	
	
	
    public boolean unsubscribeFromProducers(Customer cons, int ctype, int user,
            int custid) {
        if (cons == null) // invalid parameters
            return false;
        
        Customer[] prod = getProducers();
        if (prod == null)
            return false;
        
        boolean unsubbed = false;
        for (int i = 0; i < prod.length; ++i){
            if (BlackboardUtil.matches(ctype, user, custid, prod[i].getInfo())){
                unsubscribeFromProducer(cons, prod[i]);
                unsubbed = true;
            }
        }
        return unsubbed;
    }
    
    public void subscribeConsumer(Customer cons) {
        if (cons == null) // invalid parameters
            return;
        if (!this.subscribers.contains(cons)) {
            if (BlackboardFactory.DEBUG)
                logger.logMessage("subbed a new consumer.");
            this.subscribers.addElement(cons);
            cons.updateProducerList(this, this.producers);
        }
    }
    
    public void unsubscribeConsumer(Customer cons) {
        if (cons == null) // invalid parameters
            return;
        this.subscribers.removeElement(cons);
    }
    
    public void unsubscribeFromProducer(Customer cons, Customer prod) {
        if (cons == null || prod == null) // invalid parameters
            return;
        if (this.producerSubscribers.get(prod) == null)
            return;
        Vector temp = (Vector) this.producerSubscribers.get(prod);
        temp.removeElement(cons);
    }

	public Vector getEventList() {
		return this.eventList;
	}
	
	public Vector getProducerEventList(Customer prod){
		//System.out.println("getProducerEventList: " + prod + "," + this.producerEventTable.containsKey(prod));
		if (this.producerEventTable.containsKey(prod)){
			return (Vector)this.producerEventTable.get(prod);
		} else {
			return null;
		}
	}
	/*
	public void registerEvent(Event e) {
		if (!eventList.contains(e)) {
			this.eventList.addElement(e);
		}
	}
	public void unregisterEvent(Event e) {
		this.eventList.removeElement(e);
	}*/


	

	
	/*
	public void subscribeEventToProducer(Event event, Customer prod) {
		System.out.println("SubscribeEventToProd: " + event + ", " + prod);
		if (!this.eventList.contains(event)) {
			this.eventList.addElement(event);
		}
		if (producerEventTable.containsKey(prod)) {
			Vector events = (Vector) producerEventTable.get(prod);
			events.addElement(event);
		} else {
			Vector events = new Vector();
			events.addElement(event);
			producerEventTable.put(prod, events);
		}
	}

	public void unsubscribeEventFromProducer(Event event, Customer prod) {
		if (producerEventTable.containsKey(prod)) {
			Vector events = (Vector) producerEventTable.get(prod);
			events.removeElement(event);
		}
	}*/
}
