package hiit.blackboard;

import hiit.blackboard.protocol.ComponentInfo;

public class CustomerInfo implements ComponentInfo{
    private static int globid = 1;
    
    private int id;
    
    private int type;
    
    private String name;
    
    private int user;
    
    private int mode;
    
    private long timeout;
    /**
     * If true, this is a BeTelGeuse-internal part, and should NOT be transmitted.
     */
    private boolean internal;
    
    private boolean alive;
    
    private String address;
    
    private String[] parameterNames;
    
    private int[] parameterTypes;
    
    public CustomerInfo(String name, int type, int user, String address,
            int mode, long timeout, String[] parameterNames,
            int[] parameterTypes, boolean internal) {
        this.id = CustomerInfo.globid;
        globid++;
        this.setName(name);
        this.setType(type);
        this.setUser(user);
        this.setAddress(address);
        this.setMode(mode);
        this.setTimeout(timeout);
        this.setParameterNames(parameterNames);
        this.setParameterTypes(parameterTypes);
        this.setInternal(internal); // TODO: Remove or use for something
        alive = true;
    }
    
    public int getId() {
        return this.id;
    }
    
    public void setParameterNames(String[] parameterNames) {
        this.parameterNames = parameterNames;
    }
    
    /**
     * Gets the names of available parameters.
     * 
     * @return String[] A String array that contains the names of the data parameters
     *         that the Customer provides.
     */
    public String[] getParameterNames() {
        return parameterNames;
    }
    
    public void setParameterTypes(int[] parameterTypes) {
        this.parameterTypes = parameterTypes;
    }
    
    /**
     * 
     * Works in the same way as {@link hiit.reader.Reader#getParameterTypes()}.
     * Returns the types of parameters in getData. The types are in the order of getParameterNames. 
     * The types are defined in {@link hiit.mapper.Properties}.
     * @return the parameter types, as described in Properties. See {@link hiit.mapper.Properties}. 
     */
    public int[] getParameterTypes() {
        return parameterTypes;
    }
    
    public void setType(int type) {
        this.type = type;
    }
    
    public int getType() {
        return type;
    }
    
    public void setName(String name) {
        this.name = name;
    }
    
    public String getName() {
        return name;
    }
    
    public void setUser(int user) {
        this.user = user;
    }
    
    public int getUser() {
        return user;
    }
    
    public void setMode(int mode) {
        this.mode = mode;
    }
    
    public int getMode() {
        return mode;
    }
    
    public void setTimeout(long timeout) {
        this.timeout = timeout;
    }
    
    public long getTimeout() {
        return timeout;
    }
    
    /**
     * Sets the internal Customer status of this Customer.
     * @param internal if true, this component's data is not to be transmitted.
     */
    
    public void setInternal(boolean internal) {
        this.internal = internal;
    }
    
    public boolean isInternal() {
        return internal;
    }
    
    public void setAlive(boolean alive) {
        this.alive = alive;
    }
    
    public boolean isAlive() {
        return alive;
    }
    
    public void setAddress(String address) {
        this.address = address;
    }
    
    public String getAddress() {
        return address;
    }
    
    public String toString() {
        return "type=" + type + " user=" + user + " id=" + id + " name=" + name;
        /*mode;timeout;internal;alive;address;parameterNames;parameterTypes;*/
    }

    public int getComponentId() {
        return getId();
    }

    public int getComponentType() {
        return getType();
    }

    public int getUserId() {
        return getUser();
    }
}
