package hiit.blackboard;

import hiit.blackboard.event.EventHandler;
import hiit.blackboard.protocol.ComponentInfo;
import hiit.blackboard.protocol.Protocol;
import hiit.connection.Connection;
import hiit.logger.Logger;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

public abstract class CustomerSkeleton extends Thread implements Customer  {
    public static final String classId = "[CS]";
    
    protected String name = "SocketCustomer";
    
    protected CustomerInfo info = null;
    
    protected/* HashMap<Customer, Object> */Hashtable sentData = new Hashtable();
    
    protected Connection connection;
    
    protected InputStream in;
    
    protected OutputStream out;
    
    protected SenderThread s;
    
    protected Logger logger;
    
    protected boolean running = true;
    
    protected boolean channelFree = false;
    
    protected EventHandler eventHandler;
    
    public CustomerSkeleton(){
        
    }
    
    public CustomerSkeleton(Logger logger, int ctype, int user, Connection s,
            InputStream is, OutputStream os) {
        this.logger = logger;
        // FIXME: This is not good! We should have a way to say that there are
        // no "parameters"
        // Since some socket clients just connect and exchange data parsed on a
        // lower level.
        // The other choice is to include parameters definitions in the
        // blackboard protocol.
        String tname = (String) hiit.mapper.Properties.getNameForCustomerType(ctype);
        if (tname == null)
            tname = name;
        this.info = new CustomerInfo(name, ctype, user, "nowhere", 0, 10000,
                new String[] {}, new int[] {}, true);
        this.connection = s;
        this.in = is;
        this.out = os;
    }
    
    class SenderThread extends Thread {
        public synchronized void run() {
            while (true) {
                try {
                    wait(1000);
                } catch (InterruptedException e) {
                    // Intentionally empty
                }
                if (channelFree) {
                    channelFree = false;
                    lookForAndReturnData();
                    channelFree = true;
                }
            }
        }
        
        private void lookForAndReturnData() {
            Enumeration c = sentData.keys();
            while (c.hasMoreElements()) {
                Customer z = (Customer) c.nextElement();
                if (z == null)
                    continue;
                CustomerData data = (CustomerData) sentData.get(z);
                ByteArrayOutputStream bos = new ByteArrayOutputStream();
                CustomerInfo ci = z.getInfo();
                byte[] datab = (byte[]) data.getData()[0].elementAt(0);
                Protocol.writeReturnData(bos, ci.getType(), ci.getUser(), ci
                        .getId(), datab);
                try {
                    if (BlackboardFactory.DEBUG && logger != null)
                        logger.logMessage(classId + " Sending data from " + z
                                + " to owner.");
                    out.write(bos.toByteArray());
                    out.flush();
                } catch (IOException e) {
                    if (logger != null)
                        logger.logMessage(classId + " Unable to send data back to our owner!");
                }
            }
            // clear the data.
            sentData = new Hashtable();
        }
    }
    
    /**
     * Read a subscription request to another client, subscribe to it,
     * and reply with the status of subscription.
     * @param command
     */
    protected void readSubRequest(int command){
     // Subscription request
        int[] subParams = Protocol.readGeneric(in);
        for (int i = 0; i < 3; ++i) {
            if (subParams[i] == -1)
                subParams[i] = Integer.MAX_VALUE;
        }
        if (BlackboardFactory.DEBUG && logger != null)
            logger.logMessage(classId + " Got a subscribe message for type="
                    + subParams[0] + " user=" + subParams[1]
                    + " id=" + subParams[2]);
        BlackboardFactory.getBlackboard(logger).subscribeToProducers(
                this, subParams[0], subParams[1], subParams[2]);
        BlackboardFactory.getBlackboard(logger).subscribeConsumer(this);
        boolean sub = true;
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        if (sub) {
            Protocol.writeAckOrError(bos, command, 0, "");
        } else {
            Protocol.writeAckOrError(bos, command, 1,
                    "Unable to subscribe with " + subParams[0]
                            + ", " + subParams[1] + ", "
                            + subParams[2]);
        }
        while (!channelFree) {
            try {
                wait(100);
            } catch (InterruptedException e) {
                // Intentionally empty
            }
        }
        sendReply(bos);
    }
    
    protected void readUnSubRequest(int command){
        // Subscription request
           int[] subParams = Protocol.readGeneric(in);
           for (int i = 0; i < 3; ++i) {
               if (subParams[i] == -1)
                   subParams[i] = Integer.MAX_VALUE;
           }
           if (BlackboardFactory.DEBUG && logger != null)
               logger.logMessage(classId + " Got an unsubscribe message for type="
                       + subParams[0] + " user=" + subParams[1]
                       + " id=" + subParams[2]);
           BlackboardFactory.getBlackboard(logger).unsubscribeFromProducers(this, subParams[0], subParams[1], subParams[2]);
           BlackboardFactory.getBlackboard(logger).subscribeConsumer(this);
           boolean sub = true;
           ByteArrayOutputStream bos = new ByteArrayOutputStream();
           if (sub) {
               Protocol.writeAckOrError(bos, command, 0, "");
           } else {
               Protocol.writeAckOrError(bos, command, 1,
                       "Unable to subscribe with " + subParams[0]
                               + ", " + subParams[1] + ", " + subParams[2]);
           }
           while (!channelFree) {
               try {
                   wait(100);
               } catch (InterruptedException e) {
                   // Intentionally empty
               }
           }
           sendReply(bos);
       }
    
    /**
     * Read a component list request and reply with the list of components to our client.
     */
    protected void readListComponents(){
        int[] listParams = Protocol.readListComponents(in);
        Customer[] producers = BlackboardFactory.getBlackboard(logger).getProducers();
        if (producers == null){ // error
            return;
        }
        Vector infos = new Vector();
        for (int i = 0; i < producers.length; i++){
            ComponentInfo info = producers[i].getInfo();
            if (BlackboardUtil.matches(listParams[0], listParams[1], listParams[2], info)){
                infos.addElement(info);
            }
        }
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        Protocol.writeComponentList(bos, infos);
        sendReply(bos);
    }
    
    /**
     * Read a data message from our client and pass it on to the Blackboard.
     */
    protected void readPushData(){
     // Pushdata.
        Object[] stuff = Protocol.readPushData(in);
        int compId = ((Integer) stuff[0]).intValue();
        if (compId == -1)
            compId = Integer.MAX_VALUE;
        
        byte[] data = (byte[]) stuff[1];
        if (BlackboardFactory.DEBUG && logger != null)
            logger.logMessage(classId + " received data for Customer: "
                    + compId + ": " + new String(data));
        Vector v = new Vector();
        v.addElement(data);
        CustomerData d = new CustomerData(new Vector[] { v }, new Date().getTime(), info);
        BlackboardFactory.getBlackboard(logger).pushData(this, d, null);
    }
    
    /**
     * Send the content of <code>bos</code> to our client.
     * @param bos a ByteArrayOutputStream containing a Blackboard Protocol message to be sent.
     */
    
    protected void sendReply(ByteArrayOutputStream bos){
        while (!channelFree){
            // wait
        }
        channelFree = false;
        try{
            out.write(bos.toByteArray());
            out.flush();
        }catch (IOException e){
            if (logger != null)
                logger.logMessage(classId + " Failed to send a reply, error: " + e);
        }
        channelFree = true;
    }
    
    public synchronized void run() {
        running = true;
        s = new SenderThread();
        s.start();
        while (running) {
            channelFree = true;
            int command = readCommand();
            if (command == -1)
                return;
            switch (command) {
            case 's': // subscribe
                readSubRequest(command);
                break;
            case 'u': // unsubscribe
                readUnSubRequest(command);
                break;
            case 'p': // data from client
                readPushData();
                break;
            case 'l': // client requested list of components
                readListComponents();
                break;
            case 'r':
                // Return - never sent by client
            case 't':
                // Subscribed - Never sent by client.
            case 'e':
                // Error message from client? Not possible...
            case 'c':
                // New create? Not possible; this should be sent to the BB instead.
            default:
                if (logger != null)
                    logger.logMessage(classId + " Received incorrect command byte: " + (char)command + " (" + (int)command + ")");
            }
        }
    }
    
    protected int readCommand(){
        int command = -1;
        try {
            command = in.read();
        } catch (IOException e) {
            if (logger != null)
                logger.logMessage(classId + " Failed reading command from InputStream. " + e);
        }
        if (command == -1) {
            if (logger != null)
                logger.logMessage(classId + " Received end of file while reading command. " +
                "Connection was closed. Quitting Customer.");
            running = false;
        }
        return command;
    }
    
    protected void stopRunning() {
        running = false;
        connection.close();
    }
    
    public String toString() {
    	return this.info != null? this.info.toString() : this.toString();
    }
    
    public void sendData(Customer sender, byte[] data) {
        if (sender == null || data == null) // invalid parameters
            return;
        sentData.put(sender, data);
    }
    
    protected int getCustomerId() {
        return info.getId();
    }
    
    protected int getCustomerType() {
        return info.getType();
    }
    
    protected int getUserId() {
        return info.getUser();
    }
    
    public CustomerInfo getInfo() {
        return this.info;
    }
    
    public boolean setProducerMode(int mode, long timeout) {
        this.info.setMode(mode);
        this.info.setTimeout(timeout);
        return true;
    }
    
    public void newData(Customer prod, CustomerData pd) {
        if (prod== null || pd == null) // invalid parameters
            return;
        // FIXME: Overwrite not considered here
        this.sentData.put(prod, pd);
    }
    
    public boolean requestData() {
        // FIXME: read data? How? Protocol would require a request-data message. return readNewData();
        return false;
    }
    
    public void updateCustomerInfo(Blackboard bb, Customer c, CustomerInfo pi) {
        // FIXME: What to do: one of our subscriptions has changed. 
        
    }
    
    public void updateProducerList(Blackboard bb, Vector customers) {
        if (bb == null || customers == null) // invalid parameters
            return;
        // FIXME: In the case of a universal subscription, replace our customer vector with this one.
        for (int i = 0; i < customers.size(); ++i) {
            if (customers.elementAt(i) == this)
                continue;
            // TODO: Choose who to subscribe to when new Customers are added that we are interested in.
            // FIXME: for now: subbing to everyone!!!
            //if ( customers[i] something...
            BlackboardFactory.getBlackboard(logger).subscribeToProducer(this,
                    (Customer) customers.elementAt(i));
        }
    }
    

    public void markParameters(){
    	
    }
}
