package hiit.blackboard;

import hiit.blackboard.CustomerSkeleton.SenderThread;
import hiit.blackboard.event.BasicEvent;
import hiit.blackboard.event.EventHandler;
import hiit.blackboard.event.GSMEvent;
import hiit.blackboard.protocol.Protocol;
import hiit.connection.Connection;
import hiit.logger.Logger;
import hiit.blackboard.event.EventData;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Date;
import java.util.Hashtable;
import java.util.Vector;

public class GSMCustomer extends CustomerSkeleton{
    
    protected String name = "GSMCustomer";
    
    public static final String classId = "[GC]";
    
    public static final int standoffTime = 120000; 
    
    protected TimerThread t = null;
      
    protected EventHandler eventHandler;
       
    private class TimerThread extends Thread{
        private long lastTime = 0;
        private GSMCustomer parent = null;
        
        public TimerThread(GSMCustomer p){
            this.parent = p;
        }
        
        
        public synchronized void run(){
            reset();
            while (running){
                try {
                    this.wait(standoffTime);
                } catch (InterruptedException e) {
                    // Intentionally empty
                }
                long now = System.currentTimeMillis();
                if (lastTime + standoffTime < now){ // not received anything for 2 minutes!
                    sendZeroData();
                    lastTime = now;
                }
            }
        }
        public void reset(){
            lastTime = System.currentTimeMillis();
        }
        
        private void sendZeroData(){
            Object[] gsmdata = new Object[11];
            gsmdata[0] = new Integer(0);
            gsmdata[1] = new Integer(0); // Default value
            gsmdata[2] = new Integer(0); // Default value
            gsmdata[3] = new Integer(0); // Default value
            gsmdata[4] = new Integer(0); // Default value
            gsmdata[5] = ""; // sigstr
            gsmdata[6] = ""; // sigbars
            gsmdata[7] = new Integer(0); // battery level default value
            gsmdata[8] = ""; // profile
            gsmdata[9] = new Integer(0);
            gsmdata[10] = new Integer(0);
            
            Vector v = new Vector();
            for (int i = 0; i < gsmdata.length; ++i) {
                v.addElement(gsmdata[i]);
            }
            CustomerData d = new CustomerData(new Vector[] { v }, new Date().getTime(), info);
            BlackboardFactory.getBlackboard(logger).pushData(parent, d, null );
        }
    };
    
    public GSMCustomer(Logger logger, int ctype, int user, Connection s,
            InputStream is, OutputStream os) {
        super(logger, ctype, user, s, is, os);
    }
    
    
    public synchronized void run() {
        running = true;
        t = new TimerThread(this);
        t.start();
        while (running) {
            //channelFree = true;
            int command = readCommand();
            if (command == -1)
                return;
            switch (command) {
            case 's': // subscribe
                // readSubRequest(command);
                /* We do not do this with GSM info (server does not want
                 * stuff back, it never subs)*/ 
                break;
            case 'u': // unsubscribe
                //readUnSubRequest(command);
                break;
            case 'p': // data from client
                readPushData();
                break;
            case 'l': // client requested list of components
                readListComponents();
                break;
            case 'r':
                // Return - never sent by client
            case 't':
                // Subscribed - Never sent by client.
            case 'e':
                // Error message from client? Not possible...
            case 'c':
                // New create? Not possible; this should be sent to the BB instead.
            default:
                if (logger != null)
                    logger.logMessage(classId + " Received incorrect command byte: " + (char)command + " (" + (int)command + ")");
            }
        }
    }
    
    /**
     * Read a data message from our client and pass it on to the Blackboard.
     */
    protected void readPushData(){
        // Pushdata.
        Object[] stuff = Protocol.readPushData(in);
        int compId = ((Integer) stuff[0]).intValue();
        if (compId == -1)
            compId = Integer.MAX_VALUE;
        
        byte[] data = (byte[]) stuff[1];
        String s = new String(data);
        int tab = -1;
        boolean gsmchanged = false;
        Object[] gsmdata = new Object[11];
        int num = 0;
        int lastpos = 0;
        while ((tab = s.indexOf('\t')) >= 0) {
            if (num == 0) { // skipped: We want the changed-flag
                            // now.
                String bool = (s.substring(lastpos, tab));
                s = s.substring(tab + 1);
                gsmchanged = bool.equals("1");
                gsmdata[num] = new Integer(gsmchanged ? 1 : 0);
                num++;
                continue;
            } else if (num == 5){ // signalstr
                gsmdata[num] = s.substring(lastpos, tab);
                s = s.substring(tab + 1);
                num++;
                continue;
            } else if (num == 6){ // signalbars
                gsmdata[num] = s.substring(lastpos, tab);
                s = s.substring(tab + 1);
                num++;
                continue;
            } else if (num == 8){
                gsmdata[num] = s.substring(lastpos, tab);
                s = s.substring(tab + 1);
                num++;
                continue;
            }
            String st = s.substring(lastpos, tab);
            if (st.equals("None")){
                gsmdata[num] = new Integer(0); // Default value
            } else {
                gsmdata[num] = new Integer(Integer.parseInt(st));    
            }
            
            s = s.substring(tab + 1);
            num++;
        }
        gsmdata[num] = new Integer(Integer.parseInt(s));
        
        if (BlackboardFactory.DEBUG && logger != null)
            logger.logMessage("Customer received data for Customer: "
                            + compId + ": " + new String(data));
        Vector v = new Vector();
        for (int i = 0; i < gsmdata.length; ++i) {
            v.addElement(gsmdata[i]);
        }
        CustomerData d = new CustomerData(new Vector[] { v }, new Date().getTime(), info);
        BlackboardFactory.getBlackboard(logger).pushData( this, d, null );
        t.reset(); // reset timer
    }
    
    public void newData(Customer prod, CustomerData pd) {
        // We do not want data from others
    }
    
    public void updateProducerList(Blackboard bb, Vector customers) {
        // subscriptions not used in this class
    }


	
	public void newData(Customer prod, CustomerData pd, EventData eventData){
		// TODO Auto-generated method stub
		
	}

}
