package hiit.blackboard.event;

import java.util.Vector;

import hiit.blackboard.Customer;
import hiit.blackboard.CustomerData;

public interface Event {
    
    public static final int GPS_DISTANCE_EVENT = 0x1001;
    public static final int CELL_ID_EVENT = 0x1002;
    
    /**
     * Returns the name of the event.
     * @return the name of the event.
     */
    
    public String getName();
	
    /**
     * Returns the identifier of the event.
     * @return the identifier.
     */
    
    public int getId();
    
	/**
	 * Updates the list of Customers available. 
	 * @param customers list of available Customers
	 */
    
	public void updateCustomers(Vector customers);
	
	/**
	 * Called when new data is available for a Customer that the Event has
	 * registered to. 
	 * @param customer the Customer in question
	 * @param data the new data object
	 */
	
	public void pushData(Customer customer, CustomerData data);
	
	/**
	 * Registeres an EventListener to the Event. 
	 * @param listener the EventListener in question.
	 */
	
	public void addListener(EventListener listener);
	
	/**
	 * Unregisteres an EventListener from the Event.
	 * @param listener the EventListener in question.
	 */
	
	public void removeListener(EventListener listener);
	
	
	public int compareTo(Object arg0);
}
