package hiit.blackboard.event;

import hiit.BetelgeuseMIDP;
import hiit.blackboard.Blackboard;
import hiit.blackboard.CustomerData;
import hiit.logger.Logger;

import java.util.Vector;

public class GPSEvent implements EventHandler {
    private Vector listeners;
    private Blackboard bb;
    private Vector gpsCustomers;
    private double minimumDistanceChange; // in kilometers
    
    public static String LATITUDE_CHANGE = "latitideChange";
    public static String LONGITUDE_CHANGE = "longitudeChange";
    
    
    private final String[] eventList = { "gpsDistanceChange", "latitudeChange", "longitudeChange", "satellitesChange", "HDOPChange", "VDOPChange", "speedChange" };
    private String[] previousValues = new String[6]; 
    private String[] currentValues = new String[6];
    
    private static final double defaultMinimumDistanceChange = 0.00; // 0.05 km = 50 meters
    private double distanceHistory = 0;
    private Logger logger = null;
    private long lastDistanceUpdateTime = -1;
    
    //private Hashtable previousData = new Hashtable();
    //private Hashtable previousDistanceData = new Hashtable();
 //   private CustomerData previousGPSData = null;
    
    private static final int[] diameters = {40000, 39993, 39975, 39945, 39902, 39847, 39780, 39701, 39610, 39507,
        39392, 39265, 39125, 38974, 38811, 38637, 38450, 38252, 38042, 37820, 37587, 37343,
        37087, 36820, 36541, 36252, 35951, 35640, 35317, 34984, 34641, 34286, 33921, 33546,
        33161, 32766, 32360, 31945, 31520, 31085, 30641, 30188, 29725, 29254, 28773, 28284,
        27786, 27279, 26765, 26242, 25711, 25172, 24626, 24072, 23511, 22943, 22367, 21785,
        21196, 20601, 19999, 19392, 18778, 18159, 17534, 16904, 16269, 15629, 14984, 14334,
        13680, 13022, 12360, 11694, 11025, 10352, 9676, 8998, 8316, 7632, 6945, 6257, 5566,
        4874, 4181, 3486, 2790, 2093, 1395, 698, 0};
    
    public GPSEvent() {
    }
    /*
    params[0] = "Latitude";
	params[1] = "Longitude";
	params[2] = "GMT";
	params[3] = "Altitude";
	params[4] = "Satellites";
	params[5] = "Speed";
	params[6] = "Ground speed";
	params[7] = "Course";
	params[8] = "PDOP";
	params[9] = "HDOP";
	params[10] = "VDOP";
	params[11] = "Protocols";
	*/
    
    public synchronized EventData[] getEvents(CustomerData pd) {
		if (pd == null)
			return null;
		
		Vector ev = new Vector();
		
		if (previousValues == null){
			try {
				String currentLatitude = (String) (pd.getData()[0]).lastElement();
				String currentLongitude = (String) (pd.getData()[1]).lastElement();
				
				String currentSatellites = (String) (pd.getData()[4]).lastElement();
				String currentHDOP = (String) (pd.getData()[9]).lastElement();
				String currentVDOP = (String) (pd.getData()[10]).lastElement();
				String currentSpeed = (String) (pd.getData()[5]).lastElement();
				
				if (currentLatitude != null &&
						currentLongitude != null &&
						currentSatellites != null &&
						currentHDOP != null &&
						currentVDOP != null){
						
					this.previousValues = new String[6];
					this.currentValues = new String[6];
					
					// latitude
					currentValues[0] = currentLatitude;
					previousValues[0] = currentLatitude;
					// longitude
					currentValues[1] = currentLongitude;
					previousValues[1] = currentLongitude;
					// satellites
					currentValues[2] = currentSatellites;
					previousValues[2] = currentSatellites;
					// HDOP
					currentValues[3] = currentHDOP;
					previousValues[3] = currentHDOP;
					// VDOP
					currentValues[4] = currentVDOP;
					previousValues[4] = currentVDOP;
					// speed
					currentValues[5] = currentSpeed;
					previousValues[5] = currentSpeed;
					
					//This is required for HDOP and SAT filters to initialize properly... otherwise the events are sent for the 2nd values, and
					//then the sat/hdop can already be "fixed" to a certain value (which can persist even for hours)
					ev.addElement(new EventData("satellitesChange", new Integer(Integer.parseInt(currentSatellites)).intValue(), null));
					ev.addElement(new EventData("HDOPChange", new Double(Double.parseDouble(currentHDOP)).doubleValue(), null));
				}
			} catch (Exception e){
				BetelgeuseMIDP.logger.logMessage("GPSEv, previousValues init: " + e);
			}
		} else {
			//this.currentValues = new String[6];
			try {
				// distance, latitude, longitude
				String currentLatitude = (String) (pd.getData()[0]).lastElement();
				String currentLongitude = (String) (pd.getData()[1]).lastElement();
				 
				if (currentLatitude != null && currentLongitude != null){
					String previousLatitude = previousValues[0];
					String previousLongitude = previousValues[1];
					
					double distance = calculateDistance(
							parseLatLong(false, currentLongitude), parseLatLong(true, currentLatitude),
							parseLatLong(false, previousLongitude), parseLatLong(true,previousLatitude)
							);
					
					this.logger.logMessage("[GPSE] Distance: " + distance);
					currentValues[0] = currentLatitude;
					currentValues[1] = currentLongitude;
					
					if (distance >= 0){
						distanceHistory += distance;
						
						if (distanceHistory >= this.defaultMinimumDistanceChange ) {
							ev.addElement(new EventData("distanceChange", distanceHistory, null));
							
							/*
							// GPS's Speed -attribute might be inaccurate, so we'll calculate it here instead
							long now = System.currentTimeMillis();
							if (lastDistanceUpdateTime != -1) {
								long elapsed = (now-lastDistanceUpdateTime)/1000;
								double metersPerSecond = distanceHistory/elapsed;
								double kmPerHour = ((metersPerSecond/1000)*60)*60;
								
								ev.addElement(new EventData("speedChange", kmPerHour, null));
							}
							lastDistanceUpdateTime = now;
							//
							*/
							
							
							
							distanceHistory = 0;
							/*
							if (currentLatitude != previousLatitude){
								ev.addElement(new EventData("latitudeChange", currentLatitude));	
							}
							if (currentLongitude != previousLongitude){
								ev.addElement(new EventData("longitudeChange", currentLongitude));
							}*/
						}
					}
				}
			} catch (Exception e){
				BetelgeuseMIDP.logger.logMessage("GPSEv, distanceChange: " + e);

				previousValues = null;
			}
			
			
			try {
				// satellites
				if ((pd.getData()[4]).lastElement() != null){
					String currentSatellites = (String) (pd.getData()[4]).lastElement();
					String previousSatellites = previousValues[2];
					if (!currentSatellites.equals(previousSatellites)){
						ev.addElement(new EventData("satellitesChange", new Integer(Integer.parseInt(currentSatellites)).intValue(), null));
						currentValues[2] = currentSatellites;
					}
				}
			} catch (Exception e){
				BetelgeuseMIDP.logger.logMessage("GPSEvent ERR satellitesChange: " + e);
			}
			
			try {
				// HDOP
				if ((pd.getData()[9]).lastElement() != null){
					String currentHDOP = (String) (pd.getData()[9]).lastElement();
					String previousHDOP = previousValues[3];
					if (!currentHDOP.equals(previousHDOP)){
						ev.addElement(new EventData("HDOPChange", new Double(Double.parseDouble(currentHDOP)).doubleValue(), null));
						currentValues[3] = currentHDOP;
					}
				}
			} catch (Exception e){
				BetelgeuseMIDP.logger.logMessage("GPSEvent ERR HDOPChange: " + e);
			}
			
			/*
			try {
				// VDOP
				String currentVDOP = (String) (pd.getData()[10]).lastElement();
				if (currentVDOP != null){
					String previousVDOP = previousValues[4];
					if (currentVDOP != previousVDOP){
						ev.addElement(new EventData("VDOPChange", new Double(Double.parseDouble(currentVDOP)).doubleValue(), null));
						previousValues[4] = currentVDOP;
					}
				}
			} catch (Exception e){
				BetelgeuseMIDP.logger.logMessage("GPSEvent ERR VDOPChange: " + e);
			}
			*/
			
			
			/*
			try {
				// speed
				String currentSpeed = (String) (pd.getData()[5]).lastElement();
				if (currentSpeed != null){
					String previousSpeed = previousValues[6];
					if (currentSpeed != previousSpeed){
						ev.addElement(new EventData("speedChange", new Double(Double.parseDouble(currentSpeed)).doubleValue(), null));
						previousValues[6] = currentSpeed;
					}
				}
			} catch (Exception e){
			}*/

			
			
		//	if (logger != null) logger.logMessage("GPSEv: ev.size(): " + ev.size() );
		}
		if (ev.size() == 0){
			return null;
		} else {
			EventData[] eventList = new EventData[ev.size()];
			for (int i=0; i<ev.size(); i++){
				eventList[i] = (EventData)ev.elementAt(i);
			}
			return eventList;
		}
    }
    
    public String[] getEventList(){
    	return eventList;
    }
  

    public int getId() {
        return Event.GPS_DISTANCE_EVENT;
    }

    public String getName() {
        return "GPS event monitor";
    }
   
    
    private static double parseLatLong(boolean latitude, String data){
        double latLong = 0;
        if (latitude){
            latLong = Double.parseDouble(data.substring(0, 2)) + (Double.parseDouble(data.substring(4, 11)) / 60);
            if (data.charAt(12) == 'S' || data.charAt(12) == 's')
                latLong = latLong * -1;
            return latLong;
        }
        else{
            latLong = Double.parseDouble(data.substring(0, 3)) + (Double.parseDouble(data.substring(5, 12)) / 60);
            if (data.charAt(13) == 'W' || data.charAt(13) == 'w')
                latLong = latLong * -1;
            return latLong;
        }
    }
    private static double calculateDistance(double longitude1, double latitude1, double longitude2, double latitude2){
        double x = longitudeDistance(longitude1, longitude2, latitude2);
        double y = latitudeDistance(latitude1, latitude2);
        double result = Math.sqrt(x * x + y * y);
        return result;
    }
    private static double longitudeDistance(double longitude1, double longitude2, double latitude){
        int latDeg = (int)latitude;
        double result;
        if(longitude1 > longitude2)
            result = longitude1-longitude2;
        else
            result = longitude2-longitude1;

        result = result * (diameters[latDeg] * 1.0 / 360);
        return result;
    }
    private static double latitudeDistance(double latitude1, double latitude2){
        double degreeLength = 111.111;
        double distance;
        if(latitude1 > latitude2)
            distance = (latitude1-latitude2) * degreeLength;
        else
            distance = (latitude2-latitude1) * degreeLength;
        return distance;
    }

	public void setLogger(Logger logger) {
		this.logger = logger;
	}

	public synchronized void markParameters() {
		System.out.println("[GPSE] Marking parameters...");
		this.logger.logMessage("[GPSE] Marking parameters...");
		this.previousValues = this.currentValues;
		this.currentValues = new String[6];
	}

}
