package hiit.blackboard.event;

import hiit.BetelgeuseMIDP;
import hiit.blackboard.Blackboard;
import hiit.blackboard.CustomerData;
import hiit.logger.Logger;

import java.util.Vector;

public class GSMEvent implements EventHandler {
    private final String[] eventList = { "cellIDChange", "areaIDChange", "batteryChange", "signalStrChange", "BTDevicesChange" };
    
    private final int AREA_ID_FROM = 10;
    private final int AREA_ID_TO = 0;
    
    private final int CELL_ID_FROM = 11;
    private final int CELL_ID_TO = 1;
    
    private final int SIGNAL_FROM = 12;
    private final int SIGNAL_TO = 2;
    
    private final int BATTERY_FROM = 14;
    private final int BATTERY_TO = 3;
    
    private final int BT_DEVICES_FROM = 7;
    private final int BT_DEVICES_TO = 4;
    
    
    private Object[] previousValues = null; 
    private Logger logger = null;
    
    public GSMEvent(Logger logger) {
    	// super();
    	this.logger = logger;
    }
    /*
       params[0] = "Activity";
        params[1] = "Discovery number";
        params[2] = "Last discovery time";
        params[3] = "Discovered devices";
        params[4] = "Transmission number";
        params[5] = "Transmission time";
        params[6] = "Transmission size";
        params[7] = "BTDevices";
        params[8] = mcc;
        params[9] = mnc;
        params[10] = areaid;
        params[11] = cellid;
        params[12] = signalstr;
        params[13] = signalbars;
        params[14] = phone_battery;
        params[15] = phone_profile;
        params[16] = GSM_CHANGE;
        params[17] = USER_INACTIVE;
        params[18] = FREE_RAM;
        
  
	*/
    
  
    public EventData[] getEvents(CustomerData pd) {

		if (pd == null)
			return null;
		
		if (this.previousValues == null){
			this.previousValues = new Object[eventList.length];
		
			try {
				// areaID
				previousValues[AREA_ID_TO] =  new Integer( Integer.parseInt( (String) (pd.getData()[AREA_ID_FROM]).lastElement() ));
				// cellID
				previousValues[CELL_ID_TO] =  new Integer( Integer.parseInt( (String) (pd.getData()[CELL_ID_FROM]).lastElement() ));
				// signal
				previousValues[SIGNAL_TO] =  new Integer( Integer.parseInt( (String) (pd.getData()[SIGNAL_FROM]).lastElement() ));
				// battery
				previousValues[BATTERY_TO] =  new Integer( Integer.parseInt( (String) (pd.getData()[BATTERY_FROM]).lastElement() ));
				// BTDevices
				previousValues[BT_DEVICES_TO] =  ""; //new String( (String) (pd.getData()[BT_DEVICES_FROM]).lastElement() );
				return null;
			} catch (Exception e){
				BetelgeuseMIDP.logger.logMessage("GSMEv, previousValues init: " + e);
			}
		} else {
			Vector ev = new Vector();
			
			
			try {
				// areaID
				String s = (String) (pd.getData()[AREA_ID_FROM]).lastElement();
				
				if (s != null){
					int currentVal = Integer.parseInt(s);
					int previousVal = ((Integer)previousValues[AREA_ID_TO]).intValue();
					if (currentVal != previousVal){
						ev.addElement(new EventData("areaIDChange", currentVal, null));
						previousValues[AREA_ID_TO] = new Integer(currentVal);
					}
				}
			} catch (Exception e){
				//BetelgeuseMIDP.logger.logMessage("GSMEv, areaIDChange: " + e);
			}
			
			
			try {
				// cellID
				String s = (String) (pd.getData()[CELL_ID_FROM]).lastElement();
				if (s != null){
					int currentVal = Integer.parseInt(s);
					int previousVal = ((Integer)previousValues[CELL_ID_TO]).intValue();
					if (currentVal != previousVal){
						// logger.logMessage(this + "," + currentCellID + ", prev " + previousCellID);
						ev.addElement(new EventData("cellIDChange", currentVal, null));
						previousValues[CELL_ID_TO] = new Integer(currentVal);
					}
				}
			} catch (Exception e){
				//BetelgeuseMIDP.logger.logMessage("GSMEv, cellIDChange: " + e);
			}
			
			
			
			
			try {
				// signal
				String s = (String) (pd.getData()[SIGNAL_FROM]).lastElement();
				
				if (s != null){
					int currentVal = Integer.parseInt(s);
					int previousVal = ((Integer)previousValues[SIGNAL_TO]).intValue();
					if (currentVal != previousVal){
						ev.addElement(new EventData("signalStrChange", currentVal, null));
						previousValues[SIGNAL_TO] = new Integer(currentVal);
					}
				}
			} catch (Exception e){
				//BetelgeuseMIDP.logger.logMessage("GSMEv, signalStrChange: " + e);
			}
			
			
			try {
				// battery
				String s = (String) (pd.getData()[BATTERY_FROM]).lastElement();
				
				if (s != null){
					int currentVal = Integer.parseInt(s);
					int previousVal = ((Integer)previousValues[BATTERY_TO]).intValue();
					if (currentVal != previousVal){
						ev.addElement(new EventData("batteryChange", currentVal, null));
						previousValues[BATTERY_TO] = new Integer(currentVal);
					}
				}
			} catch (Exception e){
				//BetelgeuseMIDP.logger.logMessage("GSMEv, batterChange: " + e);
			}
			
			
			if (BetelgeuseMIDP.emitBTDevicesChangeEvents == true) {
				
				try {
					// BTDevices
					String s = (String) (pd.getData()[BT_DEVICES_FROM]).lastElement();
					
					if (s != null){
						String currentVal = s;
						String previousVal = (String)previousValues[BT_DEVICES_TO];
						if (currentVal.equalsIgnoreCase(previousVal) == false) {
							ev.addElement(new EventData("BTDevicesChange", currentVal, null));
							previousValues[BT_DEVICES_TO] = currentVal;
							BetelgeuseMIDP.logger.logMessage("emit BTDevicesChange");
						}
					}
				} catch (Exception e){
					//BetelgeuseMIDP.logger.logMessage("GSMEv, BTDevicesChange: " + e);
				}
			}
			
			if (ev.size() == 0){
				return null;
			} else {
				EventData[] eventList = new EventData[ev.size()];
				for (int i=0; i<ev.size(); i++){
					eventList[i] = (EventData)ev.elementAt(i);
				}
				return eventList;
			}
		}
    	
		return null;
		
    }
    public String[] getEventList(){
    	return eventList;
    }
  

    public int getId() {
        return -1;
    }

    public String getName() {
        return "";
    }


	public void setLogger(Logger logger) {
		// TODO Auto-generated method stub
		
	}


	public void markParameters() {
		// TODO Auto-generated method stub
		
	}
   
    
   

}
