package hiit.blackboard.filter;

import java.util.Vector;

import hiit.BetelgeuseMIDP;
import hiit.blackboard.Blackboard;
import hiit.blackboard.BlackboardFactory;
import hiit.blackboard.Customer;
import hiit.blackboard.CustomerData;
import hiit.blackboard.CustomerInfo;
import hiit.blackboard.event.EventData;
import hiit.logger.Logger;

public class CompareFilter extends Filter {
	private int int_limit;
	private double double_limit;
	
	private int prev_int_val = 0;
	private double prev_double_val = 0;
	
	
	private int int_history = 0;
	private double double_history = 0;
	
	private boolean keepHistory;
	
	private int valueType;
	private int type;
	
	public static final int VALUE_TYPE_INT = 1;
	public static final int VALUE_TYPE_DOUBLE = 2;
	
	public static final int TYPE_GREATER_THAN = 10;
	public static final int TYPE_LESS_THAN = 11;
	
	private boolean inRange = false;
	public CompareFilter(Customer prod, Customer cons, int limit, int type, boolean keepHistory) {
	
		super(prod, cons);
		
		this.valueType = VALUE_TYPE_INT;
		this.type = type;
		setLimit(limit);
		
		this.keepHistory = keepHistory;
	}
	public CompareFilter(Customer prod, Customer cons, double limit, int type, boolean keepHistory) {
		super(prod, cons);
		
		this.valueType = VALUE_TYPE_DOUBLE;
		this.type = type;
		setLimit(limit);
		
		this.keepHistory = keepHistory;
	}
	
	public void newData(Customer prod, CustomerData pd, EventData eventData) {
		//if (logger != null) logger.logMessage("GreaterFilter: newData" + eventData );

		EventData newEventData = null;
		
		try {
			switch (eventData.getType()){
			case VALUE_TYPE_INT:
				int int_val = eventData.getIntValue();
				if (keepHistory){
					int_history += int_val;
					inRange = checkIfInRange(int_history);
					if (inRange){
						newEventData = new EventData(eventData.getName(), int_history, this);
						
						prev_int_val = int_history;
						int_history = 0;
					}
				} else {
					inRange = checkIfInRange(int_val);
					if (inRange) 
						newEventData = new EventData(eventData.getName(), int_val, this);
					
					prev_int_val = int_val;
				}
				break;
				
			case VALUE_TYPE_DOUBLE:
				double double_val = eventData.getDoubleValue(); 
				//if (logger != null) logger.logMessage("GreaterFilter: " + double_val + ", keepHis: " + keepHistory + ", his: " + double_history + ", cons: " + consumer );
				if (keepHistory == true){
					double_history += double_val;
					inRange = checkIfInRange(double_history);
					if (inRange){
						newEventData = new EventData(eventData.getName(), double_history, this);
						
						//if (logger != null) logger.logMessage("greatFil SEND: lim: " + double_limit + ", his: " + double_history);
						prev_double_val = double_history;
						double_history = 0;
						inRange = true;
					}
				} else {
					inRange = checkIfInRange(double_val);
					if (inRange)
						newEventData = new EventData(eventData.getName(), double_val, this);
					
					prev_double_val = double_val;
				}
				break;
			
			}

		} catch (Error e){
			System.out.println(this + ": newData: "+ e);
		}
		
		//if (logger != null) logger.logMessage("GreaterFil send?: " + send + ", prod: " + prod + ", cons: " + this.consumer);
		if (inRange){
			if (this.consumer != null)
				this.consumer.newData(prod, pd, newEventData);
		}
			
		
	}
	
	public String getName(){
		return "CompareF: " + (this.type == TYPE_GREATER_THAN? ">" : "<") + " " + (this.valueType == VALUE_TYPE_INT? "int: " + int_limit : "doub: " + double_limit) ;
	}
	public boolean isInRange(){
		return inRange;
	}

	public void setLimit(int limit){
		int_limit = limit;
		inRange = checkIfInRange(limit);
	}
	public void setLimit(double limit){
		double_limit = limit;
		inRange = checkIfInRange(limit);
	}
	
	private boolean checkIfInRange(int val){
		if (type == TYPE_GREATER_THAN){
			return val > int_limit;
		} else {
			return val < int_limit;
		}
	}
	private boolean checkIfInRange(double val){
		if (type == TYPE_GREATER_THAN){
			return val > double_limit;
		} else {
			return val < double_limit;
		}
	}
	/*
	private boolean checkIfInRange(Object val){
		switch (valueType){
		case VALUE_TYPE_INT:
			int int_val = ((Integer)val).intValue() ;
			if (type == TYPE_GREATER_THAN){
				return int_val > int_limit;
			} else {
				return int_val < int_limit;
			}
		case VALUE_TYPE_DOUBLE:
			double double_val = ((Double)val).doubleValue() ;
			if (type == TYPE_GREATER_THAN){
				return double_val > double_limit;
			} else {
				return double_val < double_limit;
			}
		}
		return false;
	}
	*/
	public CustomerInfo getInfo() {
		return null;
	}

	public boolean requestData() {
		return false;
	}

	public boolean setProducerMode(int mode, long timeout) {
		return false;
	}

	public void updateCustomerInfo(Blackboard bb, Customer c, CustomerInfo pi) {
	}

	public void updateProducerList(Blackboard bb, Vector producers) {
	}

}
