package hiit.blackboard.filter;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;

import hiit.BetelgeuseMIDP;
import hiit.blackboard.Blackboard;
import hiit.blackboard.Customer;
import hiit.blackboard.CustomerData;
import hiit.blackboard.CustomerInfo;
import hiit.blackboard.event.EventData;
import hiit.logger.Logger;

public class ConstraintFilter extends Filter {
	Vector constraints = new Vector();
	private CustomerData producerData = null;
	private EventData producerEventData = null;
	
	public ConstraintFilter(Customer cons, Customer producer, Logger logger) {
		super(null, cons);
		this.logger = logger;
		this.producer = producer;
	}
	
	public void addConstraint(Filter f){
		if (!constraints.contains(f))
			constraints.addElement(f);
	}
	public void newData(Customer prod, CustomerData pd, EventData evData) {
		// BetelgeuseMIDP.logger.logMessage("constraintF EV: " + this.name + ", broadcast?: " + this.timeToBroadcast(prod, pd, evData) + ", f: " + evData.getFilter());
		if (prod == this.producer){
			this.producerData = pd;
			this.producerEventData = evData;
			
			
			if (evData.getFilter() == null)
				evData.setFilter(this); // mark data as coming from ConstraintFilter
			
			// first check if constraints are ok, then wait for a little while and check again
			if (this.timeToBroadcast(prod, pd, null)){
				//this.producer.markParameters();
				waitForConstraintPropagation(prod, pd, evData);
			}
			
		}
	}
	
	public void newData(Customer prod, CustomerData pd) {
		// BetelgeuseMIDP.logger.logMessage("constraintF RAW: " + this.name + ", broadcast?: " + this.timeToBroadcast(prod, pd, null) + ", f NULL");

		if (prod == this.producer){
			this.producerData = pd;
			this.producerEventData = null;
			
			
			// first check if constraints are ok, then wait for a little while and check again
			if (this.timeToBroadcast(prod, pd, null)){
				//this.producer.markParameters();
				waitForConstraintPropagation(prod, pd, null);
			}
			
		}
	}
	

	private boolean timeToBroadcast(Customer prod, CustomerData pd, EventData evData){
		// BetelgeuseMIDP.logger.logMessage("## ConstraintF, broadcast?");
		for (Enumeration e = constraints.elements() ; e.hasMoreElements() ;) {
			Filter f = (Filter)e.nextElement();
			if (!f.isInRange()){
				return false;
			}
		}
		return true;
	}
	
	
	public CustomerInfo getInfo() {
		return null;
	}

	public boolean requestData() {
		return false;
	}

	public boolean setProducerMode(int mode, long timeout) {
		return false;
	}

	public void updateCustomerInfo(Blackboard bb, Customer c, CustomerInfo pi) {
	}

	public void updateProducerList(Blackboard bb, Vector producers) {
	}

	
	
	
	
	
	// Timer that makes sure that the events have been propagated to the constraints before we call "timeToBroadcast"
	private Timer eventPropagationWaitTimer = null;
	
	private void waitForConstraintPropagation(Customer prod, CustomerData pd, EventData ev){
		if (eventPropagationWaitTimer != null) {
			eventPropagationWaitTimer.cancel();
			eventPropagationWaitTimer = null;
		}
			
		eventPropagationWaitTimer = new Timer();
		EventPropagationWait task = new EventPropagationWait(this, prod, pd, ev);
	    eventPropagationWaitTimer.schedule(task, 500);  
	}    
    
    private void eventPropagationWaitDone(Customer prod, CustomerData pd, EventData ev){
		boolean send = ev != null? this.timeToBroadcast(prod, pd, ev) : this.timeToBroadcast(prod, pd, null);
		if (send)
			if(ev==null)
				consumer.newData(this.producer, this.producerData);
			else
				consumer.newData(this.producer, this.producerData, ev);
	}
    
    
	class EventPropagationWait extends TimerTask {
		private ConstraintFilter parent;
		private boolean completed = false;
		private Customer prod;
		private CustomerData pd;
		private EventData ev;
		
		public EventPropagationWait(ConstraintFilter parent, Customer prod, CustomerData pd, EventData ev){
			this.parent = parent;
			this.prod = prod;
			this.pd = pd;
			this.ev = ev;
		}
		public void run() {
			if (!completed){
				this.parent.eventPropagationWaitDone(this.prod, this.pd, this.ev);
				this.cancel();
				this.completed = true;
			}
		}
	}
	//
}
