package hiit.blackboard.filter;

import java.util.Vector;

import hiit.blackboard.Blackboard;
import hiit.blackboard.Customer;
import hiit.blackboard.CustomerData;
import hiit.blackboard.CustomerInfo;
import hiit.blackboard.event.EventData;

public class DynamicCompareFilter extends Filter{
	
	private double limit = 0; 
	
	private int prev_int_val = 0;
	private double prev_double_val = 0;
	
	private double multiplier = 0;
	
	
	private int int_history = 0;
	private double double_history = 0;
	
	private boolean keepHistory;
	private boolean isint = true;
	
	private int type;
	
	public static final int VALUE_TYPE_INT = 1;
	public static final int VALUE_TYPE_DOUBLE = 2;
	
	public static final int TYPE_GREATER_THAN = 10;
	public static final int TYPE_LESS_THAN = 11;
	
	private int paramIndex = -1;
	private int paramType = -1;
	private String paramName;
	
	private boolean inRange = false;
	public DynamicCompareFilter(Customer prod, Customer cons, double multiplier, int type, boolean keepHistory, String paramName) {
	
		super(prod, cons);
		
		this.type = type;
		this.multiplier = multiplier;
		
		this.keepHistory = keepHistory;
		this.paramName = paramName;
	}
	
	public void newData(Customer prod, CustomerData pd, EventData eventData) {
		//if (logger != null) logger.logMessage("GreaterFilter: newData" + eventData );

		EventData newEventData = null;

		if(paramIndex == -1){
			String[] names = pd.getParameterNames();
			for(int i=0;i<names.length;i++){
				if(names[i].equals(this.paramName)){
					this.paramIndex = i;
					this.paramType = pd.getParameterTypes()[i];
					break;
				}
			}
		}

		try {
			inRange = false;
			Vector data = pd.getData()[this.paramIndex];
			if(data != null && data.size()>0){
				if(this.paramType == 0){
					//String
					this.limit = this.multiplier * Double.parseDouble((String)data.elementAt(0));
				}else if(this.paramType == 1){
					this.limit = this.multiplier * ((Integer)data.elementAt(0)).intValue();
				}else if(this.paramType == 2){
					this.limit = this.multiplier * ((Double)data.elementAt(0)).doubleValue();
				}else if(this.paramType == 3){
					this.limit = this.multiplier * ((Float)data.elementAt(0)).floatValue();
				}
			}else{
				//nothing to be done...
				return;
			}
			
			
			switch (eventData.getType()){
			case VALUE_TYPE_INT:
				int int_val = eventData.getIntValue();
				if (keepHistory){
					int_history += int_val;
					inRange = checkIfInRange(int_history);
					if (inRange){
						newEventData = new EventData(eventData.getName(), int_history, this);
						
						prev_int_val = int_history;
						int_history = 0;
					}
				} else {
					inRange = checkIfInRange(int_val);
					if (inRange) 
						newEventData = new EventData(eventData.getName(), int_val, this);
					
					prev_int_val = int_val;
				}
				break;
				
			case VALUE_TYPE_DOUBLE:
				double double_val = eventData.getDoubleValue(); 
				//if (logger != null) logger.logMessage("GreaterFilter: " + double_val + ", keepHis: " + keepHistory + ", his: " + double_history + ", cons: " + consumer );
				if (keepHistory == true){
					double_history += double_val;
					inRange = checkIfInRange(double_history);
					if (inRange){
						newEventData = new EventData(eventData.getName(), double_history, this);
						
						//if (logger != null) logger.logMessage("greatFil SEND: lim: " + double_limit + ", his: " + double_history);
						prev_double_val = double_history;
						double_history = 0;
						inRange = true;
					}
				} else {
					inRange = checkIfInRange(double_val);
					if (inRange)
						newEventData = new EventData(eventData.getName(), double_val, this);
					
					prev_double_val = double_val;
				}
				break;
			
			}

		} catch (Error e){
			System.out.println(this + ": newData: "+ e);
		}
		
		//if (logger != null) logger.logMessage("GreaterFil send?: " + send + ", prod: " + prod + ", cons: " + this.consumer);
		if (inRange){
			if (this.consumer != null)
				this.consumer.newData(prod, pd, newEventData);
		}
			
		
	}
	
	public String getName(){
		return "DynamicCompareF: " + (this.type == TYPE_GREATER_THAN? ">" : "<") + " Current limit:" + this.limit ;
	}
	public boolean isInRange(){
		return inRange;
	}

	private boolean checkIfInRange(int val){
		if (type == TYPE_GREATER_THAN){
			return val > limit;
		} else {
			return val < limit;
		}
	}
	private boolean checkIfInRange(double val){
		System.out.println("[DCF] Value: " + val + ", limit: " + limit);
		if (type == TYPE_GREATER_THAN){
			return val > limit;
		} else {
			return val < limit;
		}
	}
	/*
	private boolean checkIfInRange(Object val){
		switch (valueType){
		case VALUE_TYPE_INT:
			int int_val = ((Integer)val).intValue() ;
			if (type == TYPE_GREATER_THAN){
				return int_val > int_limit;
			} else {
				return int_val < int_limit;
			}
		case VALUE_TYPE_DOUBLE:
			double double_val = ((Double)val).doubleValue() ;
			if (type == TYPE_GREATER_THAN){
				return double_val > double_limit;
			} else {
				return double_val < double_limit;
			}
		}
		return false;
	}
	*/
	public CustomerInfo getInfo() {
		return null;
	}

	public boolean requestData() {
		return false;
	}

	public boolean setProducerMode(int mode, long timeout) {
		return false;
	}

	public void updateCustomerInfo(Blackboard bb, Customer c, CustomerInfo pi) {
	}

	public void updateProducerList(Blackboard bb, Vector producers) {
	}

}
