package hiit.blackboard.filter;

import java.util.Vector;

import hiit.BetelgeuseMIDP;
import hiit.blackboard.Blackboard;
import hiit.blackboard.Customer;
import hiit.blackboard.CustomerData;
import hiit.blackboard.CustomerInfo;
import hiit.blackboard.event.EventData;

public class GPSVelocityFilter extends Filter{
	
	private double limit = 0; 
	
	private double multiplier = 0;
	
	
	private double double_history = 0;
	
	private int type;
	
	public static final int VALUE_TYPE_INT = 1;
	public static final int VALUE_TYPE_DOUBLE = 2;
	
	public static final int TYPE_GREATER_THAN = 10;
	public static final int TYPE_LESS_THAN = 11;
	
	private int paramIndex = -1;
	private int paramType = -1;
	private String paramName;
	
	private boolean inRange = false;
	
	long previousTime = 0;
	private String previousLatitude, previousLongitude;
	
	private Vector velocities = new Vector();
	
	private int count = 0; 
	private double prevStochasticMean = 0;
	
    private static final int[] diameters = {40000, 39993, 39975, 39945, 39902, 39847, 39780, 39701, 39610, 39507,
        39392, 39265, 39125, 38974, 38811, 38637, 38450, 38252, 38042, 37820, 37587, 37343,
        37087, 36820, 36541, 36252, 35951, 35640, 35317, 34984, 34641, 34286, 33921, 33546,
        33161, 32766, 32360, 31945, 31520, 31085, 30641, 30188, 29725, 29254, 28773, 28284,
        27786, 27279, 26765, 26242, 25711, 25172, 24626, 24072, 23511, 22943, 22367, 21785,
        21196, 20601, 19999, 19392, 18778, 18159, 17534, 16904, 16269, 15629, 14984, 14334,
        13680, 13022, 12360, 11694, 11025, 10352, 9676, 8998, 8316, 7632, 6945, 6257, 5566,
        4874, 4181, 3486, 2790, 2093, 1395, 698, 0};
    
	
	public GPSVelocityFilter(Customer prod, Customer cons, double multiplier, int type, String paramName) {
	
		super(prod, cons);
		
		this.type = type;
		this.multiplier = multiplier * 1000;
		
		this.paramName = paramName;
	}
	
	public void newData(Customer prod, CustomerData pd, EventData eventData) {
		//if (logger != null) logger.logMessage("GreaterFilter: newData" + eventData );

		EventData newEventData = null;

		if(paramIndex == -1){
			String[] names = pd.getParameterNames();
			for(int i=0;i<names.length;i++){
				if(names[i].equals(this.paramName)){
					this.paramIndex = i;
					this.paramType = pd.getParameterTypes()[i];
					break;
				}
			}
		}

		try {
			inRange = false;
			Vector data = pd.getData()[this.paramIndex];
			if(data != null && data.size()>0){
				if(this.paramType == 0){
					//String
					this.limit = this.multiplier * Double.parseDouble((String)data.elementAt(0));
				}else if(this.paramType == 1){
					this.limit = this.multiplier * ((Integer)data.elementAt(0)).intValue();
				}else if(this.paramType == 2){
					this.limit = this.multiplier * ((Double)data.elementAt(0)).doubleValue();
				}else if(this.paramType == 3){
					this.limit = this.multiplier * ((Float)data.elementAt(0)).floatValue();
				}
			}else{
				//nothing to be done...
				return;
			}

			double currentValue = eventData.getDoubleValue() * 1000; 
			long currentTime = pd.getTimestamp();
			
			String currentLatitude = (String) (pd.getData()[0]).lastElement();
			String currentLongitude = (String) (pd.getData()[1]).lastElement();
			
			if(this.previousLatitude == null && this.previousLongitude == null){
				//1st value
				
				//Since we don't really have any idea about the distance for the first value, send the values
				if (this.consumer != null && this.previousTime == 0){
					newEventData = new EventData(eventData.getName(), currentValue, this);
					this.producer.markParameters();
					this.consumer.newData(prod, pd, newEventData);
				}
				
				//Update the previous variables...
				this.previousTime = currentTime;
				this.previousLatitude = currentLatitude;
				this.previousLongitude = currentLongitude;
				return;
			}
			
			//Calculate the speed of the user...
			if (currentLatitude != null && currentLongitude != null){
				double distance = calculateDistance(
						parseLatLong(false, currentLongitude), parseLatLong(true, currentLatitude),
						parseLatLong(false, previousLongitude), parseLatLong(true,previousLatitude)
				);
				//Speed of the user since last measurement...
				double speed = (distance*1000) / ((currentTime - this.previousTime)/1000);
				
				count++;
				
				double alpha = 0;
				if(count > 1000)
					alpha = 0.001;
				else
					alpha = 1.0/count;
				
				double stochasticMean = prevStochasticMean + alpha *(speed - prevStochasticMean);
				prevStochasticMean = stochasticMean;
				
				System.out.println("[GPSVF] stochastic mean: " + stochasticMean + " d*HDOP: " + this.limit + " limit: " + (this.limit + Math.floor(stochasticMean)*60));
				
				this.limit = this.limit + (2 * Math.floor(stochasticMean)*60);		
				
				/*
				this.velocities.addElement(new Double(speed));
				
				//FIXME HACK! Keeps last 5 values and calculates an average speed using them...
				if(this.velocities.size() > 5){
					this.velocities.removeElementAt(0);
				}
				double averageSpeed = 0;
				for(int i=0;i<this.velocities.size();i++){
					averageSpeed += ((Double)this.velocities.elementAt(i)).doubleValue();
				}
				
				averageSpeed = averageSpeed / this.velocities.size();
				
				//TODO multiply the limit...
				*/
				
				this.previousTime = currentTime;
				this.previousLatitude = currentLatitude;
				this.previousLongitude = currentLongitude;
				
				inRange = checkIfInRange(currentValue);
				if (inRange)
					newEventData = new EventData(eventData.getName(), currentValue, this);
			}

		} catch (Error e){
			System.out.println(this + ": newData: "+ e);
		}
		
		//if (logger != null) logger.logMessage("GreaterFil send?: " + send + ", prod: " + prod + ", cons: " + this.consumer);
		if (inRange){
			this.producer.markParameters();
			if (this.consumer != null)
				this.consumer.newData(prod, pd, newEventData);
		}
			
		
	}
	
	public String getName(){
		return "DynamicCompareF: " + (this.type == TYPE_GREATER_THAN? ">" : "<") + " Current limit:" + this.limit ;
	}
	public boolean isInRange(){
		return inRange;
	}

	private boolean checkIfInRange(int val){
		if (type == TYPE_GREATER_THAN){
			return val > limit;
		} else {
			return val < limit;
		}
	}
	private boolean checkIfInRange(double val){
		System.out.println("[GPSVF] Value: " + val + ", limit: " + limit);
		if (type == TYPE_GREATER_THAN){
			return val > limit;
		} else {
			return val < limit;
		}
	}
	/*
	private boolean checkIfInRange(Object val){
		switch (valueType){
		case VALUE_TYPE_INT:
			int int_val = ((Integer)val).intValue() ;
			if (type == TYPE_GREATER_THAN){
				return int_val > int_limit;
			} else {
				return int_val < int_limit;
			}
		case VALUE_TYPE_DOUBLE:
			double double_val = ((Double)val).doubleValue() ;
			if (type == TYPE_GREATER_THAN){
				return double_val > double_limit;
			} else {
				return double_val < double_limit;
			}
		}
		return false;
	}
	*/
	public CustomerInfo getInfo() {
		return null;
	}

	public boolean requestData() {
		return false;
	}

	public boolean setProducerMode(int mode, long timeout) {
		return false;
	}

	public void updateCustomerInfo(Blackboard bb, Customer c, CustomerInfo pi) {
	}

	public void updateProducerList(Blackboard bb, Vector producers) {
	}
	
    private static double parseLatLong(boolean latitude, String data){
        double latLong = 0;
        if (latitude){
            latLong = Double.parseDouble(data.substring(0, 2)) + (Double.parseDouble(data.substring(4, 11)) / 60);
            if (data.charAt(12) == 'S' || data.charAt(12) == 's')
                latLong = latLong * -1;
            return latLong;
        }
        else{
            latLong = Double.parseDouble(data.substring(0, 3)) + (Double.parseDouble(data.substring(5, 12)) / 60);
            if (data.charAt(13) == 'W' || data.charAt(13) == 'w')
                latLong = latLong * -1;
            return latLong;
        }
    }
	
    private static double calculateDistance(double longitude1, double latitude1, double longitude2, double latitude2){
        double x = longitudeDistance(longitude1, longitude2, latitude2);
        double y = latitudeDistance(latitude1, latitude2);
        double result = Math.sqrt(x * x + y * y);
        return result;
    }
    private static double longitudeDistance(double longitude1, double longitude2, double latitude){
        int latDeg = (int)latitude;
        double result;
        if(longitude1 > longitude2)
            result = longitude1-longitude2;
        else
            result = longitude2-longitude1;

        result = result * (diameters[latDeg] * 1.0 / 360);
        return result;
    }
    private static double latitudeDistance(double latitude1, double latitude2){
        double degreeLength = 111.111;
        double distance;
        if(latitude1 > latitude2)
            distance = (latitude1-latitude2) * degreeLength;
        else
            distance = (latitude2-latitude1) * degreeLength;
        return distance;
    }

}
