package hiit.blackboard.httpd;

import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;

public class HttpRequest {
    private String operation;
    private Path path;
    private String version;
    private Hashtable queryParameters;
    private Hashtable headers;
    private Vector postedData;

    public HttpRequest() {
    	this.operation = "";
    	this.path = new Path("/");
    	this.version = "HTTP/1.1";
    	this.queryParameters = new Hashtable();
    	this.headers = new Hashtable();
    	this.postedData = new Vector();
    }

    public HttpRequest(String message) throws HttpFormatException {
    	Vector lines = new Vector();
        char ch;

        // Split entire message at line breaks
        for (int i = 0; i < message.length(); i++) {
            StringBuffer linebuffer = new StringBuffer();
            while (i < message.length() && (ch = message.charAt(i)) != '\n') {
                linebuffer.append(ch);
                i++;
            }
            String line = linebuffer.toString().trim();
            lines.addElement(line.toString().trim());
        }
        
        HttpRequest r = new HttpRequest(lines);
        this.headers = r.headers;
        this.operation = r.operation;
        this.path = r.path;
        this.postedData = r.postedData;
        this.queryParameters = r.queryParameters;
        this.version = r.version;
    }
    
    public HttpRequest(Vector lines) throws HttpFormatException {
        // TODO: Don't read infinitely long messages?

        this.operation = "";
        this.path = new Path("/");
        this.version = "";
        this.queryParameters = new Hashtable();
        this.headers = new Hashtable();
        this.postedData = new Vector();

        // 1) Split the first line at blankspace
        String[] firstline = splitAt(' ', (String) lines.elementAt(0));

        // 2) Get the operation- and version-fields
        if (firstline.length == 3) {
        	this.operation = firstline[0];
            this.version = firstline[2];
        } else {
        	throw new HttpFormatException("Badly formatted request line");
        }

        // 3) Separate the path from the query parameters
        String[] pathAndParams = splitAt('?', firstline[1]);
        this.path = new Path(pathAndParams[0]);
        
        // 4) If there are query parameters, then separate the parameters
        //  at '&', and divide them into key-value pairs at '='
        if (pathAndParams.length > 1) {
            String[] params = splitAt('&', pathAndParams[1]);
            for (int i = 0; i < params.length; i++) {
                String[] keyAndValue = splitAt('=', params[i]);
                if (keyAndValue.length > 1)
                    queryParameters.put(keyAndValue[0], keyAndValue[1]);
                else if (keyAndValue.length == 1)
                    queryParameters.put(keyAndValue[0], "");
            }
        }

        // 5) The next lines are headers. Split them at ':' into key-value pairs.
        int i;
        for (i = 1; i < lines.size(); i++) {
        	String line = (String) lines.elementAt(i); 
        	if (line.equals("")) {
        		// stop if an empty line is reached
        		// an empty line serves as a separator before POSTed data
        		break;
        	}
            String[] headerAndValue = splitAt(':', line);
            if (headerAndValue.length > 1)
                headers.put(headerAndValue[0], headerAndValue[1]);
            else if (headerAndValue.length == 1)
                headers.put(headerAndValue[0], "");
        }
        
        // 6) If there are still lines left (after an empty line)
        // they are regarded as POSTed data.
        for (i = i + 1; i < lines.size(); i++) {
        		this.postedData.addElement(lines.elementAt(i));
        }
    }

    public static String[] splitAt(char delim, String text) {
        Vector tokens = new Vector();
        char ch;

        for (int i = 0; i < text.length(); i++) {
            StringBuffer tokenbuffer = new StringBuffer();
            while (i < text.length() && (ch = text.charAt(i)) != delim) {
                tokenbuffer.append(ch);
                i++;
            }
            String token = tokenbuffer.toString().trim();
            // System.out.println(line);
            tokens.addElement(token.trim());
        }

        String[] arrayOfTokens = new String[tokens.size()];
        for (int i = 0; i < tokens.size(); i++)
            arrayOfTokens[i] = (String) tokens.elementAt(i);
        return arrayOfTokens;
    }

    /*
     * public static String getPathString(String httpRequest) { StringBuffer path = new
     * StringBuffer(); char ch; // find the end of the word "GET" int i = httpRequest.indexOf("GET") +
     * 3; // move forward until path begins while (httpRequest.charAt(i) == ' ') { i++; } // move
     * forward until path ends, reading the path one char at a time while ((ch =
     * httpRequest.charAt(i)) != ' ') { path.append(ch); i++; } return path.toString(); }
     */

    public static HttpRequest parseMessage(String message) {
        HttpRequest request = null;
        try {
            request = new HttpRequest(message);
        } catch (HttpFormatException e) {
            e.printStackTrace();
        }
        return request;
    }

    public String getOperation() {
        return operation;
    }

    public Path getPath() {
        return path;
    }

    public Hashtable getQueryParameters() {
        return queryParameters;
    }

    public Hashtable getHeaders() {
        return headers;
    }

    /*
     * Returns the HTTP-request as a String. It should look the same as the one originally parsed,
     * EXCEPT that some fields may be in another order. (A Hashtable is not ordered.)
     * 
     * @see java.lang.Object#toString()
     */
    public String toString() {
        // First line
        String message = "";
        message = operation + " " + path;
        if (!queryParameters.isEmpty()) {
            Enumeration keys = queryParameters.keys();
            // The first parameter is preceded by a "?".
            String key = (String) keys.nextElement();
            message += "?" + key + "=" + queryParameters.get(key);
            // Possible other parameter are preceded by an "&".
            while (keys.hasMoreElements()) {
                key = (String) keys.nextElement();
                message += "&" + key + "=" + queryParameters.get(key);
            }
        }
        message += " " + version;

        // Header lines
        if (!headers.isEmpty()) {
            Enumeration keys = headers.keys();
            while (keys.hasMoreElements()) {
                String thisKey = (String) keys.nextElement();
                message += "\r\n" + thisKey + ": " + headers.get(thisKey);
            }
        }

        // Finally, add a line break and an empty line
        message += "\r\n\r\n";

        // If this is a POST message, there could be posted data also.
        if (postedData != null) {
        	for (int i = 0; i < postedData.size(); i++) {
        		message += postedData.elementAt(i);
        	}
            message += "\r\n\r\n";
        }

        return message;
    }

    /*
     * A main method for testing that the parsing ant printing works correctly.
     */
    public static void main(String[] args) throws Exception {

        System.out.println("Parsing a lot of stuff...");
        HttpRequest r = null;
        for (int i = 0; i < 1; i++) {
            r = new HttpRequest(
                    "GET /index.php?namn=boris&smurf=ja%20och%20nej&a=24 HTTP/1.1"
                            + "\n"
                            + "Accept-Language: en"
                            + "\n"
                            + "Accept-Encoding: gzip, deflate"
                            + "\n"
                            + "User-Agent: Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_4_11; en) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13"
                            + "\n"
                            + "Accept: text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5"
                            + "\n" + "Connection: keep-alive" + "\n"
                            + "Host: 127.0.0.1:10000" + "\n\n"
                            + "fooBar");
        }
        System.out.println("Here we go:\n");
        System.out.println(r.toString());
    }
}

class HttpFormatException extends Exception {
    HttpFormatException() {
        super();
    }
    
    HttpFormatException(String s) {
    	super(s);
    }
}
