package hiit.blackboard.httpd;

import hiit.blackboard.Blackboard;
import hiit.blackboard.BlackboardFactory;
import hiit.blackboard.Customer;
import hiit.blackboard.CustomerData;
import hiit.blackboard.CustomerInfo;
import hiit.logger.Logger;
import hiit.connection.Connection;
import hiit.connection.MIDPServer;
import hiit.connection.ServerConnection;
import hiit.mapper.Properties;
import hiit.blackboard.event.EventData;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintStream;

//import javax.microedition.io.Connector;
//import javax.microedition.io.ServerSocketConnection;
//import javax.microedition.io.SocketConnection;
//import javax.microedition.lcdui.Display;
//import javax.microedition.lcdui.Form;
//import javax.microedition.midlet.MIDlet;
//import javax.microedition.midlet.MIDletStateChangeException;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

public class HttpServer extends Thread implements Customer {

    public static final int SERVER_PORT = 10000;
    public static final String SERVER_INFO = "BetelgeuseMIDP (Port " + SERVER_PORT + ")";
    public static final String HTTP_VERSION = "HTTP/1.0";

    private Blackboard bb;
    private Vector allProducers;
    private Logger logger = null;
    private Hashtable data;
    private Hashtable registeredComponents;
    private boolean keepListening;
    
    private static final int REPLY_FORMAT_JSON = 1;
    private static final int REPLY_FORMAT_HTML = 2;

    public HttpServer(Logger logger) {
        this.allProducers = new Vector();
        this.logger = logger;
        this.data = new Hashtable();
        this.registeredComponents = new Hashtable();
        this.keepListening = true;

        this.bb = BlackboardFactory.getBlackboard(logger);
        // logger.logMessage("[httpd] got the blackboard");
        this.bb.subscribeConsumer(this);
        // logger.logMessage("[httpd] subscribing to blackboard");
    }

    public void run() {
        logger.logMessage("[httpd] Creating MIDPServer-object");
        MIDPServer midpServer = new MIDPServer(logger);

        logger.logMessage("[httpd] Opening server socket");
        ServerConnection listeningSocket = midpServer.open(SERVER_PORT);
        logger.logMessage("[httpd] Now listening to port " + SERVER_PORT);

        while (keepListening) {
            Connection connection = listeningSocket.accept();
            logger.logMessage("[httpd] client connected");
            RequestHandler h = new RequestHandler(connection, this);
            h.start();
        }
    }
    
    public void registerComponent(HttpComponent component) {
    	Path[] watchedPaths = component.getWatchedPaths();
    	for (int i = 0; i < watchedPaths.length; i++) {
    		if (!this.registeredComponents.containsKey(watchedPaths[i])) {
    			this.registeredComponents.put(watchedPaths[i], component);
    			logger.logMessage(("[httpd] component registered for " + watchedPaths[i]));
    		}
    	}
    }
    
    public HttpComponent getComponent(Path path) {
    	return (HttpComponent) this.registeredComponents.get(path);
    }
    
    /**
     * Returns the data of all producers given as parameter in a single HTML-document.
     * The data is formatted as a series of HTML tables.
     * 
     * @return A HTML document containing &lt;table&gt; elements containing produced data.
     */
    public String getProducerDataAsHtml(Vector producers) {
        String bbData = "<html><body>";
        // for every producer
        for (int i = 0; i < producers.size(); i++) {
            Customer producer = (Customer) producers.elementAt(i);
            bbData += "<h1>" + producer.getInfo().getName() + "</h1>";
            bbData += "<h2>Sensor Info</h2>";
            bbData += "Id: " + producer.getInfo().getId() + "<br>";
            bbData += "Internal: " + producer.getInfo().isInternal() + "<br>";
            CustomerData producerData = (CustomerData) data.get(producer);
            Vector[] dataArray = null;
            String[] parameterNames = null;
            int[] parameterTypes = null;
            if (producerData != null) {
                dataArray = producerData.getData();
                parameterNames = producerData.getParameterNames();
                parameterTypes = producerData.getParameterTypes();
            }
            if (dataArray != null) {
            	bbData += "<h2>Sensor Data</h2>";
                bbData += "Data timestamp: " + producerData.getTimestamp();
                bbData += "<table border=\"1\">";
                bbData += "<th></th><th>Name</th><th>Value</th>";
                // for every vector of data
                for (int j = 0; j < dataArray.length; j++) {
                    bbData += "<tr>";
                    bbData += "<td>" + j + "</td>";
                    bbData += "<td>" + parameterNames[j] + "</td>";
                    //bbData += "<td>" + parameterTypes[j] + "</td>";
                    // for every element in the vector
                    bbData += "<td>";
                    Enumeration elements = dataArray[j].elements();
                    while (elements.hasMoreElements()) {
                        bbData += (String) elements.nextElement() + "<br>";
                    }
                    bbData += "</td></tr>";
                }
                bbData += "</table>";
            }
        }
        bbData += "</body></html>";
        
        return bbData;
    }
    
    
    public String getProducerData(Vector producers) {

        String d = ""; // the data to be returned

        d += "<br />"; // begin array of producer objects
        for (int i = 0; i < producers.size(); i++) {
            // begin new producer object
          
            Customer prod = (Customer) producers.elementAt(i);
            CustomerInfo prodInfo = prod.getInfo();
            /* "name": "value", */
            d = d.concat("<h2>name: " + prodInfo.getName() + "</h2>"); // String name,
            d = d.concat("id: " + prodInfo.getId() + "<br />"); // int id,
            d = d.concat("type: " + prodInfo.getType() + "<br />"); // int type,
            
            d = d.concat("user: " + prodInfo.getUser() + "<br />"); // int user,
            d = d.concat("mode: " + prodInfo.getMode() + "<br />"); // int mode,
            d = d.concat("timeout: " + prodInfo.getTimeout() + "<br />"); // long timeout,
            d = d.concat("alive: " + prodInfo.isAlive() + "<br />"); // boolean alive,
            d = d.concat("address:" + prodInfo.getAddress() + "<br />"); // String address
            CustomerData producerData = (CustomerData) data.get(prod);
            d = d.concat("data: <br /> "); // start data object
            if (producerData != null) {
                Vector[] dataArray = producerData.getData();
                String[] paramNames = producerData.getParameterNames();
               
                for (int j = 0; j < dataArray.length; j++) {
                    d = d.concat(paramNames[j] + ": "); // "name":
                    Vector dataElement = dataArray[j];
                    if (dataElement.size() == 1) {
                        // Single value, no array
                        d = d.concat("" + dataElement.elementAt(0)+"<br />"); // value
                    } else {
                        // Set of values, make an array
                        d += "<br />";
                        for (int k = 0; k < dataElement.size(); k++) {
                            d = d.concat("" + dataElement.elementAt(k)+", "); // value
                            if (k == (dataElement.size() - 1))
                                d=d.substring(0, d.length()-1); // add a comma as separator, unless this is the last
                                            // element
                            d += "<br />";
                        }
                    }
                    if (j < (dataArray.length - 1))
                        d = d.concat(", <br />");
                    else 
                    	d= d.concat("<br />");
                }
            }
            // end data object
            d = d.concat("<br />");
            // end producer object
            
            // add a comma as separator, unless this is the last element
            if (i < (producers.size() - 1))
                d += ", ";
            d += "<br />"; // end with newline for clarity
        }
        d += "<br />"; // end array of producer objects
        logger.logMessage(("[httpd]"+d));
        //System.out.println(d);
        return d;
    }
    /*Taru--> */
    
    /*Helper method, not needed at the moment, but might be int the future for capella*/
    public String getUsername(){
    	String username = bb.getUsername();
    	return username;
    }
    
    /*Helper method, not needed at the moment, but might be needed the future for capella*/
    public String getBluetoothAddress(){
    	return bb.getBTAddress();
    }
    
    /*Helper method, not needed at the moment, might be needed in future for capella*/ 
    public int getSessionId(){
    	return bb.getSessionId();
    }
    
    /*Method for returning username, bluetoothaddress and the contextdata to capella when asked...*/
    public String getUsernameAndDataAsJson(Vector producers){
    	String d = "{\"user\":\""+getUsername()+"\",\"sessionid\":\""+getSessionId()+"\",\"btaddress\":\""+getBluetoothAddress()+"\"}";//, \"context\":"+getProducerDataAsJson(producers)+"}";
    	return d;
    }
    
    /*This is no longer (or at the moment) needed... */
    public String getSingleProducerDataAsJson(Vector producers, int index) {
        String d = ""; // the data to be returned

        // begin new producer object
        d += "{";
        
        d=d.concat("\"size\":"+producers.size()+", ");
        
        Customer prod = (Customer) producers.elementAt(index);
        
        CustomerInfo prodInfo = prod.getInfo();
        /* "name": "value", */
        d = d.concat("\"id\": " + prodInfo.getId() + ",\n"); // int id,
        d = d.concat("\"type\": " + prodInfo.getType() + ",\n"); // int type,
        d = d.concat("\"name\": \"" + prodInfo.getName() + "\",\n"); // String name,
        d = d.concat("\"user\": " + prodInfo.getUser() + ",\n"); // int user,
        d = d.concat("\"mode\": " + prodInfo.getMode() + ",\n"); // int mode,
        d = d.concat("\"timeout\": " + prodInfo.getTimeout() + ",\n"); // long timeout,
        d = d.concat("\"alive\": " + prodInfo.isAlive() + ",\n"); // boolean alive,
        d = d.concat("\"address\": \"" + prodInfo.getAddress() + "\",\n"); // String address
        CustomerData producerData = (CustomerData) data.get(prod);
        d = d.concat("\"data\": {\n"); // start data object
        if (producerData != null) {
        Vector[] dataArray = producerData.getData();
            String[] paramNames = producerData.getParameterNames();
            int[] paramTypes = producerData.getParameterTypes();
            

            for (int j = 0; j < dataArray.length; j++) {
                d = d.concat("\"" + paramNames[j] + "\": "); // "name":
                Vector dataElement = dataArray[j];
                if (dataElement.size() == 1) {
                    // Single value, no array
                    if (paramTypes[j] == Properties.DATATYPE_STRING)
                        d = d.concat("\""); // Surround value by quotes if String
                    d = d.concat("" + dataElement.elementAt(0)); // value
                    if (paramTypes[j] == Properties.DATATYPE_STRING)
                        d = d.concat("\"");
                } else {
                    // Set of values, make an array
                    d += "[\n";
                    for (int k = 0; k < dataElement.size(); k++) {
                        if (paramTypes[j] == Properties.DATATYPE_STRING)
                            d = d.concat("\""); // Surround value by quotes if String
                        d = d.concat("" + dataElement.elementAt(k)); // value
                        if (paramTypes[j] == Properties.DATATYPE_STRING)
                            d = d.concat("\"");
                        if (k < (dataElement.size() - 1))
                            d += ", "; // add a comma as separator, unless this is the last
                                            // element
                        d += "\n";
                    }
                    d += "]\n";

                }
                if (j < (dataArray.length - 1))
                    d += ", "; // add a comma as separator, unless this is the last element
                d = d.concat("\n");
            }
        }
        // end data object
        d = d.concat("}\n");
        // end producer object
        d += "}";
        return d;
    }

    /*-->Taru*/
    
    /**
     * Get the data from one or more producers in JSON format.
     * 
     * @param producers
     *            the producers whose data we want (a Vector of Customers)
     * @return JSON objects representing the data of the producers
     */
    public String getProducerDataAsJson(Vector producers) {

        String d = ""; // the data to be returned

        d += "[\n"; // begin array of producer objects
       /* if(producers.size()>1){
        	d=d.concat("\"size\":"+producers.size()+", ");
        }*/
        for (int i = 0; i < producers.size(); i++) {
            // begin new producer object
            d += "{";

            Customer prod = (Customer) producers.elementAt(i);
            CustomerInfo prodInfo = prod.getInfo();
            /* "name": "value", */
            d = d.concat("\"id\": " + prodInfo.getId() + ",\n"); // int id,
            d = d.concat("\"type\": " + prodInfo.getType() + ",\n"); // int type,
            d = d.concat("\"name\": \"" + prodInfo.getName() + "\",\n"); // String name,
            d = d.concat("\"user\": " + prodInfo.getUser() + ",\n"); // int user,
            d = d.concat("\"mode\": " + prodInfo.getMode() + ",\n"); // int mode,
            d = d.concat("\"timeout\": " + prodInfo.getTimeout() + ",\n"); // long timeout,
            d = d.concat("\"alive\": " + prodInfo.isAlive() + ",\n"); // boolean alive,
            d = d.concat("\"address\": \"" + prodInfo.getAddress() + "\",\n"); // String address
            CustomerData producerData = (CustomerData) data.get(prod);
            d = d.concat("\"data\": {\n"); // start data object
            if (producerData != null) {
                Vector[] dataArray = producerData.getData();
                String[] paramNames = producerData.getParameterNames();
                int[] paramTypes = producerData.getParameterTypes();

                for (int j = 0; j < dataArray.length; j++) {
                    d = d.concat("\"" + paramNames[j] + "\": "); // "name":
                    Vector dataElement = dataArray[j];
                    if (dataElement.size() == 1) {
                    	/*if(paramNames[j].equals("BTDevices")){
                    		
                    		 //JSON object
                    		String dataString = dataArray[j].toString();
                    		if(dataString.length()>2){
                    			dataString=dataString.substring(1,dataString.length()-1);
                    			d += "{";
                        		do{
                        			int btDevSepInd  = dataString.indexOf('\n');
                        			String singleBTDevData;
                        			if(btDevSepInd>0){
                        				singleBTDevData = dataString.substring(0,btDevSepInd);
                        			}else{
                        				singleBTDevData = dataString;
                        			}
                        			int sepInd = singleBTDevData.indexOf(' ');
                        			String BTCode = singleBTDevData.substring(0, sepInd);
                        			String BTName = singleBTDevData.substring(sepInd+1);
                        			d+="\""+BTCode+"\":\""+BTName+"\",";
                        			dataString = dataString.substring(btDevSepInd+1);
                        		}while(dataString.indexOf('\n')!=-1);	
                        	    d= d.substring(0, d.length()-1);
                        		d+="}";
                    		}*/
                    		//array
                    		/*String dataString = dataArray[j].toString();
                    		if(dataString.length()>2){
                    			dataString=dataString.substring(1,dataString.length()-1);
                    			d += "[";
                        		do{
                        			int btDevSepInd  = dataString.indexOf('\n');
                        			String singleBTDevData;
                        			if(btDevSepInd>0){
                        				singleBTDevData = dataString.substring(0,btDevSepInd);
                        			}else{
                        				singleBTDevData = dataString;
                        			}
                        			d+="\""+singleBTDevData+"\",";
                        			dataString = dataString.substring(btDevSepInd+1);
                        		}while(dataString.indexOf('\n')!=-1);	
                        	    d= d.substring(0, d.length()-1);
                        		d+="]";
                    		}
                    		
                    	}else{*/
                    		// Single value, no array
                    		if (paramTypes[j] == Properties.DATATYPE_STRING)
                    			d = d.concat("\""); // Surround value by quotes if String
                    		d = d.concat("" + dataElement.elementAt(0)); // value
                    		if (paramTypes[j] == Properties.DATATYPE_STRING)
                    			d = d.concat("\"");
                    	//}
                    } else {
                    	
                        // Set of values, make an array
                        d += "[\n";
                        for (int k = 0; k < dataElement.size(); k++) {
                            if (paramTypes[j] == Properties.DATATYPE_STRING)
                                d = d.concat("\""); // Surround value by quotes if String
                            d = d.concat("" + dataElement.elementAt(k)); // value
                            if (paramTypes[j] == Properties.DATATYPE_STRING)
                                d = d.concat("\"");
                            if (k < (dataElement.size() - 1))
                                d += ", "; // add a comma as separator, unless this is the last
                                            // element
                            d += "\n";
                        }
                        d += "]\n";

                    }
                    if (j < (dataArray.length - 1))
                        d += ", "; // add a comma as separator, unless this is the last element
                    d = d.concat("\n");
                }
            }
            // end data object
            d = d.concat("}\n");
            // end producer object
            d += "}";
            // add a comma as separator, unless this is the last element
            if (i < (producers.size() - 1))
                d += ", ";
            d += "\n"; // end with newline for clarity
        }
        d += "]\n"; // end array of producer objects
        logger.logMessage(("[httpd]"+d));
        //System.out.println(d);
        return d;
    }

    /**
     * Returns producers based on their names.
     * 
     * Finds all procures whose name matches producerName (ignoring case) and returns them in a
     * Vector. A producerName of "*" matches all producers.
     * 
     * @param producerName
     *            the name of the producer, or * for all of them
     * @return the producers with matching names
     */
    public Vector getProducersByName(String producerName) {
        Vector producers; // producers, whose data will be returned

        // Get all producers if name is *
        if (producerName.equals("*")) {
            producers = allProducers;
        } else {
            // Find producers whose name matches producerName, ignoring case.
            producers = new Vector();
            for (int i = 0; i < allProducers.size(); i++) {
                String otherName = ((Customer) allProducers.elementAt(i)).getInfo().getName();
                if (otherName.equalsIgnoreCase(producerName))
                    producers.addElement((Customer) allProducers.elementAt(i));
            }
        }
        return producers;
    }

    /**
     * Returns all producers.
     * 
     * @return all producers associated with this blackboard
     */
    public Vector getAllProducers() {
        return allProducers;
    }

    // Implementing hiit.blackboard.Customer

    public CustomerInfo getInfo() {
        // TODO Auto-generated method stub
        return null;
    }

    /*
     * Puts the new data from producer prod into this.data (replacing the last data from that
     * producer) (non-Javadoc)
     * 
     * @see hiit.blackboard.Customer#newData(hiit.blackboard.Customer, hiit.blackboard.CustomerData)
     */
    public void newData(Customer prod, CustomerData pd) {
        data.put(prod, pd);
    }

    public boolean requestData() {
        // TODO Auto-generated method stub
        return false;
    }

    public boolean setProducerMode(int mode, long timeout) {
        // TODO Auto-generated method stub
        return false;
    }

    public void updateCustomerInfo(Blackboard bb, Customer c, CustomerInfo pi) {
        // TODO Auto-generated method stub

    }

    public void updateProducerList(Blackboard bb, Vector producers) {
        for (int i = 0; i < producers.size(); i++) {
            Customer cust = (Customer) producers.elementAt(i);
            if (!allProducers.contains(cust) && (!cust.getInfo().isInternal())) {
                allProducers.addElement(cust);
                bb.subscribeToProducer(this, cust);
            }
        }
    }
    
    public void logMessage(String message) {
        if (this.logger != null)
            logger.logMessage(message);
    }
    
    public String toString() {
        return HttpServer.SERVER_INFO;
    }

    /**
     * Class for handling an individual HTTP request
     * by A) reading and parsing the request
     * and B) finding an appropriate reply and sending it to the client.
     * After sending a reply the connection is closed and the thread dies.
     * 
     * A new RequestHandler-thread is created by HttpServer for every incoming TCP
     * connection that is established with its server socket.
     */
    class RequestHandler extends Thread {

        private Connection socket;
        private HttpServer server;
        
        /**
         * Creates a new RequestHandler
         * 
         * @param fromClient the socket connection to the client
         * @param server the HttpServer this RequestHandler belongs to
         */
        public RequestHandler(Connection fromClient, HttpServer server) {
            this.socket = fromClient;
            this.server = server;
        }

        // Try reading a HTTP message from the input stream of the socket and
        // send an appropriate reply back through the same socket.
        public void run() {
            InputStream in = null;
            in = socket.getIs();
            
            if (in != null) {
            	HttpResponse response = null;
                char ch;
                StringBuffer b = new StringBuffer();
                Vector lines = new Vector();
                try {
                    // Read first line of the message, terminates at newline or end of stream
                    while ((ch = (char) in.read()) != -1 && ch != '\n') {
                        b.append(ch);
                    }
                    String line = b.toString().trim();
                    lines.addElement(line);
                    
                    // Check what kind of message it is.
                    //   For now we support GET, POST and HEAD messages.
                    
                    // GET or HEAD
                    if (line.startsWith("GET") || line.startsWith("HEAD")) {
                        // Read rest of message.
                        b = new StringBuffer();
                        while ((ch = (char) in.read()) != -1) {
                            if (ch == '\n') {
                                if (b.length() == 0) {
                                	// stop when the first empty line is encountered
                                    break;
                                } else {
                                    lines.addElement(b.toString());
                                    b = new StringBuffer();
                                }
                            } else if (ch == '\r') {
                                continue;
                            } else {
                                b.append(ch);
                            }
                        }
                        HttpRequest request = null;
                        try {
                        	// Parse message as a HttpRequest
                            request = new HttpRequest(lines);
                            // Deal with the request
                            response = this.handleGet(request);
                        } catch (HttpFormatException e) {
                            server.logMessage("[httpd] error while parsing HTTP-request: " + e.getMessage());
                            response = HttpResponse.badRequest(e.getMessage(), server.toString());
                        }
                    }
                    
                    // POST
                    else if (line.startsWith("POST")) {
                        // Read rest of message plus the posted data.
                    	int emptyLines = 0;
                        b = new StringBuffer();
                        while ((ch = (char) in.read()) != -1) {
                            if (ch == '\n') {
                                if (b.length() == 0) {
                                	emptyLines++;
                                	if (emptyLines == 2) {
                                		// stop when the second empty line is encountered
                                		break;
                                	}
                                } else {
                                    lines.addElement(b.toString());
                                    b = new StringBuffer();
                                }
                            } else if (ch == '\r') {
                                continue;
                            } else {
                                b.append(ch);
                            }
                        }

                        HttpRequest request = null;
                        try {
                        	// Parse message as a HttpRequest
                            request = new HttpRequest(lines);
                            // Deal with the request
                            response = this.handlePost(request);
                        } catch (HttpFormatException e) {
                            // Bad request
                            server.logMessage("[httpd] error while parsing HTTP-request: " + e.getMessage());
                            response = HttpResponse.badRequest(e.getMessage(), server.toString());
                        }
                    }
                    
                    // Unsupported (but valid) operation
                    else if (line.startsWith("PUT ") || line.startsWith("DELETE ") 
                    		|| line.startsWith("TRACE ") || line.startsWith("OPTIONS ") 
                    		|| line.startsWith("CONNECT ") ) {
                        String op = line.substring(0, line.indexOf(" "));
                        server.logMessage("[httpd] unsupported operation: " + op);
                        response = HttpResponse.methodNotImplemented(op, "", server.toString());
                    }
                    
                    // Invalid request
                    else {
                    	response = HttpResponse.badRequest("\"" + line + "\""
                    			+ " does not seem to be a valid HTTP request.", server.toString());
                    }
                    
                } catch (IOException e) {
                    server.logMessage("[httpd] error reading input stream: " + e.toString());
                }
                
                // Get output stream to client
                OutputStream os = socket.getOs();
                PrintStream out = new PrintStream(os);
                
                // Send response.
                // End with two CRLF line breaks.
                out.print(response.toString() + "\r\n\r\n");
                out.flush();
                out.close();
                
                socket.close();
            } // end "if (is != null)"
        } // end run()
        
        
        /**
         * Figures out an appropriate HTTP response to a request and returns it.
         * 
         * Called from the run-method of RequestHandler when a valid GET or HEAD -request
         * has been read and parsed.
         * 
         * @param request the request to deal with
         * @return a response to the request
         */
        public HttpResponse handleGet(HttpRequest request) {
        	HttpResponse response = null;
        	if (request != null) {
        		Path path = request.getPath();
        		int format;
        		String formatString = (String) request.getQueryParameters().get("format");
        		if (formatString != null && formatString.equalsIgnoreCase("html")) {
        			format = REPLY_FORMAT_HTML;
        		} else {
        			format = REPLY_FORMAT_JSON;
        		}
        		String responseData;
        		
        		if (path.length() == 0) {
            		// root
        			if (format == REPLY_FORMAT_HTML) {
        				responseData = server.getProducerDataAsHtml(server.getAllProducers());
        			} else {
        				 server.logMessage("[httpd] getting all producers");
        				responseData = server.getProducerDataAsJson(server.getAllProducers());
        				while(responseData.indexOf('\'')!=-1){
        					responseData = responseData.replace('\'', ' ');
        				}
        			    
        			    while(responseData.indexOf('\n')!=-1){
        			    	   responseData= responseData.replace('\n', ' ');
        			    }
        			}
        			response = new HttpResponse(responseData);
        			
        		} else {
        			// non-root
        			Vector matchingProducers = server.getProducersByName(path.getLevel(0));
        			if (matchingProducers.size() > 0) {
        				// matching producer name found in blackboard
        				if (format == REPLY_FORMAT_HTML) {
        					responseData = server.getProducerDataAsHtml(matchingProducers);
        				} else {
        					 server.logMessage("[httpd] matching:"+matchingProducers.toString());
        					responseData = server.getProducerDataAsJson(matchingProducers);
        					while(responseData.indexOf('\'')!=-1){
            					responseData = responseData.replace('\'', ' ');
            				}
        					while(responseData.indexOf('\n')!=-1){
          			    	   responseData= responseData.replace('\n', ' ');
          			        }
        				}
        				response = new HttpResponse(responseData);
        				
        			} else {
        				// no matching producer in blackboard, looking for registered HttpComponents
        				HttpComponent c = server.getComponent(path);
        				if (c != null) {
        					// registered component found
        					response = c.getReply(request);

        				} else {
        					// no match found for path
        					response = HttpResponse.notFound(path.toString(), server.toString());
        				}
        			}	
        		}
        		
            	// check for callback function in json cases
            	if (response.getStatusCode() == 200 && format != REPLY_FORMAT_HTML) {
            		String callbackFunction1 = (String) request.getQueryParameters().get("jsoncallback");
            		String callbackFunction2 = (String) request.getQueryParameters().get("callback");
            		if (callbackFunction1 != null) {
            			response.setEntity(callbackFunction1 + "(" + response.getEntity() + ")" );
            		} else if (callbackFunction2 != null) {
            			response.setEntity(callbackFunction2 + "(" + response.getEntity() + ")" );
            		}
            	}
        	}
        	  	
            // Quick and dirty. :-P
            // if it's a HEAD request we just remove the response body from the reply...
            if (request.getOperation().equals("HEAD")) {
            	response.setEntity(null);
            }
            return response;
        }
        
        /* This is old
        public HttpResponse handleGet_test(HttpRequest request) {
            HttpResponse response = null;
            if (request != null) {
                String path = request.getPath().toString();
                
                // HTML-responses
                if (path.equals("/html") || path.equals("/")) {
                    String html = "<html><body><p>Welcome to Betelgeuse!</p>"
                            + "<p>The blackboard data as HTML is <a href = \"http://127.0.0.1:10000/html/data\">here</a>.</p>"
                            + "<p>You can also get it in <a href=\"http://127.0.0.1:10000/data\">JSON</a> format.</p></body></html>";
                    response = new HttpResponse(html);
                } else if (path.equals("/html/data")) {
                    response = new HttpResponse(server.getProducerDataAsHtml(server.getAllProducers()));
                } else if (path.equals("/serpens")) {
                	HttpComponent locationPlugin = server.getComponent(new Path(path));
                	response = locationPlugin.getReply(request);
                	if (response == null) {
                		response = HttpResponse.notFound(path.toString(), server.toString());
                	}
                }

                // JSON-responses
                else {
                    // Return data from all producers
                    if (path.equals("/data")) {
                        String bbData = server.getProducerDataAsJson(server.getAllProducers());
                        String callbackFunction = (String) request.getQueryParameters().get("callback");
                        if (callbackFunction != null) {
                            bbData = callbackFunction + "(" + bbData + ")";
                        }
                        response = new HttpResponse(bbData);
                        // response.setContentType("application/x-javascript");
                        response.setContentType("");
                        response.setCharset("");

                    } else {
                        // Search for producers with matching name
                        Vector foundProducers = server.getProducersByName(path
                                .substring(1)); // substring(1) removes the leading slash
                        if (foundProducers.size() > 0) {
                            String bbData = server
                                    .getProducerDataAsJson(foundProducers);
                            String callbackFunction = (String) request
                                    .getQueryParameters().get("callback");
                            if (callbackFunction != null)
                                bbData = callbackFunction + "(" + bbData + ")";
                            response = new HttpResponse(bbData);
                            // response.setContentType("application/x-javascript");
                            response.setContentType("");
                            response.setCharset("");
                        } else {
                            response = HttpResponse.notFound(path,
                                    server.toString());
                        }
                    }
                }
            }
            // Quick and dirty. :-P
            // if it's a HEAD request we just remove the response body from the reply...
            if (request.getOperation().equals("HEAD")) {
            	response.setEntity(null);
            }
            return response;
        }
        */
        
        /**
         * Figures out an appropriate response to the request and returns it.
         * 
         * Called from the run-method of RequestHandler when a valid POST-request
         * has been read and parsed.
         * @param request the request to deal with
         * @return a response to the request
         */
        public HttpResponse handlePost(HttpRequest request) {
            return HttpResponse.methodNotAllowed(request.getOperation(), request.getPath().toString(), server.toString());
        }

    }

	
	public void newData(Customer prod, CustomerData pd, EventData eventData){
		// TODO Auto-generated method stub
		//Not valid
		
	}

	public void markParameters() {
		// TODO Auto-generated method stub
		//Not valid
	}

}
