package hiit.blackboard.httpd;
import hiit.logger.Logger;

/**
 * Unified way to get a reference to the HttpServer.
 * 
 * @author mjmander
 */
public class HttpServerFactory {
	private static HttpServer server = null;
	private static Logger logger;
	
	// debug variable that prohibits all attempts to
	// start the HttpServer through this factory
	public static final boolean NO_HTTP_SERVER = false;
	
	/**
	 * Returns the HttpServer.
	 * 
	 * Returns the HttpServer that is running in this instance of BeTelGeuse.
	 * If there is no HttpServer it will be created. Only one server will
	 * be running at any time.
	 * 
	 * @param logger The logger object used to store log messages. 
	 * @return The HttpServer running in this instance of BeTelGeuse
	 */
	public static HttpServer getHttpServer(Logger logger) {
		if (NO_HTTP_SERVER) {
			return null;
		}
		
		if (HttpServerFactory.logger == null) {
			HttpServerFactory.logger = logger;
		}
		if (server == null) {
			server = new HttpServer(logger);
			server.start();
			ContextFetcherComponent cfc = new ContextFetcherComponent(logger);
			server.registerComponent(cfc);
		}
		return server;
	}
	
	public static void startHttpServer(Logger logger) {
		getHttpServer(logger);
	}
}
