package hiit.blackboard.httpd;

import java.util.Vector;

public class Path {
	
	private Vector parts;
	
	public static final char SEPARATOR = '/';
	public static final String ROOT = "/";
	
	public Path(String path) {
		this.parts = new Vector();
		
		// read the path char by char, splitting at each SEPARATOR
		// e.g. "/foo/bar/" and "//foo////bar" become ["foo", "bar"]
		for (int i = 0; i < path.length(); i++) {
			StringBuffer part = new StringBuffer();
			while (i < path.length() && path.charAt(i) != SEPARATOR) {
				part.append(path.charAt(i));
				i++;
			}
			if (part.length() > 0) {
				// non-zero parts are added
				this.parts.addElement(part.toString());
			}
		}
	}

	public int length() {
		return this.parts.size();
	}
	
	public String toString() {
		if (this.parts.size() == 0) {
			return ROOT;
		} else {
			String path = "";
			for (int i = 0; i < this.parts.size(); i++) {
				path += SEPARATOR + (String)parts.elementAt(i);
			}
			return path;
		}
	}
	
	public String getLevel(int level) {
		if (level < parts.size()) {
			return (String) parts.elementAt(level);
		} else {
			return null;
		}
	}
	
	public boolean equals(Object o) {
		if (o == null) {
			return false;
		} else if (this.toString().equals(o.toString())) {
			return true;
		} else {
			return false;
		}
	}
	
	public int hashCode() {
		return this.toString().hashCode();
	}
	
	// test
	public static void main(String[] args) {
		Path p = new Path("serpens");
		System.out.println(p.length());
		System.out.println(p);
		
		Path p2 = new Path("/serpens/");
		System.out.println(p.equals(p2));
	}
}
