package hiit.blackboard.testing;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.microedition.io.Connector;
import javax.microedition.io.StreamConnection;

import hiit.blackboard.BlackboardFactory;
import hiit.blackboard.Blackboard;
import hiit.mapper.Properties;
import hiit.blackboard.protocol.Protocol;
import hiit.logger.Logger;
import hiit.logger.MIDPLogger;

public class SocketCommTest {
    
    private static StreamConnection s1 = null;
    
    private static InputStream in1 = null;
    
    private static OutputStream out1 = null;
    
    private static StreamConnection s2 = null;
    
    private static InputStream in2 = null;
    
    private static OutputStream out2 = null;
    
    public static void main(String[] args) {
        test();
    }
    
    private static void test() {
        Logger logger = new MIDPLogger();
        Blackboard b = BlackboardFactory.getBlackboard(logger);
        // started listening on Properties.LISTEN_PORT.
        // let's try connect:
        initConnection(true);
        // connection ok if we did not exit yet.
        // Let's try a create, a subscribe and sending data.
        sendMessages();
    }
    
    private static void initConnection(boolean one) {
        try {
            if (one) {
                s1 = (StreamConnection) Connector.open("socket://localhost:"
                        + Properties.LISTEN_PORT);
                in1 = s1.openInputStream();
                out1 = s1.openOutputStream();
            } else {
                s2 = (StreamConnection) Connector.open("socket://localhost:"
                        + Properties.LISTEN_PORT);
                in2 = s2.openInputStream();
                out2 = s2.openOutputStream();
            }
        } catch (IOException e) {
            sop("Unable to connect to port " + Properties.LISTEN_PORT + ": "
                    + e);
            System.exit(1);
        }
    }
    
    private static void sendMessages() {
        // create
        int type = 0x201;
        int type2 = 0x202;
        int userid = 1;
        String username = "pahvi";
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        Protocol.writeCreate(bos, type, userid, username);
        try {
            out1.write(bos.toByteArray());
        } catch (IOException e) {
            sop("Could not send create: " + e);
        }
        sop("Wrote a create message to output.");
        // Now we should get a created message.
        // should read created next:
        if (!testGeneric('d', "created", true, type, userid, 2, out1, in1)) { // compid
                                                                                // does
                                                                                // not
                                                                                // matter
                                                                                // when
                                                                                // reading
            sop("Unable to receive a proper created message. Quitting.");
            System.exit(1);
        }
        // now we can subscribe:
        if (!testGeneric('s', "subscribe", false, 2, Integer.MAX_VALUE,
                Integer.MAX_VALUE, out1, in1)) {
            sop("Unable to receive a proper created message. Quitting.");
            System.exit(1);
        }
        // read ack for our sub.
        if (!testAck('a', "ack", true, out1, in1)) {
            sop("Unable to receive a proper created message. Quitting.");
            System.exit(1);
        }
        
        // Create another customer.
        initConnection(false);
        bos = new ByteArrayOutputStream();
        Protocol.writeCreate(bos, type2, userid, username);
        try {
            out2.write(bos.toByteArray());
        } catch (IOException e) {
            sop("Could not send create: " + e);
        }
        sop("Wrote a create message to output.");
        // Now we should get a created message.
        // should read created next:
        if (!testGeneric('d', "created", true, type2, userid, 2, out2, in2)) { // compid
                                                                                // does
                                                                                // not
                                                                                // matter
                                                                                // when
                                                                                // reading
            sop("Unable to receive a proper created message. Quitting.");
            System.exit(1);
        }
        // now we can subscribe:
        if (!testGeneric('s', "subscribe", false, 2, Integer.MAX_VALUE,
                Integer.MAX_VALUE, out2, in2)) {
            sop("Unable to receive a proper created message. Quitting.");
            System.exit(1);
        }
        // read ack for our sub.
        if (!testAck('a', "ack", true, out2, in2)) {
            sop("Unable to receive a proper created message. Quitting.");
            System.exit(1);
        }
        
        // Send a pushdata.
        testPushData('p', "Pushdata", false, out2, in2);
        
        testPushData('p', "Pushdata", false, out1, in1);
        
        testReturnData('u', "Returndata", true, out2, in2);
        
        testReturnData('u', "Returndata", true, out1, in1);
    }
    
    private static boolean testReturnData(char cmd, String cmdname,
            boolean read, OutputStream out, InputStream in) {
        if (!read) {
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            int userid = 13;
            int compid = 12;
            int type = 214343;
            String data = "blablafaekeoam''42";
            
            StringBuffer msgs = new StringBuffer();
            msgs.append("Wrote a " + cmdname + " message with type=" + type
                    + " userid=" + userid + " compid=" + compid + " data="
                    + data + '\n');
            
            Protocol
                    .writeReturnData(bos, type, userid, compid, data.getBytes());
            byte[] b = bos.toByteArray();
            try {
                out.write(b);
                out.flush();
                return true;
            } catch (IOException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        } else {
            int cmdr;
            try {
                cmdr = in.read();
                if (cmdr != cmd) {
                    sop("ERROR: Wrote a " + cmdname
                            + " message but got message command: "
                            + (char) cmdr);
                    return false;
                }
                sop("Read message command: " + (char) cmdr);
                Object[] stuff = Protocol.readReturnData(in);
                int rtype = ((Integer) stuff[0]).intValue();
                int ruserid = ((Integer) stuff[1]).intValue();
                int rcompid = ((Integer) stuff[2]).intValue();
                String rdata = new String((byte[]) stuff[3]);
                sop("Read a " + cmdname + " message with type=" + rtype
                        + " userid=" + ruserid + " compid=" + rcompid
                        + " data=" + rdata);
                return true;
            } catch (IOException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
        return false;
    }
    
    private static boolean testAck(char cmd, String cmdname, boolean read,
            OutputStream out, InputStream in) {
        if (!read) {
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            int ack = 'c';
            int status = 32;
            String msg = "";
            
            StringBuffer msgs = new StringBuffer();
            msgs.append("Wrote a " + cmdname + " message with ack="
                    + (char) ack + " status=" + status + " message=" + msg
                    + '\n');
            
            Protocol.writeAckOrError(bos, ack, status, msg);
            byte[] b = bos.toByteArray();
            try {
                out.write(b);
                out.flush();
                return true;
            } catch (IOException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        } else {
            int cmdr = -1;
            try {
                cmdr = in.read();
            } catch (IOException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
            if (cmdr != cmd) {
                sop("ERROR: Wrote a " + cmdname
                        + " message but got message command: " + (char) cmdr);
                return false;
            }
            Object[] stuff = Protocol.readAckOrError(in);
            int rack = ((Integer) stuff[0]).intValue();
            int rstat = ((Integer) stuff[1]).intValue();
            String rmsg = new String((byte[]) stuff[2]);
            
            sop("Read a " + cmdname + " message with ack=" + (char) rack
                    + " status=" + rstat + " msg=" + rmsg + '\n');
        }
        return true;
    }
    
    private static boolean testGeneric(char cmd, String cmdname, boolean read,
            int type, int userid, int compid, OutputStream out, InputStream in) {
        if (!read) {
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            
            sop("Wrote a " + cmdname + " message with type=" + type
                    + " userid=" + userid + " compid=" + compid + '\n');
            
            writeSwitch(cmd, bos, type, userid, compid);
            byte[] b = bos.toByteArray();
            try {
                out.write(b);
                out.flush();
                return true;
            } catch (IOException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        } else {
            int cmdr;
            try {
                cmdr = in.read();
            } catch (IOException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
                return false;
            }
            if (cmdr != cmd) {
                sop("ERROR: Expected a  " + cmdname
                        + " message but got message command: " + (char) cmdr);
                return false;
            }
            int[] stuff = readSwitch(cmd, in);
            int rtype = stuff[0];
            int ruserid = stuff[1];
            int rcompid = stuff[2];
            sop("Read a " + cmdname + " message with type=" + rtype
                    + " userid=" + ruserid + " compid=" + rcompid + '\n');
            return true;
        }
        return false;
    }
    
    private static boolean testPushData(char cmd, String cmdname, boolean read,
            OutputStream out, InputStream in) {
        if (!read) {
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            String data = "blablablafish12242";
            int compid = 21;
            sop("Wrote a " + cmdname + " message with compid=" + compid
                    + " and data=" + data + '\n');
            
            Protocol.writePushData(bos, compid, data.getBytes());
            byte[] b = bos.toByteArray();
            try {
                out.write(b);
                out.flush();
                return true;
            } catch (IOException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
                return false;
            }
        } else {
            int cmdr;
            try {
                cmdr = in.read();
            } catch (IOException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
                return false;
            }
            if (cmdr != cmd) {
                sop("ERROR: Expected a " + cmdname
                        + " message but got message command: " + (char) cmdr);
                return false;
            }
            Object[] stuff = Protocol.readPushData(in);
            int rcompid = ((Integer) stuff[0]).intValue();
            String rdata = new String((byte[]) stuff[1]);
            sop("Read a " + cmdname + " message with compid=" + rcompid
                    + " data=" + rdata + '\n');
            return true;
        }
    }
    
    /**created, delete, listcomponents, requestdata, 
     * subscribe and unsubscribe 
     */
    private static void writeSwitch(char c, ByteArrayOutputStream os, int type,
            int user, int compid) {
        switch (c) {
        case 'd':
            Protocol.writeCreated(os, type, user, compid);
            break;
        case 'l':
            Protocol.writeListComponents(os, type, user, compid);
            break;
        case 'r':
            Protocol.writeRequestData(os, type, user, compid);
            break;
        case 'x':
            Protocol.writeDelete(os, type, user, compid);
            break;
        case 's':
            Protocol.writeSubscribe(os, type, user, compid);
            break;
        case 'z':
            Protocol.writeUnsubscribe(os, type, user, compid);
            break;
        }
    }
    
    private static int[] readSwitch(char c, InputStream in) {
        switch (c) {
        case 'd':
            return Protocol.readCreated(in);
        case 'l':
            return Protocol.readListComponents(in);
        case 'r':
            return Protocol.readRequestData(in);
        case 'x':
            return Protocol.readDelete(in);
        case 's':
            return Protocol.readSubscribe(in);
        case 'z':
            return Protocol.readUnsubscribe(in);
        }
        return null;
    }
    
    private static void sop(Object o) {
        System.out.println("[SocketCommTest] " + o);
    }
}
