/*
 * BeTelGeuse - A context data gathering tool for Bluetooth devices
 * www.cs.helsinki.fi/group/acs/betelgeuse
 * Copyright (C) 2006 Joonas Kukkonen, Eemil Lagerspetz and Petteri Nurmi
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

package hiit.logger;

import javax.microedition.lcdui.*;

/**
 * This class handles the error and message loggin in midlets.
 * The messages are saved in a buffer. Calling getLogScreen()
 * returns a Form with the messages. The Form does not have any
 * commands attached to it, and it has not been set visible.
 * @author Joonas Kukkonen and Eemil Lagerspetz
 *
 */

public class MIDPLogger implements Logger {

        private String[] messages;
        private int index;
        private int counter;
        private int[]  entryNumbers;
        private int LOGLENGTH;
        private boolean logFull = false;
        
        /**
         * Creates a MIDPLogger object.
         * @param loglength the length of the message buffer.
         */
        
       
        public MIDPLogger(int loglength) {
        	  this.LOGLENGTH = loglength;
              this.messages = new String[LOGLENGTH];
              this.entryNumbers = new int[LOGLENGTH];
              index = 0;
              counter = 0;
        }

        /**
         * Creates a default MIDPLogger with buffer size of 30 messages.
         *
         */
        public MIDPLogger(){
        	this(30);
        }
        
        /**
         * Logs a message.
         */

        public void logMessage(String message) {
        		counter++;
        		if (index >= messages.length){
        			index = 0;
        			logFull = true;
        		}
        		this.messages[index] = message;
        		this.entryNumbers[index] = counter; 
        		index++;

        }

        /**
         * Does nothing in this implementation. The method is meant to 
         * write the buffer to the default output stream, which MIDP
         * devices do not often have.
         * @see hiit.logger.Logger#outputLog()
         */
        
        public void outputLog() {
        	
        }

        /**
         * Clears the log.
         */
        public void clearLog() {
        	this.messages = new String[LOGLENGTH];
        }
        
        /**
         * Does nothing in this implementation.
         * @see hiit.logger.Logger#flushLog()
         */
        
        public void flushLog() {
        }
        
        /**
         * Returns the buffer of the logger.
         * @return the buffer
         */
        
        public String[] getLog(){
        	return messages;
        }
        
        /**
         * Returns a From with the messages. The Form does not have any commands
         * attached to it and it has not been set visible. The caller must add
         * any needed commands, a CommandListener and set the Form visible.
         * @return a Form with the messages.
         */
        
        public Form getLogScreen(){
        	Form logForm = new Form("Log");
        	fillLogScreen(logForm);
         	return logForm;        	
        }
        
        protected void fillLogScreen(Form logForm){
        	if(logFull){
        		logForm.append("0: The log is full. Only showing the last " + LOGLENGTH + " messages.");
        		for (int i = index; i < messages.length; i++){
        			logForm.append(entryNumbers[i] + ": " + messages[i] + "\n");
        		}for (int i = 0; i < index; i++){
        			logForm.append(entryNumbers[i] + ": " + messages[i] + "\n");
        		}
        	}else
        		for (int i = 0; i < index; i++){
        			logForm.append(entryNumbers[i] + ": " + messages[i] + "\n");
        		}
        }
}
