/*
 * BeTelGeuse - A context data gathering tool for Bluetooth devices
 * www.cs.helsinki.fi/group/acs/betelgeuse
 * Copyright (C) 2006 Joonas Kukkonen, Eemil Lagerspetz and Petteri Nurmi
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
package hiit.mapper;

/**
 * This interface defines a method for identifying bluetooth devices. 
 * @author Petteri Nurmi, Joonas Kukkonen and Eemil Lagerspetz
 *
 */

public interface Mapper {
	
	
	/**
	 * Returns the bluetooth device ID.
	 * @param btAddress The address of the bluetooth device
	 * @param friendlyName The friendly name of the device
	 * @return The identification number
	 */
	public int getReaderIdentifier(String btAddress, String friendlyName);
	
	/**
	 * Updates the methods in the Mapper.
	 *
	 */
	
	public void updateRecords();
	
	/**
	 * Returns the recorder.
	 * @return the recorder.
	 */
	
	public Recorder getRecorder();
	
	/**
	 * Deletes a record, or sensor rule, from the Mapper using the Recorder object.
	 * @param deviceID the identification number of the device
	 * @param address the name, address, etc, which is used to identify the device.
	 * @return true if the deletion was succesfull, false otherwise.
	 */
	
	public boolean deleteRecord(int deviceID, String address);
	
	/**
	 * Saves a record in the format specified by the id number. See Recorder interface for the id numbers.
	 * The Mapper uses a Recorder to save the rule record.
	 * @param id The identification number of the saving format.
	 * @param sensorType The sensor type (an integer from 0 to 65535)
	 * @param sensorIdentifier The sensor identifier used to mach the sensor to the sensor type 
	 * @return true if the saving of the record succeeded, false otherwise
	 */
	
	public boolean saveRecord(int id, int sensorType, String sensorIdentifier);
	
	/**
	 * Returns true if there is a record (or rules) where friendly name is used
	 * to identify the device.
	 * @return true if there is a record that uses friendly name, false otherwise
	 */
	
	public boolean friendlyNameUsed();

}
