/*
 * BeTelGeuse - A context data gathering tool for Bluetooth devices
 * www.cs.helsinki.fi/group/acs/betelgeuse
 * Copyright (C) 2006 Joonas Kukkonen, Eemil Lagerspetz and Petteri Nurmi
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

package hiit.mapper;

import java.util.Hashtable;

/**
 * Recorder interface defines the methods for saving mapper rules. The interface
 * also defines ID numbers for record types. For example, 0x01 means the record
 * has bluetooth friendly name in addition to reader identification number, which
 * are defined in the Mapper interface.
 * @author Joonas Kukkonen and Eemil Lagerspetz
 *
 */


public interface Recorder {

	/**
	 * Saves a record in the format specified by the id number. See Recorder interface for the id numbers.
	 * @param id The identification number of the saving format.
	 * @param sensorType The sensor type (an integer from 0 to 65535)
	 * @param sensorIdentifier The sensor identifier used to mach the sensor to the sensor type 
	 * @return true if the saving of the record succeeded, false otherwise
	 */
	
	public boolean saveRecord(int id, int sensorType, String sensorIdentifier);
	
	//public Vector retrieve();

	/**
	 * This method returns all the device identifier - address / name pairs which are in the "id" format. For example
	 * if id is 1, the method returns all bluetooth sensors which are defined by bluetooth friendly name. Check the Recorder
	 * interface for possible id numbers. The maximum number id can have is 255 and the minimum 0. If the id is 0, all devices  identifier -
	 * address / name pairs in the repository are returned. 
	 * @param id - the identification number of the format of which the rules are saved in
	 * @return the found rules in a Hastable object
	 */	
	
	public Hashtable retrieve(int id);
	
	/**
	 * Deletes the record with the sensorType and sensorIdentifier.
	 * @param sensorType the sensor type of record to be deleted
	 * @param sensorIdentifier the record identifier of the record to be deleted
	 * @return true if the deletion of record succeeded, false otherwise
	 */
	
	public boolean deleteRecord(int sensorType, String sensorIdentifier);
	
	/**
	 * Closes the Recorder.
	 */
	
	public void close();
	
	/**
	 * Destroys the RecordStore the MIDPRecorder is using.
	 * @return true if RecordStore was destroyed, false otherwise
	 */
	
	public boolean destroy();
	
	public final int BT_FRIENDLY_NAME = 0x01;
	public final int BT_ADDRESS = 0x02;
}
