/*
 * BeTelGeuse - A context data gathering tool for Bluetooth devices
 * www.cs.helsinki.fi/group/acs/betelgeuse
 * Copyright (C) 2006 Joonas Kukkonen, Eemil Lagerspetz and Petteri Nurmi
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

package hiit.midpgui;

import java.util.Vector;
import javax.microedition.lcdui.Graphics;
/**
 * @author Petteri Nurmi
 */
public class TabPane extends TabCanvasItem  {

	private Vector _tabs;
	private int activeTab = 0;
		
	public TabPane() {
		_tabs = new Vector();
	}
	
	public void addTab(Tab _tab) {
		this._tabs.addElement(_tab);
	}
	
	public void removeTab(Tab _tab) {
		this._tabs.removeElement(_tab);
		this.tabLeft();
	}
	
	public void tabLeft() {		
		if(this.activeTab>0)
			this.activeTab--;
	}
	
	public void tabRight() {
		if(this.activeTab<(_tabs.size()-1))
			this.activeTab++;		
	}
	
	public Tab createTab(InformationObject _object) {
		Tab _t = new Tab(_object.getTitle());
		if(_t!=null)
			_tabs.addElement(_t);
		return _t;
	}
	
	public Tab getActiveTab() {
		if (_tabs.size() == 0)
			return null;
		return (Tab)this._tabs.elementAt(activeTab);
	}
	
	protected void draw(Graphics g, int x0, int y0, int width, int height) {
		g.setClip(x0,y0,width,height);
		g.setColor(255,231,184);
		g.fillRect(x0,y0,width,height);		
		// Draw the tabs
		int w25 = (int)(width/4);
		int[] _inte = selectTabs();
		if(_inte!=null) { 			
			for(int i=0,j=_inte.length;i<j;i++) {
				Tab _t = (Tab)_tabs.elementAt(_inte[i]);
				if(_inte[i]==activeTab)
					_t.setActive(true);
				else
					_t.setActive(false);
				if(_t!=null)
					_t.draw(g,x0 + (i * w25),y0, w25, height);
			}
		}		
		if(_tabs.size()>3) {
			g.setColor(75,5,5);
			g.setClip(x0,y0,width,height);
			// We need to draw a pointer to left or to right
			int h5 = (int)(height/20);
			//int w5 = (int)(0.05 * width);
			int w10 = (int)(width/10);
			//g.drawRect(x0+width+w5-w25,y0+h5+1,w25-w5-2,height-h5-2);
			if((activeTab-2)>=0 && (activeTab+2)<_tabs.size()) {
				// Draw both ways				
				g.fillTriangle(width-w10/2-1,y0+height-h5,width-w10/2-1,y0+h5,width-w10,y0+(height)/2);
				g.fillTriangle(width-w10/2+1,y0+height-h5,width-w10/2+1,y0+h5,width-2,y0+(height)/2);				
			}else if((activeTab+2)<_tabs.size()){
				// Draw to the right
				g.fillTriangle(width-w10/2+1,y0+height-h5,width-w10/2+1,y0+h5,width-2,y0+(height)/2);
			}else {
				// Draw to the left
				g.fillTriangle(width-w10/2-1,y0+height-h5,width-w10/2-1,y0+h5,width-w10,y0+(height)/2);
			}
		}
		g.setColor(155,155,215);
		g.drawLine(x0,y0+height,width,y0+height);
	}
	
	private int[] selectTabs() {
		if(_tabs.size()<3) {
			int l = _tabs.size();
			int[] _i = new int[l];
			for(int k=0;k<l;k++) {
				_i[k] = k;
			}
			return _i;
		}
		int startIndex = activeTab - 1;
		if(activeTab==_tabs.size()-1)
			startIndex = activeTab - 2;
		if(startIndex<0)startIndex = 0;
		int[] _i = new int[3];
		for(int k=0;k<3;k++) {
			_i[k] = startIndex + k;
		}
		System.gc();
		return _i;
	}
}
