/*
 * BeTelGeuse - A context data gathering tool for Bluetooth devices
 * www.cs.helsinki.fi/group/acs/betelgeuse
 * Copyright (C) 2006 Joonas Kukkonen, Eemil Lagerspetz and Petteri Nurmi
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
package hiit.observer;

import java.util.Vector;

/**
 * Interface class for push-style interaction with the BTDeviceConnector. Observer objects need to subscribe
 * to the connector, and they will receive notify() events when device queries are completed.
 * The connector sends current readers with the update(...) method, and then wakes the subscribers.
 * If the subscribing objects need something further than the list of readers, they can call the connector's 
 * other methods. This interface serves primarily as a way to negate unnecessary waits so that only the 
 * connector needs to look for devices.
 * 
 * @author Eemil Lagerspetz and Joonas Kukkonen
 *
 */
public interface ConnectorObserver {

	/**
	 * Sends the list of currently available readers to the Observer object.
	 * @param readers the vector of readers currently active. 
	 * @param connector A pointer to the BTDeviceConnector.
	 */
	public void update(Vector readers, Object connector);
	
}
