/*
 * BeTelGeuse - A context data gathering tool for Bluetooth devices
 * www.cs.helsinki.fi/group/acs/betelgeuse
 * Copyright (C) 2006 Joonas Kukkonen, Eemil Lagerspetz and Petteri Nurmi
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

package hiit.observer;

import hiit.reader.Reader;

import java.util.Vector;

/**
 * Interface class for push-style interaction with Readers. Observer objects need to subscribe
 * to the reader objects from which they want data. The Reader objects call the update(...) method
 * and pass the new data as a parameter. If an Observer has subscribed to multiple readers,
 * the programmer must make sure the different readers do not overwrite each other's data.
 * 
 * @author Eemil Lagerspetz and Joonas Kukkonen
 *
 */
public interface Observer {

	
	/**
	 * Sends data to the Observer object.
	 * @param data New data from the Reader object. 
	 * @param reader A pointer to the reader object.
	 */
	public void update(Vector[] data, Reader reader);
	
	
}
