/*
 * BeTelGeuse - A context data gathering tool for Bluetooth devices
 * www.cs.helsinki.fi/group/acs/betelgeuse
 * Copyright (C) 2006 Joonas Kukkonen, Eemil Lagerspetz and Petteri Nurmi
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
package hiit.plugin;

import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;

/**
 * An interface which defines a plugin. The communication and data
 * processing capabilities of Readers are limited. Plugins offer a way
 * to add features, which need a GUI, to the program without editing 
 * the code of core classes.
 * @author Joonas Kukkonen and Eemil Lagerspetz
 *
 */

public interface MIDPPlugin {

	/**
	 * Calling this method should cause the plugin to display its GUI.
	 * @param display the Display object of the device
	 * @param current the Displayable object that should be returned
	 * as current when user exits the plugin's GUI
	 */
	
	public void showPluginGui(Display display, Displayable current);
	
	/**
	 * Returns the name of the plugin.
	 * @return the name of the plugin
	 */
	
	public String getName();
	
	/**
	 * Closes the plugin.
	 */
	
	public void close();
}
