package hiit.plugin.activity;

import hiit.plugin.MIDPPlugin;

import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.Gauge;
import javax.microedition.lcdui.StringItem;
import javax.microedition.lcdui.TextField;

public class ActivityGUI2 implements MIDPPlugin, CommandListener {
	
	private ActivityPlugin2 plugin = null;
	
	private Displayable current = null;
	private Display display = null;
	
	private Form activityScreen = new Form("Activity plugin");
	private Form calibrationScreen = new Form("Sensor Calibration");
	private Form recordScreen = new Form("Data Recorder");
	
	private Alert sidesAlert = null;
	
	private Alert gAlert = new Alert("");
		
	//Stuff for the Activity Main Screen / General Stuff
	private Command ok = new Command("OK", Command.ITEM, 1);
	private Command back = new Command("Back", Command.BACK, 1);
	private Command close = new Command("Close", Command.ITEM, 1);
	private Command calibrate = new Command("Calibrate", Command.ITEM, 1);
	private Command record = new Command("Record Data", Command.ITEM, 1);
    private Command flush = new Command("Flush Buffer", Command.ITEM, 1);
  
    public StringItem activity = new StringItem("Activity", "");
    private StringItem mean = new StringItem("Mean", "");
    private StringItem variance = new StringItem("Variance", "");
    public StringItem norm = new StringItem("2-Norm", "");
    
	//Stuff for the Calibration Screen
	private Command startCalb = new Command("Start Calibration", Command.ITEM, 1);
	
	public StringItem Xmean = new StringItem("X", "");
    public StringItem Ymean = new StringItem("Y", "");
    public StringItem Zmean = new StringItem("Z", "");
    public StringItem msg = new StringItem("Status:", "Idle");
     
    public Gauge progress = new Gauge("Progress", false, 100, 0);
   	
	//Stuff for the Data Recorder
	private Command startRec = new Command("Record!", Command.ITEM, 1);
	private Command stopRec = new Command("Stop Recording", Command.ITEM, 1);
	public TextField filename  = new TextField("Filename:", ".csv", 20, 0);
    public StringItem time = new StringItem("Record Time:", "0");
	
	public ActivityGUI2(ActivityPlugin2 plugin){
		this.plugin = plugin;
		
		this.gAlert.setTimeout(Alert.FOREVER);
		
		this.activityScreen.addCommand(back);
		this.activityScreen.addCommand(close);
		this.activityScreen.addCommand(calibrate);
		this.activityScreen.addCommand(record);
        this.activityScreen.addCommand(flush);
		this.activityScreen.setCommandListener(this);
		this.activityScreen.append(norm);
        this.activityScreen.append("\n");
        this.activityScreen.append(activity);
        this.activityScreen.append("\n");
        this.activityScreen.append(mean);
        this.activityScreen.append("\n");
        this.activityScreen.append(variance);
        this.activityScreen.append("\n");
        
		this.sidesAlert = new Alert(null);
		this.sidesAlert.setTitle("Position sensor:");
		this.sidesAlert.addCommand(this.ok);
		this.sidesAlert.setTimeout(Alert.FOREVER);
		this.sidesAlert.setCommandListener(this);
		
        this.calibrationScreen.append(msg);
        this.calibrationScreen.append("\n");
        this.calibrationScreen.append(progress);
        this.calibrationScreen.append("\n");
        this.calibrationScreen.append(Xmean);
		this.calibrationScreen.append("\n");
		this.calibrationScreen.append(Ymean);
		this.calibrationScreen.append("\n");
		this.calibrationScreen.append(Zmean);
		this.calibrationScreen.append("\n");
        this.calibrationScreen.addCommand(back);
        this.calibrationScreen.addCommand(startCalb);
		this.calibrationScreen.setCommandListener(this);
		
		this.recordScreen.append(filename);
		this.recordScreen.append(time);
		this.recordScreen.addCommand(back);
		this.recordScreen.addCommand(startRec);
		this.recordScreen.setCommandListener(this);
		this.filename.insert("", 0); 
		this.gAlert.addCommand(this.ok);
	}
	
	public void close() {
		this.plugin.close();
	}

	public String getName() {
		return "Activity plugin";
	}

	public void showPluginGui(Display display, Displayable current) {
		this.current = current;
		this.display = display;
		display.setCurrent(this.activityScreen);
	}
	
	public void update(String activity, double[] features){
		//this.activity.setText(activity); //uncomment!
		this.variance.setText("" + features[1]);
		this.mean.setText("" + features[0]);
	}
	
	public void showGeneralAlert(String title, String msg){
	    plugin.alertState = true;
	    this.gAlert.setTitle(title);
	    this.gAlert.setString(msg);
	    this.display.setCurrent(gAlert, this.display.getCurrent());
	}
	
	public void showSidesAlert(String side){
	    plugin.alertState = false;
	    this.sidesAlert.setString("Please position the sensor on its " + side + " side");
	    this.display.setCurrent(sidesAlert, calibrationScreen);
	}
	
	public void startCalibrate(){
	    this.calibrationScreen.removeCommand(startCalb);
	    this.calibrationScreen.removeCommand(back);
	}
	
	//Function to restore the Calibration Screen - is called automatically by ActivityPlugin2.calibrate()
	public void stopCalibrate(){
        this.calibrationScreen.addCommand(startCalb);
        this.calibrationScreen.addCommand(back);
        this.msg.setText("Idle");
        this.progress.setValue(0);
	}
	
	public boolean startRecord(){
	    if (filename.getString() != "" || filename.getString() != ".csv"){
            this.filename.setConstraints(TextField.UNEDITABLE);
    	    this.recordScreen.removeCommand(startRec);
    	    this.recordScreen.removeCommand(back);
    	    this.recordScreen.addCommand(stopRec);
    	    return true;            
    	}
	    else return false;
	}
	
	public void stopRecord(){
	    this.filename.setConstraints(TextField.ANY);
	    this.recordScreen.removeCommand(stopRec);
        this.recordScreen.addCommand(startRec);
        this.recordScreen.addCommand(back);
	}

	public void commandAction(Command arg0, Displayable arg1) {
	    if(arg0 == this.back)
			if (activityScreen.isShown()) this.display.setCurrent(this.current);
			else this.display.setCurrent(activityScreen);
		else if(arg0 == this.close){
			this.plugin.close();
			this.display.setCurrent(this.current);
		}	
		else if (arg0 == this.calibrate){
		   // plugin.setRunMode(plugin.CALIBRATE);
		    this.display.setCurrent(this.calibrationScreen);
		}
		else if (arg0 == this.startCalb){
		    plugin.setRunMode(plugin.CALIBRATE);
		    //this.startCalibrate();
		}
		else if (arg0 == this.record){
		    //plugin.setRunMode(plugin.RECORD);
		    this.display.setCurrent(this.recordScreen);
		}
		else if (arg0 == this.flush){
		    plugin.flush();
		}
		else if (arg0 == this.startRec){
		    //startRecord();
		    this.plugin.setRunMode(plugin.RECORD);
		}
		else if (arg0 == this.stopRec){
		    this.plugin.setRunMode(plugin.CLASSIFY);
		}
		else if (arg0 == this.ok){
		    synchronized(plugin.alertLock){
		        plugin.alertState = true;
		        plugin.alertLock.notifyAll();
		    }
            if (arg1 == this.sidesAlert) this.display.setCurrent(calibrationScreen);
            else if (arg1 == this.gAlert) this.display.setCurrent(recordScreen);
		}
		else if (plugin.logger != null) plugin.logger.logMessage("[AP2] unknown: " + arg0.getLabel());
	}
}
