package hiit.plugin.activity;

import hiit.BetelgeuseMIDP;
import hiit.logger.Logger;
import hiit.mapper.Properties;
import hiit.observer.ConnectorObserver;
import hiit.observer.Observer;
import hiit.reader.LocalDeviceReader;
import hiit.reader.Reader;

import java.util.Vector;

public class ActivityPlugin implements ConnectorObserver, Observer, Runnable{
	
	protected ActivityGUI gui = null;
	protected Logger logger = null;

	protected int dataBatchSize = 75;
	protected Reader aliveECG = null;
	protected LocalDeviceReader localReader = null;
	protected int unprocessedData = 0;

	protected Vector xacc = new Vector();
	protected Vector yacc = new Vector();
	protected Vector zacc = new Vector();
	
	protected double[] features = null;
	protected String activity = "";

	protected boolean isRunning = true;

	public ActivityPlugin(BetelgeuseMIDP btg, Logger logger){
		gui = new ActivityGUI(this);
		btg.addPlugin(gui);
		btg.getDataGatherer().getConnector().subscribe(this);
		this.logger = logger;
	}

	public void update(Vector[] data, Reader reader) {
		if(unprocessedData > dataBatchSize)
			return;
		int amount = data[4].size();
		if(data[5].size() < amount)
			amount = data[5].size();
		if(data[6].size() < amount)
			amount = data[6].size();

		unprocessedData += amount;
		xacc.addElement(data[4]);
		yacc.addElement(data[5]);
		zacc.addElement(data[6]);
	}

	public synchronized void run(){
		if(this.logger != null) logger.logMessage("[AP] Activity plugin started");
		while(this.aliveECG == null)
			try {
				this.wait(10000);
			} catch (InterruptedException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}

		while(isRunning){
			//Wait until the plugin has enough data...
			while(unprocessedData < dataBatchSize){
				try {
					this.wait(500);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
			//ok, now the hard part: the data is in multiple vectors...

			int processed = 0;
			int x,y,z;
			int xmin = 128, xmax = -128, ymin = 128, ymax = -128, zmin = 128, zmax = -128;
			int sumx = 0, sumy = 0, sumz = 0;
			int sumXsqr = 0, sumYsqr = 0, sumZsqr = 0;
			
			while(processed < dataBatchSize && this.xacc.size()>0 && this.yacc.size()>0 && this.zacc.size()>0){
				Vector xData = (Vector)this.xacc.elementAt(0);
				Vector yData = (Vector)this.yacc.elementAt(0);
				Vector zData = (Vector)this.zacc.elementAt(0);

				int dataSize = xData.size();
				if(yData.size() < dataSize)
					dataSize = yData.size();
				if(zData.size() < dataSize)
					dataSize = zData.size();

				
				boolean remove = false;
				int limit;
				if(processed + dataSize <= dataBatchSize){
					limit = dataSize;
				}else{
					limit = dataBatchSize - processed;
				}
				for(int i=0; i < limit; i++){
					x=((Integer)xData.elementAt(i)).intValue()-128;
					y=((Integer)yData.elementAt(i)).intValue()-128;
					z=((Integer)zData.elementAt(i)).intValue()-128;
					
					if(x > xmax)
						xmax = x;
					if(y > ymax)
						ymax = y;
					if(z > zmax)
						zmax = z;
					if(x < xmin)
						xmin = x;
					if(y < ymin)
						ymin = y;
					if(z < zmin)
						zmin = z;
					
					sumx+=x;
					sumy+=y;
					sumz+=z;
					
					sumXsqr +=x*x;
					sumYsqr +=y*y;
					sumZsqr +=z*z;
					
				}
				xacc.removeElement(xData);
				yacc.removeElement(yData);
				zacc.removeElement(zData);
				processed +=dataSize;
				this.unprocessedData -= dataSize;
			}
			
			double[] temp = new double[12];
			
			temp[0] = sumx*1.0 / dataBatchSize;
			temp[1] = sumy*1.0 / dataBatchSize;
			temp[2] = sumz*1.0 / dataBatchSize;
			
			temp[3] = (sumXsqr - sumx * temp[0]) / (dataBatchSize -1);
			temp[4] = (sumYsqr - sumy * temp[1]) / (dataBatchSize -1);
			temp[5] = (sumZsqr - sumz * temp[2]) / (dataBatchSize -1);
			
			temp[6] = xmax;
			temp[7] = ymax;
			temp[8] = zmax;
			
			temp[9] = xmin;
			temp[10] = ymin;
			temp[11] = zmin;
			
			String tempActivity = classify(temp);
			if(this.localReader != null && !tempActivity.equals(this.activity))
				this.localReader.updateActivity(tempActivity);
			this.activity = tempActivity;
			this.features = temp;

			this.gui.update(this.activity, this.features);
		}

	}

	public String classify(double[] feat){
		double varianceSum = feat[3]/4096 + feat[4]/4096 + feat[5]/4096;
		if(varianceSum < 0.02)
			return "Sitting / Standing";
		if(varianceSum > 0.49)
			return "Running";
		return "Walking";
	}


	public void close() {
		this.isRunning = false;
	}

	public void update(Vector readers, Object connector) {
		if(this.aliveECG == null){
			for(int i=0; i < readers.size(); i++){
				Reader r = (Reader)readers.elementAt(i);
				if (r.getDeviceId() == Properties.ECG_ACC_BYTE_READER){
					this.aliveECG = r;
					r.subscribe(this, 0);
					if(this.logger != null) logger.logMessage("[AP] Subscribing to Alive Heart Monitor");
					break;
				}
			}
		}if(this.localReader == null){
			for(int i=0; i < readers.size(); i++){
				Reader r = (Reader)readers.elementAt(i);
				if (r.getDeviceId() == Properties.LOCAL_READER || r.getDeviceId() == Properties.GSM_READER){
					this.localReader = (LocalDeviceReader)r;
					break;
				}
			}
		}
	}
}
