package hiit.plugin.activity;

import hiit.BetelgeuseMIDP;
import hiit.logger.Logger;
import hiit.mapper.Properties;
import hiit.observer.ConnectorObserver;
import hiit.observer.Observer;
import hiit.plugin.activity.data.Data;
import hiit.plugin.activity.data.Instance;
import hiit.plugin.activity.utils.AccDataSource;
import hiit.reader.LocalDeviceReader;
import hiit.reader.Reader;

import java.io.OutputStream;
import java.util.Date;
import java.util.Vector;

import javax.microedition.io.Connector;
import javax.microedition.io.file.FileConnection;

/**
 * New and improved activity classifier plugin.
 * 
 * @author Eemil Lagerspetz and Marja Hassinen
 *
 */
public class ActivityPlugin2 extends AccDataSource implements ConnectorObserver, Observer, Runnable{

    private int dataBufferSize = 512;
    private int counter = 0;
    
    private String sides[] = {"top (flat)", "bottom (flat)", "left", "right", "top", "bottom"};
    public final int OFF = 0;
    public final int CLASSIFY = 1;
    public final int CALIBRATE = 2;
    public final int IDLE = 3;
    public final int RECORD = 4;
    
    private OutputStream out = null;
    private FileConnection fc = null;
    
    public boolean alertState = false;
    public final Object alertLock = new Object();
    private final Object notifier = new Object();
   
    private AriesTester classifier;
    
    double normSum = 0;
    double normSumSq = 0;
    protected ActivityGUI2 gui2 = null; 
    
    protected Logger logger = null;

    protected Reader aliveECG = null;
    protected LocalDeviceReader localReader = null;
    protected int unprocessedData = 0;

   // protected Vector xacc = new Vector(dataBatchSize);
 //   protected Vector yacc = new Vector(dataBatchSize);
  //  protected Vector zacc = new Vector(dataBatchSize);
    protected Vector xAcc = new Vector(dataBufferSize);
    protected Vector yAcc = new Vector(dataBufferSize);
    protected Vector zAcc = new Vector(dataBufferSize);

    protected double[] offset = {124.778, 129.684, 129.676};
    protected double[] scale = {0.02149611777662674, 0.020650728568986212, 0.021587082289957683};

//    protected double[] offset = {0,0,0};
//    protected double[] scale = {1,1,1};
    
    protected double[] temp = new double[6];

    protected double[] features = null;
    protected String activity = "";

    protected int runMode = CLASSIFY;
    protected BetelgeuseMIDP btg;

    public ActivityPlugin2(BetelgeuseMIDP betelgeuseMIDP, Logger logger){
        super(null, "ap2");
        gui2 = new ActivityGUI2(this);
        betelgeuseMIDP.addPlugin(gui2);
        betelgeuseMIDP.getDataGatherer().getConnector().subscribe(this);
        this.logger = logger;
        this.btg = betelgeuseMIDP;
        try {
            classifier = new AriesTester(this, logger);
        } catch (Exception e) {
            if (logger != null) logger.logMessage("[AP2] Exception: " + e.getMessage());
            e.printStackTrace();
            //gui2.showGeneralAlert("Exception", e.getMessage());
        }
    }

    public synchronized void run(){
        if(this.logger != null) logger.logMessage("[AP2] Activity plugin started");
        while(this.aliveECG == null){
            try {
                this.wait(1000);
            } catch (Exception e) {
                //gui2.showGeneralAlert("Exception", e.getMessage());
            }
            this.update(this.btg.getDataGatherer().getConnector().getConnectedDevices(), this.btg.getDataGatherer().getConnector());
        }
        while (runMode != OFF) {
            try {
                // switch(runMode){
                // case CLASSIFY:{
                if (runMode == CLASSIFY) {
                    this.gui2.activity.setText(classifier.classify());
                  //  break;
                }

                // case CALIBRATE: {
                else if (runMode == CALIBRATE) {
                    calibrate();
                    this.setRunMode(CLASSIFY);
//                    break;
                }

                // case IDLE: {
                else if (runMode == IDLE) {
                    synchronized (notifier) {
                        notifier.wait();
                    }
                 //   break;
                }

                // case RECORD: {
                else if (runMode == RECORD) {
                    record();
                    this.setRunMode(CLASSIFY);
                //    break;
                }

                // default:
                else
                    this.setRunMode(CLASSIFY);
            }
            catch (Exception e) {
                if (logger != null)
                    logger.logMessage("[AP2,mainloop] Exception: "
                            + e.getMessage());
                e.printStackTrace();
                counter++;
                if (counter > 3) break;
                this.setRunMode(CLASSIFY);
            }

        }
    }

    public void close() {
        setRunMode(OFF);
    }

    public void update(Vector readers, Object connector) {
        if(this.aliveECG == null){
            for(int i=0; i < readers.size(); i++){
                Reader r = (Reader)readers.elementAt(i);
                if (r.getDeviceId() == Properties.ECG_ACC_BYTE_READER){
                    this.aliveECG = r;
                    r.subscribe(this, 0);
                    if(this.logger != null) logger.logMessage("[AP] Subscribing to Alive Heart Monitor");
                    break;
                }
            }
        }if(this.localReader == null){
            for(int i=0; i < readers.size(); i++){
                Reader r = (Reader)readers.elementAt(i);
                if (r.getDeviceId() == Properties.LOCAL_READER || r.getDeviceId() == Properties.GSM_READER || r.getDeviceId() == Properties.LOCAL_GSM_READER){
                    this.localReader = (LocalDeviceReader)r;
                    break;
                }
            }
        }
    }

    public void setRunMode(int mode){
        runMode = mode;
        synchronized(notifier){
            notifier.notifyAll();
        }
    }

    public void flush(){
        if (logger != null) logger.logMessage("[AP2] Flushing data...");
        synchronized(xAcc){
            unprocessedData = 0;
            //xacc.removeAllElements();
            //yacc.removeAllElements();
            //zacc.removeAllElements();
            xAcc.removeAllElements();
            yAcc.removeAllElements();
            zAcc.removeAllElements();
        }
    }

    public Data getResult(){
//        if (runMode != CLASSIFY) return null;
        while (true)
        {
            if (unprocessedData < 1)
                synchronized (notifier){
                    try {
                        notifier.wait();
                    } catch (InterruptedException e) {
                        // TODO Auto-generated catch block
                        e.printStackTrace();
                    }
                }
            synchronized(xAcc){
                if (unprocessedData > 0)
                {
                    double [] temp = new double[3];
                    temp[0] = ((Double)xAcc.elementAt(0)).doubleValue();
                    temp[1] = ((Double)yAcc.elementAt(0)).doubleValue();
                    temp[2] = ((Double)zAcc.elementAt(0)).doubleValue();                    
                    xAcc.removeElementAt(0);
                    yAcc.removeElementAt(0);
                    zAcc.removeElementAt(0);
                    unprocessedData = xAcc.size();
                    //gui2.activity.setText(unprocessedData + "/" + ++counter);
                    return new Instance(temp);
                }
            }
            
        }
    }
    
    private void record() throws Exception{
        if (!gui2.startRecord()){
            this.setRunMode(CLASSIFY);
            return;
        }
            String url = "file:///e:/ecg." + gui2.filename.getString();
        this.fc = (FileConnection) Connector.open(url);
        // Ok... Now we have a connection, but the file might not exists (doh!). Check the existence
        // and create a file if needed
        if (!this.fc.exists())
            this.fc.create();
        // Yikes... There isn't a write method that appends data to the end of the file, so we need
        // to open the
        // outputstream to a specific location inside the file
        long size = this.fc.fileSize();
        if (size == -1)
            this.out = this.fc.openOutputStream();
        else
            this.out = this.fc.openOutputStream(size);
               
        this.flush();
        Date startTime = new Date();
               
        while(true){
            while (unprocessedData > 0) {
                synchronized (xAcc) {
                    this.out.write(((Double) xAcc.elementAt(0) + ","
                            + (Double) yAcc.elementAt(0) + ","
                            + (Double) zAcc.elementAt(0) + "\n").getBytes());
                    xAcc.removeElementAt(0);
                    yAcc.removeElementAt(0);
                    zAcc.removeElementAt(0);
                    unprocessedData--;

                }
            }
            
            this.gui2.time.setText((int)((new Date()).getTime() - startTime.getTime())/1000 + "");
            
            out.flush();

            synchronized (notifier) {
                notifier.wait();
            }
            if (runMode != RECORD) {
                out.flush();
                out.close();
                //fc.close();
                gui2.stopRecord();
                return;
            }
        }
    }

    private void calibrate() throws Exception{
        double diffs[] = new double[3];
        double Xsum = 0;
        double Ysum = 0;
        double Zsum = 0;
        
        gui2.startCalibrate();

        for (int i = 0; i < 3; i++){
            offset[i] = 0.0;
            scale[i] = 1.0;
        }

        for (int i = 0; i < 6; i++){
            gui2.showSidesAlert(sides[i]);
            synchronized(alertLock){
                while (!alertState){
                    alertLock.wait();
                  
                }
            }

            gui2.msg.setText("One Moment...");

            this.wait(2000);
                        
            this.flush();
            gui2.msg.setText("Collecting Data...");

            //Wait until the plugin has enough data...
            while(unprocessedData < 500){
                synchronized (notifier) {
                    notifier.wait();
                    gui2.progress.setValue(unprocessedData / 5);
                }
            }
            
            synchronized(xAcc){
                for (int j = 0; j < 500; j++){
                    Xsum += ((Double)xAcc.elementAt(j)).doubleValue();
                    Ysum += ((Double)yAcc.elementAt(j)).doubleValue();
                    Zsum += ((Double)zAcc.elementAt(j)).doubleValue();
                }
                
                Xsum = Xsum/500;
                Ysum = Ysum/500;
                Zsum = Zsum/500;
                gui2.Xmean.setText("" + (Xsum));
                gui2.Ymean.setText("" + (Ysum));
                gui2.Zmean.setText("" + (Zsum));

                diffs[0] = Xsum - (Ysum + Zsum)/2;
                diffs[1] = Ysum - (Zsum + Xsum)/2;
                diffs[2] = Zsum - (Xsum + Ysum)/2;

                if (Math.abs(diffs[0]) > Math.abs(diffs[1]) && Math.abs(diffs[0]) > Math.abs(diffs[2])){
                    if (diffs[0] > 0) temp[0] = Xsum;
                    if (diffs[0] < 0) temp[1] = Xsum;
                }
                if (Math.abs(diffs[1]) > Math.abs(diffs[2]) && Math.abs(diffs[1]) > Math.abs(diffs[0])){
                    if (diffs[1] > 0) temp[2] = Ysum;
                    if (diffs[1] < 0) temp[3] = Ysum;
                }
                if (Math.abs(diffs[2]) > Math.abs(diffs[0]) && Math.abs(diffs[2]) > Math.abs(diffs[1])){
                    if (diffs[2] > 0) temp[4] = Zsum;
                    if (diffs[2] < 0) temp[5] = Zsum;
                }
            }
        }
        synchronized(xAcc){
            for (int i = 0; i < 3; i++){
                offset[i] = (temp[2*i] + temp[2*i + 1])/2;
                scale[i] = 1/(temp[2*i] - offset[i]);
            }
    
            for (int i = 0; i < 6; i++){
                temp[i] = (temp[i] - offset[i/2]) * scale[i/2];
            }
        }
        
        this.flush();
        gui2.showGeneralAlert("Results:", "Offset X: " + offset[0] + "\n" + "Offset Y: " + offset[1] + "\n" + "Offset Z: " + offset[2]);
        gui2.stopCalibrate();
        setRunMode(CLASSIFY);
    }

    public void dataNotify(){
        synchronized(notifier){
            notifier.notifyAll();
        }
    }
    
    public void update(Vector[] data, Reader reader) {
        // logger.logMessage("[AP2] Received data "+ (data != null ? data.length : -1) + " from reader.");
        if(unprocessedData >= dataBufferSize){
            synchronized(notifier){
                notifier.notifyAll();
            }
            return;
        }
        
        //Add the offset-corrected and scaled sensor readings to the data vectors
        synchronized(xAcc){
            for (int i = 0; i < data[4].size(); i++){
                xAcc.addElement(new Double((((Integer)data[4].elementAt(i)).doubleValue() - offset[0])*scale[0]));
            }
            for (int i = 0; i < data[5].size(); i++){
                yAcc.addElement(new Double((((Integer)data[5].elementAt(i)).doubleValue() - offset[1])*scale[1]));
            }
            for (int i = 0; i < data[6].size(); i++){
                zAcc.addElement(new Double((((Integer)data[6].elementAt(i)).doubleValue() - offset[2])*scale[2]));
            }
    
            int amount = xAcc.size();
            if(yAcc.size() < amount)
                amount = yAcc.size();
            if(zAcc.size() < amount)
                amount = zAcc.size();
    
            unprocessedData = amount;
        }

        // xacc.addElement(data[4]);
        // yacc.addElement(data[5]);
        // zacc.addElement(data[6]);

        gui2.norm.setText(Math.sqrt(((Double)xAcc.lastElement()).doubleValue()*((Double)xAcc.lastElement()).doubleValue() + ((Double)yAcc.lastElement()).doubleValue()*((Double)yAcc.lastElement()).doubleValue() +((Double)zAcc.lastElement()).doubleValue()*((Double)zAcc.lastElement()).doubleValue()) + "");
        synchronized(notifier){
            notifier.notifyAll();
        }
    }
}
