package hiit.plugin.activity;

import hiit.logger.Logger;
import hiit.plugin.activity.classifier.BoostStump;
import hiit.plugin.activity.norm.Norm_2;
import hiit.plugin.activity.utils.AccDataSource;
import hiit.plugin.activity.utils.Module;
import hiit.plugin.activity.utils.TreeMan;
import hiit.plugin.activity.utils.Windower;

public class AriesTester {

    private AccDataSource dataSource = null;
    private Logger logger = null;
    private Windower windower = null;
    private TreeMan tree = null;

    private int lastres = 8;

    private String[] activities = { "walk", "bwalk", "jog", "run", "car",
            "bus", "metro", "rest", "other" };

    private BoostStump bus = null;
    private BoostStump bwalk = null;
    private BoostStump car = null;
    private BoostStump human = null;
    private BoostStump jog = null;
    private BoostStump metro = null;
    private BoostStump motor = null;
    private BoostStump rest = null;
    private BoostStump run = null;
    private BoostStump walk = null;

    public AriesTester(AccDataSource dataSource, Logger logger)
            throws Exception {
        this.dataSource = dataSource;
        this.logger = logger;
        init();
    }

    public void init() throws Exception {
        tree = new TreeMan();

        // features = new Vector();

        Module current = null;
        current = dataSource;
        windower = new Windower((AccDataSource) dataSource, new Norm_2(), 256,
                64, 0);
        current.addChild(windower);
        tree.setRoot(windower);

        bus = new BoostStump("bus", tree);
        bwalk = new BoostStump("bwalk", tree);
        car = new BoostStump("car", tree);
        human = new BoostStump("human", tree);
        jog = new BoostStump("jog", tree);
        metro = new BoostStump("metro", tree);
        motor = new BoostStump("motor", tree);
        rest = new BoostStump("rest", tree);
        run = new BoostStump("run", tree);
        walk = new BoostStump("walk", tree);

    }

    public String classify() throws Exception {
        int res = 8;

        if (human.classify() >= 0.0) {
            double walkres = walk.classify();
            double bwalkres = bwalk.classify();
            double jogres = jog.classify();
            double runres = run.classify();
            double max = -Double.MAX_VALUE;
            if (walkres >= 0.0) {
                if (lastres == 0) {
                    res = 0;
                    max = Double.MAX_VALUE; //make sure nobody steals the result from us!
                }

                if (walkres >= max) {
                    max = walkres;
                    res = 0;
                }
            }
            if (bwalkres >= 0.0) {
                if (lastres == 1) {
                    res = 1;
                    max = Double.MAX_VALUE;
                }

                if (bwalkres >= max) {
                    max = bwalkres;
                    res = 1;
                }
            }
            if (jogres >= 0.0) {
                if (lastres == 2) {
                    res = 2;
                    max = Double.MAX_VALUE;
                }

                if (jogres >= max) {
                    max = jogres;
                    res = 2;
                }
            }
            if (runres >= 0.0) {
                if (lastres == 3) {
                    res = 3;
                    max = Double.MAX_VALUE;
                }

                if (runres >= max) {
                    max = runres;
                    res = 3;
                }
            }
        } else {
            double max = -Double.MAX_VALUE;
            if (motor.classify() >= 0.0) {
                double carres = car.classify();
                double busres = bus.classify();
                double metrores = metro.classify();
                if (carres >= 0.0) {
                    if (lastres == 4) {
                        res = 4;
                        max = Double.MAX_VALUE;
                    }

                    if (carres >= max) {
                        max = carres;
                        res = 4;
                    }
                }
                if (busres >= 0.0) {
                    if (lastres == 5) {
                        res = 5;
                        max = Double.MAX_VALUE;
                    }

                    if (busres >= max) {
                        max = busres;
                        res = 5;
                    }
                }
                if (metrores >= 0.0) {
                    if (lastres == 6) {
                        res = 6;
                        max = Double.MAX_VALUE;
                    }

                    if (metrores >= max) {
                        max = metrores;
                        res = 6;
                    }
                }
            }
            double restres = rest.classify();
            if (restres >= 0.0)
                if (restres > max || lastres == 7) {
                    res = 7;
                }
        }
        lastres = res;

        dataSource.reset();

        // return result;
        return activities[res];
    }

}
