package hiit.plugin.activity.classifier;

import java.util.Vector;

import hiit.plugin.activity.data.Instance;
import hiit.plugin.activity.feature.Feature;
import hiit.plugin.activity.utils.ResReader;
import hiit.plugin.activity.utils.StringTokenizer;
import hiit.plugin.activity.utils.TreeMan;

//class for classifying according to boosted decision stumps
//only binary classifiers supported atm, one class _has_ to be called "other"


public class BoostStump extends Classifier{
    private Vector nodes = new Vector();
    public BoostStump(String name, TreeMan tree) throws Exception{
        readModel(name, tree);
    }
    
    public double classify() throws Exception{
        double result = 0;
        for (int i = 0; i < nodes.size(); i++){
            result += ((DecisionNode)nodes.elementAt(i)).classify();
        }
        return result;
    }
    
    //things are in the form
    //Classifications
    //FFT.ABS.MAXPOS0 <= 0.40678000000000003 : run
    //FFT.ABS.MAXPOS0 > 0.40678000000000003 : other
    //Weight: 2.29
    //but watch out - sometimes it's the same class twice!
    private void readModel(String fileName, TreeMan tree) throws Exception{
        StringTokenizer st;
        ResReader reader = new ResReader(fileName + ".boost");
        Feature feat = null;
        String line;
        String temp;
        boolean first = true;
        boolean second = true;
        double thresh;
        double weight;
        
        while((line = reader.readLine()) != null){
            if (line.equals("Classifications")){
                first = false; //true: first-> class. false: first->other
                second = false; //dito
                //do the first line of content
                line = reader.readLine();
                st = new StringTokenizer(line);
                temp = st.nextToken();
                for (int i = 0; i < temp.length(); i++){
                    if (temp.charAt(i) == '0' || temp.charAt(i) == '('){
                        temp = temp.substring(0, i);
                        feat = (Feature)tree.addModule(temp);
                        break;
                    }
                }
                st.nextToken();
                temp = st.nextToken();
                thresh = Double.parseDouble(temp);
                while (st.hasMoreTokens()) temp = st.nextToken();
                if (!temp.equals("other")) first = true;
                //now for the second line - we only care for the class
                line = reader.readLine();
                st = new StringTokenizer(line);
                while (st.hasMoreTokens()) temp = st.nextToken();
                if (!temp.equals("other")) second = true;
                line = reader.readLine();
                st = new StringTokenizer(line);
                while (st.hasMoreTokens()) temp = st.nextToken();
                weight = Double.parseDouble(temp);
                nodes.addElement(new DecisionNode(feat, thresh, weight, second, first == second));
                
            }
        }
        first = false;
    }
    
    private class DecisionNode{
        private Feature feature = null;
        private double threshold = 0;
        private double weight = 0;
        private boolean bigger = true;
        private boolean same = false;
        
        public DecisionNode(Feature feature, double threshold, double weight, boolean bigger, boolean same) throws Exception{
            if (feature == null) throw new Exception("DecisionNode initialized without feature");
            this.feature = feature;
            this.threshold = threshold;
            this.weight = weight;
            this.bigger = bigger;
            this.same = same;
        }
        
        public double classify() throws Exception{
            if (same){
                if (bigger) return weight; //if same && bigger: return class
                else return -weight; //return anti-class
            }
            double featVal = ((Instance)feature.getResult()).getValues()[0];
            if (featVal > threshold){
                if (bigger) return weight;
                else return -weight;
            }
            else {
                if (bigger) return -weight;
                else return weight;
            }
        }
        
        
        
    }
}


