package hiit.plugin.activity.classifier;

import java.util.Vector;

import hiit.logger.Logger;
import hiit.plugin.activity.data.Instance;
import hiit.plugin.activity.feature.Feature;
import hiit.plugin.activity.svm.svm;
import hiit.plugin.activity.svm.svm_model;
import hiit.plugin.activity.svm.svm_node;
import hiit.plugin.activity.svm.svm_parameter;
import hiit.plugin.activity.utils.ResReader;
import hiit.plugin.activity.utils.StringTokenizer;
import hiit.plugin.activity.utils.TreeMan;

public class SVM {

    private svm_model model = null;
    private Feature[] features = null;
    private double[][] coeff = null;
    private int svm_type;
    private int nr_class;
    private Logger logger;

    public SVM(String fileName, TreeMan tree, Logger logger) throws Exception {
        this.logger = logger;
        long time = System.currentTimeMillis();
        model = svm.svm_load_model(fileName + ".model");
        time = System.currentTimeMillis() - time;
        if (logger != null) logger.logMessage("svm-coeffs read in " + time);
        
        readFeatures(fileName, tree);
        readCoeff(fileName);
                
        svm_type = svm.svm_get_svm_type(model);
        nr_class = svm.svm_get_nr_class(model);
        
    }
    
    public double classify(int predict_probability) throws Exception {

        double[] prob_estimates = null;
        prob_estimates = new double[2];

        svm_node[] x = new svm_node[features.length];
        for (int j = 0; j < features.length; j++) {
            x[j] = new svm_node();
            x[j].index = j + 1;
            x[j].value = coeff[1][j] + coeff[0][j]*((Instance) features[j].getResult()).getValues()[0];
        }

        double v;
        if (predict_probability == 1 //does not work for some reason... :-(
                && (svm_type == svm_parameter.C_SVC || svm_type == svm_parameter.NU_SVC)) {
            v = svm.svm_predict_probability(model, x, prob_estimates);
            return prob_estimates[1];
        } else {
            v = svm.svm_predict(model, x);
            return v;
        }
    }
       
    private void readCoeff(String fileName) throws Exception{
        ResReader reader = new ResReader(fileName + ".param");
        reader.readLine(); //first line is crap
        reader.readLine(); //second line is uninteresting
        String line;
        StringTokenizer tok;
        
        Vector lower = new Vector();
        Vector upper = new Vector();
        double lowerd;
        double upperd;
        
        while((line = reader.readLine()) != null){
            tok = new StringTokenizer(line);
            if (tok.countTokens() == 0) continue;
            if (tok.countTokens() != 3 ) throw new Exception("Unexpected stuff in param file");
            tok.nextToken();
            lower.addElement(tok.nextToken());
            upper.addElement(tok.nextToken());
        }
        
        coeff = new double[2][lower.size()];
        
        for (int i = 0; i < lower.size(); i++){
            lowerd = Double.parseDouble((String) lower.elementAt(i));
            upperd = Double.parseDouble((String) upper.elementAt(i));
            coeff[0][i] = 2 / (upperd - lowerd); //factor
            coeff[1][i] = - (upperd + lowerd) / (upperd - lowerd); //shift
        }    
        
    }
    
    private void readFeatures(String fileName, TreeMan tree) throws Exception{
        ResReader reader = new ResReader(fileName + ".feats");
        String line;
        Vector feats = new Vector();
        while((line = reader.readLine()) != null && line != ""){
            feats.addElement(tree.addModule(line));
        }
        features = new Feature[feats.size()];
        for (int i = 0; i < features.length; i++){
            features[i] = (Feature) feats.elementAt(i);
        }
    }

}
