package hiit.plugin.activity.feature;

import hiit.plugin.activity.data.Data;
import hiit.plugin.activity.data.Instance;
import hiit.plugin.activity.data.Series;
import hiit.plugin.activity.utils.Module;

/** 
 * Base class for all features. 
 * @author ptnurmi
 * @version 1.0
 */
public abstract class Feature extends Module {
	
	/**
	 * The corresponding data transform
	 */
	
	protected int transformId;
	
	/**
	 * The value of the feature. 
	 */
	protected Instance result;	
	
	/**
	 * Create a new feature with a given name. 
	 * @param name The name of the feature.
	 */
	public Feature(Module parent, String name) {
		super(parent, name);
		setParent(parent);
	    setName(name);	
		setTransform(transformId);
	}
	
	/**
     * Set the corresponding transform-id of the feature.
     * @param transformId The transform-id of the feature.
     */
    public void setTransform(int transformId){
        this.transformId = transformId;
    }
    
    /**
     * Set the result value to be null, s.t. the value will be re-calculated next call.
     */
    public void resetThis(){
        this.result = null;
    }
	
	/**
	 * Calculates the value of the feature. At this point 
	 * checks have been made that the instances are valid so the feature has to 
	 * check that the (i) values are valid and (ii) compute the value of the feature. 
	 * @return The value of the feature.
	 */
	public Data getResult() throws Exception{
        if (result != null) return result;
        Series source = (Series) parent.getResult();
        double[] temp = getFeatResult(source.getData());
        result = new Instance(temp);
        result.setLabel(source.getLabel());
        return result;
	}
	
	protected double[] getFeatureValues(String name) throws Exception{
	    Feature temp = (Feature)parent.getChild(name);
	    if (temp == null) throw new Exception("Tried to access uninitialized feature, name was " + name);
	    return ((Instance)temp.getResult()).getValues();
	}
		
	protected abstract double[] getFeatResult(Instance[] source) throws Exception;
}