package hiit.plugin.activity.feature.statistical;

import hiit.plugin.activity.data.Instance;
import hiit.plugin.activity.feature.Feature;
import hiit.plugin.activity.utils.Module;

public class Variance extends Feature {

    public Variance(Module parent) {
        super(parent, "VAR");
        dependencies = new String[] {"MEAN"};
    }

    protected double[] getFeatResult(Instance[] source) throws Exception {

        // Create storage
        double[] result = new double[source[0].getDimensionality()];

        // The number of datapoints within the window
        final double divisor = source.length - 1.0;

        final double[] _mean = getFeatureValues(dependencies[0]);

        // Loop through the values and compute estimate
        for (int i = 0; i < source.length; i++) {

            // Get the next instance
            final double[] _instanceValues = source[i].getValues();

            // Update estimates
            for (int j = 0; j < _instanceValues.length; j++) {

                // The first estimate computes the squared sum \sum y_i^2
                result[j] += (_instanceValues[j] * _instanceValues[j]);
            }
        }

        // Combine results: the variance is given by
        // 1 / (n - 1) * \sum y_i^2 - n / (n - 1) * \hat{y}^2
        // where \hat{y} is the mean
        for (int j = 0; j < result.length; j++) {
            result[j] = result[j] / divisor - source.length * (_mean[j] * _mean[j]) / divisor;
        }

        // Return results
        return result;
    }

}
