package hiit.plugin.activity.norm;

import hiit.plugin.activity.data.Instance;

public abstract class Norm {

    /**
     * Calculates the norm over a set of values
     * @param instances
     *          Input is an array of instances to normify
     * @return 
     *          Returns an array of new instances containing the norms
     */
    public Instance[] calculateNorm(Instance[] instances) {
        if (instances == null)
            return null;
        Instance[] result = new Instance[instances.length];
        for (int i = 0; i < instances.length; i++) {
            result[i] = calculateNorm(instances[i]);
        }
        return result;
    }

    /**
     * Calculates the norm of one instance
     * Retains the label of the instance
     * @param instance
     *          The instance to calculate the norm of
     * @return
     *          A new instance containing the norm
     */
    public Instance calculateNorm(Instance instance) {
        if (instance == null)
            return null;
        Instance result = new Instance(calculateNorm(instance.getValues()));
        result.setLabel(instance.getLabel());
        return result;
    }
    
    public abstract double[] calculateNorm(double[] samples);

    /**
     * Function to get the number of out-channels of the norm
     * returns 0 if unknown, e.g. for the none-norm    
     * @return number of out-channels, or 0 if unknown.
     */
    public abstract int outChannels();
}