package hiit.plugin.activity.utils;

public class MathUtils {
	
	/**
	 * Constant used in FFT calculations.
	 */
	public static final double _PI = Math.PI;
	
	/**
	 * Constant used in FFT calculations.
	 */
	public static final double _2PI = 2.0 * Math.PI;
	
	public static final double _POW(double _input, int _power) {
		double _retVal = 1.0;
		for(int i=0;i<_power;i++) {
			_retVal *= _input;
		}
		return _retVal;
	}
	
	public static final double[][] _HEAPSORT(double[][] _array) {
		
		final int _n = _array.length;
		final int _d = _array[0].length;
		
		double _tmp = 0.0;
	
		// Loop through the dimensions
		for(int j=0;j<_d;j++) {
			
			int _N = _n;
			
			// Step 1. Build the heap structure
			for(int _v = (_n /2 - 1);_v>=0;_v--) {
				
				// Perform downheap with parameters: v, _N
				int _w = (_v << 1) + 1;
				while(_w < _N) {
					
					if(_w < (_N - 1) && _array[_w][j] < _array[_w+1][j])
						_w++;
					
					if(_array[_w][j]<=_array[_v][j])break;
					
					// Swap elements v and w
					_tmp = _array[_w][j];
					_array[_w][j] = _array[_v][j];
					_array[_v][j] = _tmp;
					
					_v = _w;
					_w = (_v << 1) + 1;
					
				}				
			}
			
			// Step 2. Order the heap structure
			while(_N > 1) {
				
				_N--;
				
				// Swap elements 0 and _N - 1
				_tmp = _array[_N][j];
				_array[_N][j] = _array[0][j];
				_array[0][j] = _tmp;
				
				// Update heap structure, call downheap: 0, _N
				int _v = 0;
				int _w = 1;
				
				while(_w < _N) {
					
					if(_w < (_N - 1) && _array[_w][j] < _array[_w+1][j])
						_w++;
					
					if(_array[_w][j]<=_array[_v][j])break;
					
					// Swap elements v and w
					_tmp = _array[_w][j];
					_array[_w][j] = _array[_v][j];
					_array[_v][j] = _tmp;
					
					_v = _w;
					_w = (_v << 1) + 1;
					
				}			
								
			}

		}
			
				
		return _array;		
	}
	
	public static double[][] FFT(double[][] _data) {
		
		final int _n = _data.length;
		final int _d = _data[0].length;
		final int _nn = _n * 2;
		
		double[][] _tempData = new double[_nn][_d];
		
		for(int i=0;i<_nn;i+=2) {
			System.arraycopy(_data[i/2],0,_tempData[i],0,_d);
		}
		
		return Fourier(_tempData, _n);
	}
	
	/**
	 * Performs
	 * @param _data
	 * @return result
	 */
	public static double[][] RealFFT(double[][] _data) {
		
		double _wtemp, _wpr, _wpi, _wr, _wi, _h1r, _h1i, _h2r, _h2i;
		int i, _i1, _i2, _i3, _i4;
		
		final int _n = _data.length;
		final int _d = _data[0].length;
				
		_data = Fourier(_data, _n>>1);
		
		final double _theta = _PI / (double)(_n >> 1);
		final double _c1 = 0.5;
		final double _c2 = -0.5;
		
		final int _np3 = _n + 3;
		
		for(int _col = 0; _col < _d; _col++) {
				
			_wtemp = Math.sin(0.5 * _theta);
			_wpr = -2.0 * _wtemp * _wtemp;
			_wpi = Math.sin(_theta);
			
			_wr = 1.0 + _wpr;
			_wi = _wpi;
		
			// Loop through the data
			// The case i = 1 is a special case and it is handled separately. 
			for(i = 2; i<=(_n>>2);i++) {
				
				// Compute the indices that are used to combine datapoints
				_i1 = i + i - 1;	// Index to the real part of the coefficient H_n
				_i2 = _i1 + 1;		// Index to the complex part of the coefficient H_n
				_i3 = _np3 - _i2;	// Index to the real part of the coefficient H_(N/2 - n)
				_i4 = 1 + _i3;		// Index to the complex part of the coefficient H_(N/2 - n)
				
				_h1r = _c1 * (_data[_i1 - 1][_col] + _data[_i3 - 1][_col]);
				_h1i = _c1 * (_data[_i2 - 1][_col] - _data[_i4 - 1][_col]);
			
				_h2r = -_c2 * (_data[_i2 - 1][_col] + _data[_i4 - 1][_col]);
				_h2i = _c2 * (_data[_i1 - 1][_col] - _data[_i3 - 1][_col]);
								
			
				_data[_i1-1][_col] = _h1r + _wr * _h2r - _wi * _h2i;
				_data[_i2-1][_col] = _h1i + _wr * _h2i + _wi * _h2r;
				_data[_i3-1][_col] = _h1r - _wr * _h2r + _wi * _h2i; 
				_data[_i4-1][_col] = -_h1i + _wr * _h2i + _wi * _h2r;
			
				_wtemp = _wr;
				_wr = _wr * _wpr - _wi * _wpi + _wr;
				_wi = _wi * _wpr + _wtemp * _wpi + _wi;
				
			}
			
			_h1r = _data[0][_col];
			_data[0][_col] = _data[0][_col] + _data[1][_col];
			_data[1][_col] = _h1r - _data[1][_col];
			
		}
		
		return _data;		
	}
	
	protected static double[][] Fourier(double[][] _data, int _nn) {
		
		// The dimensionality of the data
		final int _d = _data[0].length;
		
		// The number of data points for the Fourier transform. Each 
		// input point is assumed to be complex valued and hence there 
		// are 2.0 * _nn points. 
		final int _n = _nn << 1;
		
		// Perform the transform on each column separately 
		for(int _col=0;_col<_d;_col++) {
			
			// ************************************************
			// Step 0. Initialization
			// ************************************************
			
			int _j = 1, _m = 0, _mmax, _istep = 1;
			double _theta = 0.0, _wtemp = 0.0, _wpr = 0.0, _wpi, _wi, _wr, _tempr, _tempi;
			
			// ************************************************
			// Step 1. Arrange the data into bit reverse order
			// ************************************************			
			
			// Loop through the data and swap data points to obtain the bit reverse order
			// The data points are assumed to be so that d[i] contains the real part of 
			// a datapoint and d[i+1] contains the imaginary part of the data point. 
			// Thus we loop over the real parts of the data
			for(int i=1;i<_n;i+=2) {
				
				// Determine if we need a swap and if so swap elements
				if(_j > i) {
					
					// Swap elements j-1 and i-1, these correspond to the real part
					_wtemp = _data[_j-1][_col];
					_data[_j-1][_col] = _data[i-1][_col];
					_data[i -1][_col] = _wtemp;
					
					// Swap elements j and i, these correspond to the imaginary part
					_wtemp = _data[_j][_col];
					_data[_j][_col] = _data[i][_col];
					_data[i][_col] = _wtemp;
						
				}
				
				// Init m to be the middle point
				_m = _n>>1;

				// The ordering is performed recursively. 
				while(_m >=2 && _j > _m) {
					_j -= _m;
					_m >>=1;
				}
				_j+=_m;
					
			}
			
			// *******************************************************
			// Step 2. Apply recursively the Danielson-Lanczos Lemma
			// *******************************************************
			
			// The term _mmax determines the length of the transform, i.e., 
			// _mmax = N in the Fourier transform equations
			_mmax = 2;
			
			// Outer loop, perform until we have performed a transformation for the 
			// entire window
			while(_n > _mmax){								
				
				// Theta stores the angle: 2 * PI / N
				_theta = _2PI / (double)_mmax;
				
				// Determine the next value of N. Determines the window over which 
				// the values are combined
				_istep = (_mmax << 1);
				
				// Calculate sin(PI / N) and sin(2 * PI / N)
				_wtemp = Math.sin(0.5 * _theta);
				_wpi = Math.sin(_theta);
				
				
				_wpr = -2.0 * _wtemp * _wtemp;				
				_wr = 1.0;
				_wi = 0.0;
				
				// Inner loop, 
				for(_m = 1; _m < _mmax; _m+=2) {
					
					// Apply the lemma over the different windows of size N. N determines
					// the number of datapoints to consider and since each of the datapoints 
					// is assumed to be complex valued, we need to consider 2 * N 
					// instances. This is why we increase the counter in steps of 2N
					for(int i = _m;i<=_n;i+=_istep) {
						
						// The variable i determines the recurrence point for 
						// the even terms and the variable j determines the recurrence 
						// point for odd terms
						_j = i + _mmax;
						
						
						_tempr = _wr * _data[_j-1][_col] - _wi * _data[_j][_col];
						_tempi = _wr * _data[_j][_col] + _wi * _data[_j-1][_col];
						
						_data[_j - 1][_col] = _data[i - 1][_col] - _tempr;
						_data[_j][_col] = _data[i][_col] - _tempi;
						
						_data[i-1][_col] += _tempr;
						_data[i][_col] += _tempi;
						
					}
					_wtemp = _wr;
					_wr = _wr * _wpr - _wi * _wpi +  _wr;
					_wi = _wi * _wpr + _wtemp * _wpi + _wi; 
				}
				_mmax = _istep;
			}
			
		}
		
		return _data;
	}


}
