package hiit.plugin.activity.utils;

import hiit.plugin.activity.data.Data;

import java.util.Enumeration;
import java.util.Hashtable;

/**
 * 
 * @author natau
 *
 * Interface for all the possible modules, including norms, filters, etc.
 * But: Not classifiers.
 * 
 */

public abstract class Module {
    protected Hashtable children = null;
    protected Module parent = null;
    protected String name = null;
    protected String[] dependencies = null;
    
    
    public Module(Module parent, String name){
        children = new Hashtable();
        setParent(parent);
        setName(name);
    }
    
    /**
     * Function for resetting the module, that is, resetting feature values etc.
     * @throws Exception 
     */
    public void reset() throws Exception
    {
        resetThis();
        if (children == null){
            return;
        }
        Enumeration elements = children.elements();
        while(elements.hasMoreElements()){
            ((Module)elements.nextElement()).reset();
        }
    }
    
    /**
     * Get the name of the module.
     * @return The name of the module.
     */
    public String getName() {
        return name;
    }
    
    /**
     * Set the name of the module.
     * @param name The name of the module.
     */
    public void setName(String name){
        if (name == null) this.name = "";
        else this.name = name;
    }   
        
    protected void setParent(Module parent){
        this.parent = parent;
    }
    
    public void addChild(Module child){
        this.children.put(child.getName(), child);
    }
    
    public Module getChild(String name){
        return (Module)children.get(name);
    }

    public String[] getDependencies(){
        return dependencies;
    }
    
    public String getFullName(){
        String result = "";
        if (parent != null){
            result = parent.getFullName();
        }
        if (result == null || result == "") return this.name;
        else return result + "." + this.name;
    }
    
    protected abstract void resetThis() throws Exception;
    
    
    public abstract Data getResult() throws Exception;
}
