package hiit.plugin.activity.utils;

import hiit.plugin.activity.feature.Feature;
import hiit.plugin.activity.feature.periodic.SpectralCentroid;
import hiit.plugin.activity.feature.periodic.MaxPos;
import hiit.plugin.activity.feature.statistical.AutoCorrelation;
import hiit.plugin.activity.feature.statistical.AverageAbsoluteDeviation;
import hiit.plugin.activity.feature.statistical.Energy;
import hiit.plugin.activity.feature.statistical.HarmonicMean;
import hiit.plugin.activity.feature.statistical.Kurtosis;
import hiit.plugin.activity.feature.statistical.Mean;
import hiit.plugin.activity.feature.statistical.MeanCrossingRate;
import hiit.plugin.activity.feature.statistical.RootMeanSquare;
import hiit.plugin.activity.feature.statistical.SignalToNoiseRatio;
import hiit.plugin.activity.feature.statistical.Skewness;
import hiit.plugin.activity.feature.statistical.StandardDeviation;
import hiit.plugin.activity.feature.statistical.Variance;
import hiit.plugin.activity.feature.statistical.ZeroCrossingRate;
import hiit.plugin.activity.feature.statistical.sorted.BowleySkewness;
import hiit.plugin.activity.feature.statistical.sorted.InterQuartileMean;
import hiit.plugin.activity.feature.statistical.sorted.InterQuartileRange;
import hiit.plugin.activity.feature.statistical.sorted.Maximum;
import hiit.plugin.activity.feature.statistical.sorted.Median;
import hiit.plugin.activity.feature.statistical.sorted.MidRange;
import hiit.plugin.activity.feature.statistical.sorted.Minimum;
import hiit.plugin.activity.feature.statistical.sorted.Percentile;
import hiit.plugin.activity.feature.statistical.sorted.QuartileVariationCoefficient;
import hiit.plugin.activity.feature.statistical.sorted.Range;
import hiit.plugin.activity.transform.ABS;
import hiit.plugin.activity.transform.FFT;
import hiit.plugin.activity.transform.None;
import hiit.plugin.activity.transform.STFT;
import hiit.plugin.activity.transform.Sort;
import hiit.plugin.activity.transform.XCorr;

import java.util.Vector;

public class TreeMan {
    
    private Module root = null;
    
    public void setRoot(Module root){
        this.root = root;
    }
    
    public Module addModule(String input) throws Exception{
        return addModule(input, root);
    }
    
    public Module addModule(String input, Module root) throws Exception{
        Vector elements = splitCommand(input);
        
        Module current = root;
        Module next = null;
        for (int i = 0; i < elements.size(); i++){
            if ((next = current.getChild((String)elements.elementAt(i))) == null){
                next = newModule((String)elements.elementAt(i), current);
                current.addChild(next);
                System.out.print(next.getName());
            }
            current = next;
        }
        System.out.println();
        return current;
    }
    
    private Vector splitCommand(String input){
        int oldIndex = 0;
        Vector elements = new Vector();
        for (int i = 0; i < input.length(); i++){
            if (input.charAt(i) == '.'){
                elements.addElement(input.substring(oldIndex, i));
                oldIndex = i+1;
            }
        }
        if (oldIndex < input.length() - 1) elements.addElement(input.substring(oldIndex, input.length()));
        return elements;
    }
    
    private Module newModule(String name, Module parent) throws Exception{
        
        Module result = null;
        
        for (int i = 0; i < 1; i++){
                        
            if (name.equals("ABS")){
                result =  new ABS(parent);
                continue;}
            if (name.equals("FFT")){
                result =  new FFT(parent);
                continue;}
            if (name.equals("NONE")){
                result =  new None(parent);
                continue;}
            if (name.equals("SORT")){
                result =  new Sort(parent);
                continue;}
            if (name.equals("XCORR")){
                result =  new XCorr(parent);
                continue;}
            if (name.equals("STFT")){
                result =  new STFT(parent);
                continue;}
            
            if (name.equals("ACORR")){
                result = new AutoCorrelation(parent);
                continue;}
            if (name.equals("AADEV")){
                result =  new AverageAbsoluteDeviation(parent);
                continue;}
            if (name.equals("BSKEW")){
                result =  new BowleySkewness(parent);
                continue;}
            if (name.equals("HMEAN")){
                result =  new HarmonicMean(parent);
                continue;}
            if (name.equals("IQMEAN")){
                result =  new InterQuartileMean(parent);
                continue;}
            if (name.equals("IQRANGE")){
                result =  new InterQuartileRange(parent);
                continue;}
            if (name.equals("ACORR")){
                result =  new AutoCorrelation(parent);
                continue;}
            if (name.equals("KURTOSIS")){
                result =  new Kurtosis(parent);
                continue;}
            if (name.equals("MAX")){
                result =  new Maximum(parent);
                continue;}
            if (name.equals("MEAN")){
                result =  new Mean(parent);
                continue;}
            if (name.equals("MCR")){
                result =  new MeanCrossingRate(parent);
                continue;}
            if (name.equals("MEDIAN")){
                result =  new Median(parent);
                continue;}
            if (name.equals("MIDRANGE")){
                result =  new MidRange(parent);
                continue;}
            if (name.equals("MIN")){
                result =  new Minimum(parent);
                continue;}
            if (name.equals("PERC")){
                result =  new Percentile(parent);
                continue;}
            if (name.equals("QVC")){
                result =  new QuartileVariationCoefficient(parent);
                continue;}
            if (name.equals("RANGE")){
                result =  new Range(parent);
                continue;}
            if (name.equals("RMS")){
                result =  new RootMeanSquare(parent);
                continue;}
            if (name.equals("SNR")){
                result =  new SignalToNoiseRatio(parent);
                continue;}
            if (name.equals("SKEW")){
                result =  new Skewness(parent);
                continue;}
            if (name.equals("STDDEV")){
                result =  new StandardDeviation(parent);
                continue;}
            if (name.equals("VAR")){
                result =  new Variance(parent);
                continue;}
            if (name.equals("ZCR")){
                result =  new ZeroCrossingRate(parent);
                continue;}
            if (name.equals("ACORR")){
                result =  new AutoCorrelation(parent);
                continue;}
            if (name.equals("ENERGY")){
                result =  new Energy(parent);
                continue;}
            if (name.equals("MAXPOS")){
                result =  new MaxPos(parent);
                continue;}
            if (name.equals("SC")){
                result =  new SpectralCentroid(parent);
                continue;}
            
            if (result == null) throw new Exception("You must have a syntax error there. Offending token: " + name);
        }        
        addDependencies(result.getDependencies(), parent);
        return result;
    }
    
    public Vector getFeatureNames(Vector features){
        Vector result = new Vector();
        for (int i = 0; i < features.size(); i++){
            result.addElement(((Feature)features.elementAt(i)).getFullName());
        }
        return result;
    }
    
    private void addDependencies(String[] dependencies, Module parent) throws Exception{
        if (dependencies == null || dependencies.length == 0) return;
        for (int i = 0; i < dependencies.length; i++){
            addModule(dependencies[i], parent);
        }
    }
}
