package hiit.plugin.activity.utils;

import hiit.plugin.activity.data.Data;
import hiit.plugin.activity.data.Instance;
import hiit.plugin.activity.data.Series;
import hiit.plugin.activity.norm.Norm;

import java.util.Vector;

public class Windower extends Module{
    private int windowSize = -1;
    private int windowHop = -1;
    public int noOfChannels = 0;
    public int outChannels = 0;
    private Norm norm = null;
    private Series result = null;
  //  private AccDataSource dataSource = null;
    private Vector buffer = null;

    /**
     * @param parent
     *            Mandatory AccDataSource
     * @param norm
     *            Mandatory Norm-Module
     * @param windowSize
     *            Defaults to 256, if no valid (>0) given
     * @param windowHop
     *            The size of the window hop. Defaults to windowSize, if no valid (>0; <=
     *            windowSize) given.
     * @param noOfChannels
     *            The number of data channels to expect. If no valid (>0) given, it will try to
     *            figure it out on its own from the first sample.
     * @throws Exception 
     */
    public Windower(AccDataSource parent, Norm norm, int windowSize, int windowHop,
            int noOfChannels) throws Exception {
        super(parent, "");
        if (parent == null || norm == null)
            throw new Exception ("Windower could not be initialized!");
        this.parent = parent;

        if (windowSize <= 0)
            this.windowSize = 256;
        else
            this.windowSize = windowSize;

        if (windowHop <= 0 || windowHop > windowSize)
            this.windowHop = this.windowSize;
        else
            this.windowHop = windowHop;

        this.outChannels = norm.outChannels();
        if (noOfChannels <= 0)
            this.noOfChannels = 0;
        else {
            this.noOfChannels = noOfChannels;
            buffer = new Vector(this.windowSize);
            if (this.outChannels == 0) this.outChannels = noOfChannels;
        }
        
        this.norm = norm;
        
        buffer = new Vector();
    }

    protected void resetThis() throws Exception {
        while (buffer.size() > windowSize - windowHop) { // argh, damn you, mobile java!
            buffer.removeElementAt(0);
        }
        result = null;
    }

    private void getNextWindow() throws Exception {
        Instance sampleSet;
        
        // fill the buffer up to the windowSize
        while (buffer.size() < windowSize) {
            sampleSet = (Instance)parent.getResult();
            // check on new label - only relevant in learning or labeled modes
            if (!sampleSet.valid) {
                buffer.removeAllElements();
                System.out.println("Flushed Window Buffer");
                continue;
            }
            
            if (sampleSet.getValues().length != noOfChannels){
                if (noOfChannels != 0) throw new Exception("Number of Channels changed!");
                else
                    noOfChannels = sampleSet.getValues().length;
            }
            buffer.addElement(norm.calculateNorm(sampleSet));
        }
    }

    public Data getWindow() throws Exception {
        if (buffer.size() < windowSize)
            getNextWindow();
        Instance[] temp = new Instance[windowSize];
        for (int i = 0; i < windowSize; i++) {
            temp[i] = (Instance) buffer.elementAt(i); // damn you, mobile java!
        }
        result = new Series(temp);
        return result;
    }

    public Data getResult() throws Exception {
        if (result == null) getWindow();
        return result;
    }
    
    //necessary for some testing purposes... really bad design :-)
    public String getLabel() throws Exception{
        return this.getWindow().getLabel();
    }
}