/*
 * BeTelGeuse - A context data gathering tool for Bluetooth devices
 * www.cs.helsinki.fi/group/acs/betelgeuse
 * Copyright (C) 2006 Joonas Kukkonen, Eemil Lagerspetz and Petteri Nurmi
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

package hiit.plugin.sampler;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Vector;

import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Choice;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.List;

import hiit.BetelgeuseMIDP;
import hiit.logger.Logger;
import hiit.plugin.MIDPPlugin;
import hiit.plugin.sampler.ConnectionException;

public class ExperienceSampler implements MIDPPlugin, CommandListener, Runnable{
	
	private Command alertYes = null;
	private Command alertNo = null;
	private Alert alert = null;
	private boolean showAlert = true;
	
	private int selectedIndex = 0;
	private Vector questionSets = new Vector();
	
	private List pluginScreen;
	private Command pluginBack;
	private Command showQuestions;
	
	//The displayable object that was shown before the new questions aler...
	//This displayable should be restored if the user chooses not to answer the questions or after user has sent the questions...
	private Displayable currentBefore;
	//The displayable object that should be restored when the user exits the main experience sampler plugin screen
	private Displayable currentBeTelGeuse;
	private Displayable beforeAlert;
	private Display display;
	
	private BetelgeuseMIDP bet;
	private boolean isRunning = true;
	private String url;
	private SamplerConnection conn = null;
	//How often is the status of BeTelGeuse sent?
	private long statusTimeout = 30000;
	//How long does the user have time to answer the questions=?

	private QuestionSet currentQuestions = null;
	private Logger logger;
	//Device ID assigned to this BeTelGeuse
	private int deviceID = 0;
	//Type of this implementation
	private int type = 0;
	
	public ExperienceSampler(BetelgeuseMIDP bet, Display display, String url, Logger logger){
		this.display = display;
		this.bet = bet;
		initializeGUI();
		this.bet.addPlugin(this);
		this.url = url;
		this.logger = logger;
		this.conn = new SamplerConnection(url);
	}
	
	private void initializeGUI(){
		this.alertYes = new Command("Yes", Command.OK, 0);
		this.alertNo = new Command("No", Command.CANCEL, 0);
		this.alert = new Alert("Questions", "Would you like to answer a few questions?", null, AlertType.CONFIRMATION);
		this.alert.addCommand(this.alertYes);
		this.alert.addCommand(this.alertNo);
		this.alert.setCommandListener(this);
		
		this.pluginScreen = new List("Experience Sampler", Choice.IMPLICIT);
				
		this.showQuestions = new Command("Select", Command.ITEM, 0);
		this.pluginScreen.addCommand(this.showQuestions);
		this.pluginBack = new Command("Back", Command.BACK, 0);
		this.pluginScreen.addCommand(this.pluginBack);
		this.pluginScreen.setCommandListener(this);
	}

	public void showPluginGui(Display display, Displayable current) {
		this.currentBeTelGeuse = current;
		this.display.setCurrent(this.pluginScreen);
	}

	public String getName() {
		return "Experience Sampler";
	}

	public void close() {
		this.isRunning = false;
		this.notify();
		//Final closing is done in the end of run method
	}

	public void commandAction(Command arg0, Displayable arg1) {
		if(arg1 == this.pluginScreen){
			if(arg0 == this.pluginBack){
				this.display.setCurrent(this.currentBeTelGeuse);
			}else if(arg0 == this.showQuestions){
				this.showAlert = false;
				this.selectedIndex = this.pluginScreen.getSelectedIndex();
				this.currentQuestions = (QuestionSet)this.questionSets.elementAt(selectedIndex);
				this.display.setCurrent(currentQuestions.getQuestionForm());
				this.currentBefore = this.pluginScreen;
			}
		}else if(arg1 == this.alert){
			if(arg0 == this.alertYes && this.currentQuestions != null){
				this.showAlert = false;
				this.currentBefore = this.beforeAlert;
				this.display.setCurrent(this.currentQuestions.getQuestionForm());
			}else if(arg0 == this.alertNo){
				this.display.setCurrent(this.beforeAlert);
			}
		}else if(arg0.getCommandType() == Command.BACK){
			this.display.setCurrent(this.currentBefore);
			this.showAlert = true;
		}else if(arg0.getLabel().equalsIgnoreCase("Send")){
			sendAnswers(this.currentQuestions.getAnswerMessage());
			this.showAlert = true;
			this.pluginScreen.delete(this.selectedIndex);
			this.questionSets.removeElementAt(this.selectedIndex);
		}
		
	}

	public synchronized void run() {
		if(this.logger != null) logger.logMessage("[ES] Experience sampler plugin started");
		boolean connected = false;
		while(isRunning){
			
			if(!connected){
				//Try connecting...
				if(this.openConnection()){
					connected = true;
				}else{
					try {
						//If connecting to the server failed, wait one minute and try again
						this.wait(60000);
					} catch (InterruptedException e) {
						// Nothing to do here... 
					}
					continue;
				}
			}
			
			int[] responce = null;
			try {
				responce = this.conn.receive();
			} catch (ConnectionException e) {
				continue;
			}
			if(responce[0] == 'S' || responce[0] == 's'){				
				int offset = 4;
				int messageLength = responce[1]*256*256 + responce[2]*256 + responce[3];
				int messageDeviceId = responce[offset]*256 + responce[offset+1];
				offset+=2;

				int questionSetId = responce[offset];
				offset++;
				
				if(this.deviceID != messageDeviceId)
					continue;
				
				QuestionSet newSet = new QuestionSet(this, questionSetId);
				
				int questions = responce[offset];
				offset++;
				
				for(int i=0; i<questions;i++){
					int questionNumber = responce[offset];
					offset++;
					int type = responce[offset];
					offset++;
					int length = responce[offset]*256 + responce[offset+1];
					offset+=2;
					String questionText = intArrayToString(responce, offset, length);
					offset += length;
					
					if(type == 1){
						newSet.addOpenQuestion(questionText, questionNumber);
					}else if(type == 2 || type == 3){
						int numberOfChoices = responce[offset];
						offset++;
						String[] choices = new String[numberOfChoices];
						for(int j=0; j<numberOfChoices;j++){
							int choiceLength = responce[offset];
							offset++;
							choices[j] = intArrayToString(responce, offset, choiceLength);
							offset += choiceLength;
						}
						if(type==2)
							newSet.addChoiceQuestion(questionText, questionNumber, choices, true);
						else
							newSet.addChoiceQuestion(questionText, questionNumber, choices, false);
					}
				}
				
				//Add new question set to the plugin screen and question set Vector...
				this.questionSets.addElement(newSet);
				this.pluginScreen.append("Question set: " + questionSetId, null);
				
				//Check if we should display an alert...
				
				if(this.showAlert){
					//Show this after the alert...
					this.currentBefore = this.display.getCurrent();
					//Element number of the last question set...
					this.selectedIndex = this.questionSets.indexOf(newSet);
					//Set the current questions variable...
					this.currentQuestions = newSet;
					//Display the alert...
					this.display.setCurrent(this.alert);
				}
			}
			
			//Wait between the status messages...
			//Check that the thread is still running (we don't want to wait if BTG is closing)
			if(isRunning){
				try {
					this.wait(this.statusTimeout);
				} catch (InterruptedException e) {
					continue;
				}
			}
		}
		
		ByteArrayOutputStream bout = new ByteArrayOutputStream();
		try {
			bout.write("Q".getBytes());
		} catch (IOException e) {
			// TODO 
		}
		intToBytes(2,3,bout);
		intToBytes(this.deviceID,2,bout);
		try {
			this.conn.send(bout.toByteArray());
			this.conn.closeConnection();
		} catch (ConnectionException e) {
			// Nothing to do in this case... We are closing BeTelGeuse anyways...
		}
	}
	
	/*
	 * Opens a connection to an experience sampling server
	 */
	
	private boolean openConnection(){
		String userName = this.bet.getUserName();
		
		try{
			this.deviceID = conn.openConnection(this.deviceID, this.type, userName);
		}catch(ConnectionException e){
			if (this.logger != null) this.logger.logMessage("[ES] " +"Could not send connection initialization message.");
			Alert a = new Alert("Could not send connection init message!");
			a.setTimeout(Alert.FOREVER);
			this.bet.showAlert(a);
			return false;
		}
		return true;
	}
	
	/*
	 * Sends answers to questions and resets the GUI...
	 */
	
	private void sendAnswers(String message){
		ByteArrayOutputStream bout = new ByteArrayOutputStream();
		byte[] data = message.getBytes();
		try{
			bout.write("A".getBytes());
			intToBytes(data.length+2,3,bout);
			intToBytes(this.deviceID,2,bout);
			bout.write(data);
		}catch (IOException e){
			//TODO
		}
		try{
			//Send the answers
			this.conn.send(bout.toByteArray());
		}catch(ConnectionException ce){
			if (this.logger != null) logger.logMessage("[ES] IOException: Unable to send an answer message to the server. Closing the plugin.");
			this.close();
		}

	}
	
	/*
	 * Convers an int array to string. Works like new String(int[], int, int) in normal Java.
	 * Note: There are no error checks in this method.
	 */
	
	private String intArrayToString(int[] array, int startIndex, int length){
		//Kids, do not use this at home... no error checks
		String temp = "";
		for(int i=startIndex; i<startIndex+length; i++)
			temp += (char)array[i];
		return temp;
	}
	
	/*
	 * Converts an int to bytes and stores them in ByteArrayOutputStream. The "size" parameter
	 * specifies how many bytes the result should take (converting 1 to bytes with size of 3
	 * results in bytes 0 0 1)
	 */
	
	private void intToBytes(int number, int size, ByteArrayOutputStream out){
		byte[] array = new byte[size];
		
		for (int i = size-1; i >= 0; i--){
			array[i] = (byte)(number % 256);
			number = number / 256;
		}
		try{
			out.write(array);
		}catch (IOException e){
			// TODO
		}
	}

}
