/*
 * BeTelGeuse - A context data gathering tool for Bluetooth devices
 * www.cs.helsinki.fi/group/acs/betelgeuse
 * Copyright (C) 2006 Joonas Kukkonen, Eemil Lagerspetz and Petteri Nurmi
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

package hiit.plugin.transmitter;

import javax.microedition.io.SocketConnection;
import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.StringItem;
import javax.microedition.lcdui.TextField;

import hiit.BetelgeuseMIDP;
import hiit.logger.Logger;
import hiit.plugin.MIDPPlugin;

/**
 * This class is the MIDP version of the TransmitterGUI.
 * @author Joonas Kukkonen and Eemil Lagerspetz
 *
 */

public class BoardTransmitterMIDPGUI implements MIDPPlugin, CommandListener {
    
    private BoardTransmitter transmitter;
    private Logger logger;
    
    private Displayable setAsCurrent;
    private Display display;
    
    //Main screen
    private Form mainScreen;
    private StringItem connectionString;
    private StringItem onString;
    private Command mainBack;
    private Command connectionComm;
    private Command optionsComm;
    
    //Connection screen
    private Form connectionScreen;
    private TextField host;
    private Command connectionBack;
    private Command saveConnection;
    private Command switchConnection;
    
    //Options screen
    private Form optionsScreen;
    private TextField maxSampleBox;
    // private TextField socketKeepAlive;
    // private TextField socketSendBufferSize;
    // private TextField adaptiveSendEnabled;
    // private TextField adaptiveSendValue;
    
    private Command optionsBack;
    private Command saveOptions;
    
    
    public BoardTransmitterMIDPGUI(BoardTransmitter transmitter, Logger logger){
        this.transmitter = transmitter;
        this.logger = logger;
        this.initialize();
    }
    
    private void initialize(){
        //Main screen initialization
        this.mainScreen = new Form("Transmitter");
        this.mainScreen.append("Data transmitter for Betelgeuse.\n\n");
        
        //this.connectionString = new StringItem(null, "Current connection url is: \nhttp://" + conn.getHost() + ":" + conn.getPort() + conn.getPath() + "\n\n");
        
        this.connectionString = new StringItem(null, "Current connection url is: "+ transmitter.getConnection().getAddress() +"\n\n");
        
        this.mainScreen.append(connectionString);   
        
        if (this.transmitter.switchedOn()) {
            this.onString = new StringItem(null, "The Transmitter is online\n\n");
        } else {
            this.onString = new StringItem(null, "The Transmitter is offline\n\n");
        }
        
        this.mainScreen.append(onString);   
        
        this.connectionComm = new Command("Connection", Command.ITEM, 1);
        this.mainScreen.addCommand(this.connectionComm);
        
        this.optionsComm = new Command("Options", Command.ITEM, 1);
        this.mainScreen.addCommand(this.optionsComm);
        
        this.mainBack = new Command("Back", Command.BACK, 1);
        this.mainScreen.addCommand(mainBack);
        
        this.switchConnection = new Command("Switch on/off", Command.ITEM, 1);
        this.mainScreen.addCommand(switchConnection);
        
        this.mainScreen.setCommandListener(this);
        
        //Connection screen initialization
        //TODO
        this.connectionScreen = new Form("Connection");
        this.host = new TextField("Address", this.transmitter.getConnection().getAddress(), 256, TextField.ANY);
        //this.port = new TextField("Host", this.transmitter.getConnection().getHost(), 6, TextField.NUMERIC);
        //this.path = new TextField("Host", this.transmitter.getConnection().getHost(), 256, TextField.ANY);
        
        this.connectionScreen.append(this.host);
        //this.connectionScreen.append(this.port);
        //this.connectionScreen.append(this.path);
        
        this.saveConnection = new Command("Save", Command.ITEM, 1);
        this.connectionScreen.addCommand(saveConnection);
        
        this.connectionBack = new Command("Back", Command.BACK, 1);
        this.connectionScreen.addCommand(this.connectionBack);
        
        this.connectionScreen.setCommandListener(this);
        
        
        //Options screen
        this.optionsScreen = new Form("Options");
        
        
        this.maxSampleBox = new TextField("Maximum read delay", "" + this.transmitter.getMaxSampleTimeout(), 9, TextField.NUMERIC);
        this.optionsScreen.append(this.maxSampleBox);
        
        /*
        // socket options
        try {
        	this.optionsScreen.append(new StringItem(null, "\nSocket options:\n"));
        	this.socketKeepAlive = new TextField("KEEPALIVE", "", 9, TextField.NUMERIC);
        	this.optionsScreen.append(this.socketKeepAlive);
        	
        	this.socketSendBufferSize = new TextField("SNDBUF", "", 9, TextField.NUMERIC);
        	this.optionsScreen.append(this.socketSendBufferSize);
        } catch (Exception e) {
        	BetelgeuseMIDP.logger.logMessage("BT GUI, error initing socket options: " + e);
        }
        */
        
        /*
        // adaptive send-interval
        this.optionsScreen.append(new StringItem(null, "\n\nAdaptive send-interval:\n"));
    	this.optionsScreen.append(this.adaptiveSendEnabled);
     	this.optionsScreen.append(this.adaptiveSendValue);
    	*/
    	
        
        this.saveOptions = new Command("Save", Command.ITEM, 1);
        this.optionsScreen.addCommand(this.saveOptions);
        
        this.optionsBack = new Command("Back", Command.BACK, 1);
        this.optionsScreen.addCommand(this.optionsBack);
        
        this.optionsScreen.setCommandListener(this);
        
    }
    
    /**
     * Shows(?) the main screen of the plugin.
     * @param display the display object of the device
     * @param current the displayable object currently shown
     */
    
    public void showPluginGui(Display display, Displayable current) {
		/*
    	// socket options
    	try {
		  	int keepAlive = this.transmitter.getConnection().getSocketOption(SocketConnection.KEEPALIVE);
		  	this.socketKeepAlive.setString("" + keepAlive);
		
		  	int sendBuffer = this.transmitter.getConnection().getSocketOption(SocketConnection.SNDBUF);
		  	this.socketSendBufferSize.setString("" + sendBuffer);
		} catch (Exception e) {
		  	BetelgeuseMIDP.logger.logMessage("BT GUI, error getting socket options: " + e);
		}
    	*/
    	
    	/*
    	// adaptive-send
    	this.adaptiveSendEnabled = new TextField("enabled", "" + this.transmitter.getAdaptiveSendEnabled(), 9, TextField.NUMERIC);
    	this.adaptiveSendValue= new TextField("value (0-1)", "" + this.transmitter.getAdaptiveSendIntervalValue(), 9, TextField.NUMERIC);
    	*/
    	
        this.display = display;
        this.setAsCurrent = current;
        this.display.setCurrent(this.mainScreen);
    }
    
    /**
     * Returns the name of the plugin.
     */
    
    public String getName() {
        return "Transmitter";
    }
    
    public void showSwitchedOn(boolean on){
        if (on)
            this.onString.setText("The Transmitter is online\n\n");
        else
            this.onString.setText("The Transmitter is offline\n\n");
    }
    
    public void commandAction(Command arg0, Displayable arg1) {
        if (arg1 == this.mainScreen) {
            if (arg0 == this.mainBack) {
                this.display.setCurrent(this.setAsCurrent);
            } else if (arg0 == this.connectionComm) {
                this.display.setCurrent(this.connectionScreen);
            } else if (arg0 == this.optionsComm) {
                this.display.setCurrent(this.optionsScreen);
            } else if(arg0 == this.switchConnection) {
                if (this.transmitter.switchedOn()) {
                    this.transmitter.switchOff();
                } else {
                    this.transmitter.switchOn();
                }
            }
        } else if (arg1 == this.connectionScreen) {
            if (arg0 == this.connectionBack) {
                this.display.setCurrent(this.mainScreen);
            } else if(arg0 == this.saveConnection) {
                String hostStr = this.host.getString();
                //String portStr = this.port.getString();
                //String pathStr = this.path.getString();
                
                BoardTransmitterConnection conn = this.transmitter.getConnection();
                conn.setAddress(hostStr);
                //conn.setAddress(hostStr + Integer.parseInt(portStr) + pathStr);
                //this.connectionString.setText("Current connection url is: \nhttp://" + conn.getHost() + ":" + conn.getPort() + conn.getPath() + "\n\n");
                this.connectionString.setText("Current connection url is: \n"+ conn.getAddress() +"\n\n");
            } 
        } else if (arg1 == this.optionsScreen) {
            if (arg0 == this.optionsBack) {
                this.display.setCurrent(this.mainScreen);
            } else if (arg0 == this.saveOptions) {
                saveTimeouts();
                
                /*
                // save socket options
                try {
	                int keepAlive = Integer.parseInt(this.socketKeepAlive.getString());
	                int sendBufferSize = Integer.parseInt(this.socketSendBufferSize.getString());
	                this.transmitter.getConnection().setSocketOption(SocketConnection.KEEPALIVE, keepAlive);
	                this.transmitter.getConnection().setSocketOption(SocketConnection.SNDBUF, sendBufferSize);
                } catch (Exception e) {
            	   BetelgeuseMIDP.logger.logMessage("BT GUI, could not set socoket options: " + e);
                }
                */
                
                // adaptive-send
                // this.transmitter.setAdaptiveSendEnabled(Integer.parseInt(this.adaptiveSendEnabled.getString()) == 1);
               // this.transmitter.setAdaptiveSendIntervalValue(Integer.parseInt(this.adaptiveSendValue.getString()));
            }
        }
    }
    
    /*
     * Readers timout values from textboxes and updates Transmitter's timeouts.
     */
    
    private void saveTimeouts() {
        char[] c1 = new char[this.maxSampleBox.size()];
        
        int i = this.maxSampleBox.getChars(c1), temp, result = 0;
        
        for(int j = 0; j < i; j++) {
            temp = c1[j];
            result = result * 10;
            result = result + temp - 48;
        }
        
        this.transmitter.setMaxSampleTimeout(result);
        result = 0;
    }
    
    public void close() {
        this.transmitter.close();
    }
}
