package hiit.reader;

import hiit.blackboard.Blackboard;
import hiit.blackboard.BlackboardFactory;
import hiit.blackboard.Customer;
import hiit.blackboard.CustomerData;
import hiit.blackboard.CustomerInfo;
import hiit.blackboard.event.EventData;
import hiit.blackboard.protocol.Protocol;
import hiit.connectorUnit.BTReaderFactory;
import hiit.logger.Logger;
import hiit.mapper.Properties;
import hiit.observer.Observer;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Date;
import java.util.Hashtable;
import java.util.Vector;

import javax.microedition.io.Connection;

public class CallReader extends ProducerReaderSkeleton {
    
    private String bta;
    
    private static final String INBOX_SIZE = "Inbox Size";
    private static final String UNREAD_MESSAGES = "Unread Messages";
    private static final String PHONE_STATE = "Phone State";
    
    private static final String CALL_CONNECTED_TIME = "Call Connected";
    private static final String CALL_DURATION = "Call Duration";
    private static final String CALL_DIRECTION = "Call Direction";
    private static final String CALL_NUMBER = "Call Number";
    
    private static final String SMS_TIME = "SMS Timestamp";
    private static final String SMS_DIRECTION = "SMS Direction";
    private static final String SMS_NUMBER = "SMS Number";
    
    private static final String SMS_READ_TIME = "Read SMS Time Arrived";
    private static final String SMS_READ_DIRECTION = "Read SMS Direction";
    private static final String SMS_READ_NUMBER = "Read SMS Number";
    private static final String SMS_READ_OPENED = "SMS Time Opened";
    
    
    public CallReader(){
        
    }
    
    public void initialize(Connection connection, String url, Logger logger,
            BTReaderFactory factory, int deviceId, boolean streamMode) {
        this.delay = 0;
        super.initialize(connection, url, logger, factory, deviceId, streamMode);
    }
    
    public void initialize(Connection connection, String url, Logger logger, BTReaderFactory factory, int deviceId, boolean mode, InputStream in, OutputStream out, int userId) {
        this.delay = 0;
        super.initialize(connection, url, logger, factory, deviceId, mode, in, out, userId);
    }
    
    protected void initializeVariables() {
        this.parameters = new Hashtable(14);
        this.parameters.put(INBOX_SIZE, "");
        this.parameters.put(UNREAD_MESSAGES, "");
        this.parameters.put(PHONE_STATE, "");
        this.parameters.put(CALL_CONNECTED_TIME, "");
        this.parameters.put(CALL_DURATION, "");
        this.parameters.put(CALL_DIRECTION, "");
        this.parameters.put(CALL_NUMBER, "");
        this.parameters.put(SMS_TIME, "");
        this.parameters.put(SMS_DIRECTION, "");
        this.parameters.put(SMS_NUMBER, "");
        this.parameters.put(SMS_READ_TIME, "");
        this.parameters.put(SMS_READ_DIRECTION, "");
        this.parameters.put(SMS_READ_NUMBER, "");
        this.parameters.put(SMS_READ_OPENED, "");
    }
    
    public int[] getParameterTypes() {
        int[] types = new int[14];
        types[0] = Properties.DATATYPE_STRING;
        types[1] = Properties.DATATYPE_STRING;
        types[2] = Properties.DATATYPE_STRING;
        types[3] = Properties.DATATYPE_STRING;
        types[4] = Properties.DATATYPE_STRING;
        types[5] = Properties.DATATYPE_STRING;
        types[6] = Properties.DATATYPE_STRING;
        types[7] = Properties.DATATYPE_STRING;
        types[8] = Properties.DATATYPE_STRING;
        types[9] = Properties.DATATYPE_STRING;
        types[10] = Properties.DATATYPE_STRING;
        types[11] = Properties.DATATYPE_STRING;
        types[12] = Properties.DATATYPE_STRING;
        types[13] = Properties.DATATYPE_STRING;
        return types;
    }
    
    public String[] getParameterNames() {
        String[] names = new String[14];
        
        names[0] = INBOX_SIZE;
        names[1] = UNREAD_MESSAGES;
        names[2] = PHONE_STATE;
        names[3] = CALL_CONNECTED_TIME;
        names[4] = CALL_DURATION;
        names[5] = CALL_DIRECTION;
        names[6] = CALL_NUMBER;
        names[7] = SMS_TIME;
        names[8] = SMS_DIRECTION;
        names[9] = SMS_NUMBER;
        names[10] = SMS_READ_TIME;
        names[11] = SMS_READ_DIRECTION;
        names[12] = SMS_READ_NUMBER;
        names[13] = SMS_READ_OPENED;
            
        return names;
    }

    public String getDeviceName() {
        return "CR";
    }
    
    public void updateProducerList(Blackboard bb, Vector customers) {
        for (int i = 0; i < customers.size(); ++i) {
            Customer cs = (Customer) customers.elementAt(i);
            if (cs == this)
                continue;
            // We only subscribe to newly appeared GSM Servers.
            if (cs.getInfo().getType() == hiit.mapper.Properties.CALL_SERVER)
                bb.subscribeToProducer(this, cs);
        }
    }

 
    
    protected void notifySubscribers() {
        
        // Read the current data only once
        Vector[] v = new Vector[this.parameters.size()];
        Vector temp = null;
        String[] names = this.getParameterNames();
        
        for (int i = 0; i < names.length; i++) {
            temp = new Vector();
            temp.addElement(this.parameters.get(names[i]));
            v[i] = temp;
        }
        
        BlackboardFactory.getBlackboard(logger).pushData(this,
                new CustomerData(v, new Date().getTime(), info), null );
        if (this.subscribers.size() == 0)
            return;
        
        synchronized (this) {
            long time = System.currentTimeMillis();
            for (int i = 0, j = this.subscribers.size(); i < j; i++) {
                if (((Long) this.lastSubscriberUpdate.elementAt(i)).longValue()
                        + ((Long) this.timeouts.elementAt(i)).longValue() < time) {
                    // synchronized(this.subscribers.elementAt(i)) {
                    // Send data to the observer. Observer decides what it does
                    // with the data.
                    ((Observer) this.subscribers.elementAt(i)).update(v, this);
                    this.lastSubscriberUpdate.removeElementAt(i);
                    this.lastSubscriberUpdate
                            .insertElementAt(new Long(time), i);
                    // Wake the observer
                    // this.subscribers.elementAt(i).notify();
                    // }
                }
            }
        }
    }

    public void readPushData() {
        try{
            // Parse the data from the call server
            Object[] stuff = Protocol.readPushData(in);
            int compId = ((Integer) stuff[0]).intValue();
            if (compId == -1)
                compId = Integer.MAX_VALUE;

            byte[] data = (byte[]) stuff[1];
            String s = new String(data);
            int tab = -1;

            tab = s.indexOf('\t');
            String inboxSize = (s.substring(0, tab));
            s = s.substring(tab + 1);
            tab = s.indexOf('\t');
            String unreadMessages = (s.substring(0, tab));
            s = s.substring(tab + 1);
            tab = s.indexOf('\t');
            String phoneState = (s.substring(0, tab));
            s = s.substring(tab + 1);

            String[] callPackages = null;
            String[] smsPackages = null;
            String[] smsReadPackages = null;

            //Read number of call packages
            tab = s.indexOf('\t');
            int numberOfCallPackages = Integer.parseInt((s.substring(0, tab)));
            s = s.substring(tab + 1);

            callPackages = new String[numberOfCallPackages];
            for(int i=0; i<numberOfCallPackages; i++){
                tab = s.indexOf('\t');
                callPackages[i] = s.substring(0, tab);
                s = s.substring(tab + 1);
            }

            //Read number of sms packages
            tab = s.indexOf('\t');
            int numberOfSmsPackages = Integer.parseInt((s.substring(0, tab)));
            s = s.substring(tab + 1);

            smsPackages = new String[numberOfSmsPackages];
            for(int i=0; i<numberOfSmsPackages; i++){
                tab = s.indexOf('\t');
                smsPackages[i] = s.substring(0, tab);
                s = s.substring(tab + 1);
            }

            //Read number of call packages
            tab = s.indexOf('\t');
            int numberOfSmsReadPackages = Integer.parseInt((s.substring(0, tab)));
            if (numberOfSmsReadPackages > 0)
                s = s.substring(tab + 1);

            smsReadPackages =  new String[numberOfSmsReadPackages];
            for(int i=0; i<numberOfSmsReadPackages; i++){
                tab = s.indexOf('\t');
                smsReadPackages[i] = s.substring(0, tab);
                s = s.substring(tab + 1);
            }

            /*
             * Check if we really have any new data or not...
             */
            if((numberOfCallPackages + numberOfSmsPackages + numberOfSmsReadPackages) == 0){
                this.helper.reset();
                return;
            }

            Vector[] parsedData = new Vector[14];

            Vector inboxSizeVec = new Vector();
            inboxSizeVec.addElement(inboxSize);
            parsedData[0] = inboxSizeVec;
            Vector unreadMessagesVec = new Vector();
            unreadMessagesVec.addElement(unreadMessages);
            parsedData[1] = unreadMessagesVec;
            Vector phoneStateVec = new Vector();
            phoneStateVec.addElement(phoneState);
            parsedData[2] = phoneStateVec;

            Vector callConnectedTime = new Vector();
            Vector callDuration = new Vector();
            Vector callDirection = new Vector();
            Vector callNumber = new Vector();

            for(int i=0; i<numberOfCallPackages; i++){
                String temp = callPackages[i];

                int coma = temp.indexOf(',');
                callConnectedTime.addElement(temp.substring(0,coma));
                temp = temp.substring(coma + 1);

                coma = temp.indexOf(',');
                callDuration.addElement(temp.substring(0, coma));
                temp = temp.substring(coma + 1);

                coma = temp.indexOf(',');
                callDirection.addElement(temp.substring(0, coma));
                callNumber.addElement(temp.substring(coma + 1));

            }

            parsedData[3] = callConnectedTime;
            parsedData[4] = callDuration;
            parsedData[5] = callDirection;
            parsedData[6] = callNumber;

            Vector smsTime = new Vector();
            Vector smsDirection = new Vector();
            Vector smsNumber = new Vector();

            for(int i=0; i<numberOfSmsPackages; i++){
                String temp = smsPackages[i];

                int coma = temp.indexOf(',');
                smsTime.addElement(temp.substring(0,coma));
                temp = temp.substring(coma + 1);

                coma = temp.indexOf(',');
                smsDirection.addElement(temp.substring(0, coma));
                smsNumber.addElement(temp.substring(coma + 1));
            }

            parsedData[7] = smsTime;
            parsedData[8] = smsDirection;
            parsedData[9] = smsNumber;

            Vector smsReadTime = new Vector();
            Vector smsReadDirection = new Vector();
            Vector smsReadNumber = new Vector();
            Vector smsReadTimeSpent = new Vector();

            for(int i=0; i<numberOfSmsReadPackages; i++){
                String temp = smsReadPackages[i];

                int coma = temp.indexOf(',');
                smsReadTime.addElement(temp.substring(0,coma));
                temp = temp.substring(coma + 1);

                coma = temp.indexOf(',');
                smsReadDirection.addElement(temp.substring(0, coma));
                temp = temp.substring(coma + 1);

                coma = temp.indexOf(',');
                smsReadNumber.addElement(temp.substring(0, coma));
                smsReadTimeSpent.addElement(temp.substring(coma + 1));

            }

            parsedData[10] = smsReadTime;
            parsedData[11] = smsReadDirection;
            parsedData[12] = smsReadNumber;
            parsedData[13] = smsReadTimeSpent;

            //Update the parameters...
            for (int i = 0; i < parsedData.length; ++i) {
                Vector sample = parsedData[i];

                if (sample.size() == 0){
                    continue;
                }

                String temp = (String)sample.elementAt(0);
                if(temp == null)
                    temp = "";
                switch(i){
                case 0:
                    this.parameters.put(INBOX_SIZE, temp);
                    break;
                case 1:
                    this.parameters.put(UNREAD_MESSAGES, temp);
                    break;
                case 2:
                    this.parameters.put(PHONE_STATE, temp);
                    break;
                case 3:
                    this.parameters.put(CALL_CONNECTED_TIME, temp);
                    break;
                case 4:
                    this.parameters.put(CALL_DURATION, temp);
                    break;
                case 5: 
                    this.parameters.put(CALL_DIRECTION, temp);
                    break;
                case 6: 
                    this.parameters.put(CALL_NUMBER, temp);
                    break;
                case 7: 
                    this.parameters.put(SMS_TIME, temp);
                    break;
                case 8: 
                    this.parameters.put(SMS_DIRECTION, temp);
                    break;
                case 9: 
                    this.parameters.put(SMS_NUMBER, temp);
                    break;
                case 10: 
                    this.parameters.put(SMS_READ_TIME, temp);
                    break;
                case 11: 
                    this.parameters.put(SMS_READ_DIRECTION, temp);
                    break;
                case 12: 
                    this.parameters.put(SMS_READ_NUMBER, temp);
                    break;
                case 13: 
                    this.parameters.put(SMS_READ_OPENED, temp);
                    break;
                }
            }

            //Push the data to the bb
            CustomerData d = new CustomerData(parsedData, new Date().getTime(), info);
            BlackboardFactory.getBlackboard(logger).pushData(this, d, null);
            this.helper.reset();

            //Update the subscribers (if any)
            if (this.subscribers.size() == 0)
                return;

            long time = System.currentTimeMillis();
            for (int i = 0, j = this.subscribers.size(); i < j; i++) {
                if (((Long) this.lastSubscriberUpdate.elementAt(i)).longValue()
                        + ((Long) this.timeouts.elementAt(i)).longValue() < time) {
                    // synchronized(this.subscribers.elementAt(i)) {
                    // Send data to the observer. Observer decides what it does
                    // with the data.
                    ((Observer) this.subscribers.elementAt(i)).update(parsedData, this);
                    this.lastSubscriberUpdate.removeElementAt(i);
                    this.lastSubscriberUpdate
                    .insertElementAt(new Long(time), i);
                    // Wake the observer
                    // this.subscribers.elementAt(i).notify();
                    // }
                }
            }
        }catch(Throwable t){
            if(this.logger != null) logger.logMessage("[CR] Throwable caught in readPushData(): " + t);
            t.printStackTrace();
        }
    }

    public void resetData() {
        // TODO Auto-generated method stub
        
    }


    public void newData(Customer prod, CustomerData pd, EventData eventData){
		// TODO Auto-generated method stub
	}
	
    public void markParameters(){
    	//Overwrite if your reader supports events... See GPS reader for example
    }
}
