package hiit.reader;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import javax.microedition.io.Connection;
import javax.microedition.io.StreamConnection;

import hiit.blackboard.Blackboard;
import hiit.blackboard.BlackboardFactory;
import hiit.blackboard.Customer;
import hiit.blackboard.CustomerData;
import hiit.blackboard.CustomerInfo;
import hiit.blackboard.CustomerSkeleton;
import hiit.blackboard.protocol.Protocol;
import hiit.connectorUnit.BTReaderFactory;
import hiit.logger.Logger;
import hiit.observer.Observer;

public abstract class ProducerReaderSkeleton extends CustomerSkeleton implements Reader  {
    
    protected String url;
    protected Logger logger;
    protected BTReaderFactory factory;
    protected int deviceId;
    protected Connection connection;
    protected boolean mode;
    
    protected int firstTimeout = 0;
    protected int finalTimeout = 0;
    protected boolean giveUp;
    
    protected boolean connected;
    protected boolean isRunning = true;
    protected boolean isDead = false;
    
    protected Hashtable parameters;
    
    //Delay between reading subsequent data messages
    protected long delay = 0;
    
    //Old component subscription variables...
    protected Vector subscribers = new Vector();
    protected Vector timeouts = new Vector();
    protected Vector lastSubscriberUpdate = new Vector();
    
    protected HelperThread helper;
    protected long lastTime;
    protected long resetTimeout = 120000;
    protected int userId;
    
    
    public ProducerReaderSkeleton(){
        
    }
    
    public ProducerReaderSkeleton(Connection connection, String url, Logger logger, BTReaderFactory factory, int deviceId, boolean streamMode, boolean giveUp) {
        this.url = url;
        this.logger = logger;
        this.factory = factory;
        this.deviceId = deviceId;
        // this.bluetoothAddress = bluetoothAddress;
        this.connection = connection;
        this.mode = streamMode;
        
        // Initialize the hash table
        initializeVariables();
        initializeInfo();
        
        this.giveUp = giveUp;
        
        // Try to create the connection
        try {
            if (connection != null){
                this.in = ((StreamConnection) connection).openInputStream();
                this.out = ((StreamConnection) connection).openOutputStream();
            }
        } catch (IOException ex) {
            if (logger != null)
                logger.logMessage(ex.toString());
            connected = false;
            isRunning = false;
            info.setAlive(false);

            try {
                connection.close();
            } catch (IOException exx) {
                // Do nothing
            }
        }

        BlackboardFactory.getBlackboard(logger).registerProducer(this, info, null);

    }
    
    public void initialize(Connection connection, String url, Logger logger, BTReaderFactory factory, int deviceId, boolean mode) {
        this.url = url;
        this.logger = logger;
        this.factory = factory;
        this.deviceId = deviceId;
        // this.bluetoothAddress = bluetoothAddress;
        this.connection = connection;
        this.mode = mode;
        
        // Initialize the hash table
        initializeVariables();
        initializeInfo();       
        
        // Try to create the connection
        try {
            if (connection != null){
                this.in = ((StreamConnection) connection).openInputStream();
                this.out = ((StreamConnection) connection).openOutputStream();
            }
        } catch (IOException ex) {
            if (logger != null)
                logger.logMessage(ex.toString());
            connected = false;
            isRunning = false;
            info.setAlive(false);
            try {
                connection.close();
            } catch (IOException exx) {
                // Do nothing
            }
        }
        BlackboardFactory.getBlackboard(logger).registerProducer(this, info, null);
    }

    public void initialize(Connection connection, String url, Logger logger, BTReaderFactory factory, int deviceId, boolean mode, InputStream in, OutputStream out, int userId) {
        this.userId = userId;
        this.url = url;
        this.logger = logger;
        this.factory = factory;
        this.deviceId = deviceId;
        // this.bluetoothAddress = bluetoothAddress;
        this.connection = connection;
        this.mode = mode;

        // Initialize the hash table
        initializeVariables();
        initializeInfo();       
        this.in = in;
        this.out = out;
        BlackboardFactory.getBlackboard(logger).registerProducer(this, info, null);
    }

    protected abstract void initializeVariables();
    
    protected void initializeInfo(){
        this.info = new CustomerInfo(this.getDeviceName(), this.deviceId, this.userId, this.getBluetoothAddress(), 0, this.delay, this.getParameterNames(), this.getParameterTypes(), false);
    }

    public void close() {
        //FIXME check that this works       
        this.isRunning = false;       
        try {
            this.in.close();
            this.out.close();
            if(this.connection != null)
                this.connection.close();
        } catch (IOException e) {
            // nothing to do here
        }
        this.connected = false;
        this.isDead = true;
    }
    
    protected void stopRunning(){
        this.close();
    }

    public String getBluetoothAddress() {
        return this.factory.getConnector().getGatherer().getBluetoothAddress();
    }

    public String[] getData() {
        
        // Avoid null pointer exceptions
        if (this.parameters == null)
            return null;
        
        // Reserve memory
        final int max = this.parameters.size();
        String[] params = new String[max];
        String[] paramNames = getParameterNames();
        
        // Loop though the keys and store the parameter names.
        for (int i=0; i<paramNames.length; i++){
            params[i] = "" + this.parameters.get(paramNames[i]);
        }
        
        // Return parameter names.
        return params;
    }

    public int getDeviceId() {
        return this.deviceId;
    }

    public abstract String getDeviceName();

    public int getLastDataNumber() {
        // TODO Auto-generated method stub
        return 0;
    }

    public String getParameter(String parameter) {
        return "" + this.parameters.get(parameter);
    }

    public abstract String[] getParameterNames();

    public abstract int[] getParameterTypes();

    public long getReadDelay() {
        return this.delay;
    }

    public boolean getStreamingMode() {
        return this.mode;
    }

    /** 
     * Getter for the reconnection timeouts.
     * 
     * @return a Vector with the elements: firstTimeout (int), finalTimeout
     *         (int), giveUp (boolean).
     */
    public Vector getTimeouts() {
        //NOTE: these do not have any effect, since the reader dies if the connection is lost
        //These are needed for the GUI to work properly
        Vector timeouts = new Vector();
        timeouts.addElement(new Integer(firstTimeout));
        timeouts.addElement(new Integer(finalTimeout));
        timeouts.addElement(new Boolean(giveUp));
        return timeouts;
    }

    public boolean isConnected() {
        return this.connected;
    }

    public boolean isDead() {
        return this.isDead;
    }

    public void readOne() {
        // TODO Auto-generated method stub

    }

    public void run() {
        try{
            isRunning = true;
            helper = new HelperThread();
            helper.start();
            while (running) {
                channelFree = true;
                int command = readCommand();
                if (command == -1)
                    return;
                switch (command) {
                case 's': // subscribe
                    readSubRequest(command);
                    break;
                case 'u': // unsubscribe
                    readUnSubRequest(command);
                    break;
                case 'p': // data from client
                    readPushData();
                    break;
                case 'l': // client requested list of components
                    readListComponents();
                    break;
                case 'r':
                    // Return - never sent by client
                case 't':
                    // Subscribed - Never sent by client.
                case 'e':
                    // Error message from client? Not possible...
                case 'c':
                    // New create? Not possible; this should be sent to the BB instead.
                default:
                    if (logger != null)
                        logger.logMessage(classId + " Received incorrect command byte: " + (char)command + " (" + (int)command + ")");
                }
            }
        }catch(Throwable t){
            if(this.logger != null) logger.logMessage("[PRS] Throwable caught in run(): " + t);
            t.printStackTrace();
        }

    }
    
    public abstract void readPushData();

    public void setReadDelay(long delay) {
        this.delay = delay;

    }

    public void setStreamingMode(boolean streaming) {
        this.mode = streaming;
    }

    /**
     * Sets timeouts for connection restoration. The timeout starts at
     * firstTimeout and it is multiplied with 2 each time connection restoration
     * fails. FinalTimeout defines the maximum lenght for a timeout. If giveUp
     * is true, the reader does not try to restore the connection after the
     * finalTimeout value is reached. If giveUp is false, the Reader tries to
     * restore connection indefinetly. Default firstTimeout is 4000 milliseconds
     * and finalTimeout 128000 milliseconds.
     * 
     * @param firstTimeout
     *            The lenght of first timeout in milliseconds
     * @param finalTimeout
     *            The maximum lenght of a timeout in milliseconds
     * @param giveUp
     *            Defines if the Reader gives up connection restoration after
     *            the final timeout is reached
     */
    
    public void setTimeouts(int firstTimeout, int finalTimeout, boolean giveUp) {
        //NOTE: these do not have any effect at the moment
        this.firstTimeout = firstTimeout;
        this.finalTimeout = finalTimeout;
        this.giveUp = giveUp;
    }

    /**
     * Subscribes to events from the reader.
     * 
     * @param obs
     *            The Observer that subscribes to events.
     */
    public void subscribe(Observer obs, long timeout) {
        synchronized (this) {
            this.subscribers.addElement(obs);
            this.timeouts.addElement(new Long(timeout));
            this.lastSubscriberUpdate.addElement(new Long(0));
        }
    }

    /**
     * Unsubscribes the Observer from events from the reader.
     * 
     * @param obs
     *            The Observer that unsubscribes.
     */
    public void unsubscribe(Observer obs) {
        synchronized (this) {
            int i = this.subscribers.indexOf(obs);
            this.subscribers.removeElement(obs);
            this.timeouts.removeElementAt(i);
            this.lastSubscriberUpdate.removeElementAt(i);
        }
    }

    public abstract void updateProducerList(Blackboard bb, Vector producers);
    
    public abstract void resetData();
    
    /*
     * HelperThread polls the data package buffer and sends the packages to the actual producer.
     * It is also used to keep track if new data has been received for a while.
     */
    
    class HelperThread extends Thread {
        
        public synchronized void run() {
            reset();
            while (isRunning) {
                try {
                    wait(5000);
                } catch (InterruptedException e) {
                    // Intentionally empty
                }
                if (channelFree) {
                    channelFree = false;
                    lookForAndReturnData();
                    channelFree = true;
                }
                long now = System.currentTimeMillis();
                if (lastTime + resetTimeout < now){ // not received anything for 2 minutes!
                    resetData();
                    lastTime = now;
                }
            }
        }
        
        private void lookForAndReturnData() {
            Enumeration c = sentData.keys();
            while (c.hasMoreElements()) {
                Customer z = (Customer) c.nextElement();
                if (z == null)
                    continue;
                CustomerData data = (CustomerData) sentData.get(z);
                ByteArrayOutputStream bos = new ByteArrayOutputStream();
                CustomerInfo ci = z.getInfo();
                byte[] datab = (byte[]) data.getData()[0].elementAt(0);
                Protocol.writeReturnData(bos, ci.getType(), ci.getUser(), ci
                        .getId(), datab);
                try {
                    if (BlackboardFactory.DEBUG && logger != null)
                        logger.logMessage(classId + " Sending data from " + z
                                + " to owner.");
                    out.write(bos.toByteArray());
                    out.flush();
                } catch (IOException e) {
                    if (logger != null)
                        logger.logMessage(classId + " Unable to send data back to our owner!");
                }
            }
            // clear the data.
            sentData = new Hashtable();
        }
        
        public void reset(){
            lastTime = System.currentTimeMillis();
        }
    }

}
