package hiit.util;

import hiit.logger.Logger;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.util.Enumeration;

import javax.microedition.io.Connector;
import javax.microedition.io.file.FileConnection;
import javax.microedition.io.file.FileSystemRegistry;

public class FileUtil {
	private static Logger logger;
	
	
	public FileUtil(Logger logger){
		this.logger = logger;
	}
	
	
	public static String readFile(String fileName) {
		try {
			//Open the file
			FileConnection fc = openFile(fileName, Connector.READ); 
			
			//Ok... Now we have a connection, but the file might not exists (doh!). Check the existence and create a file if needed
			if (fc == null)
				return null;
			
			if(!fc.exists())
				return null;
			
			
			StringBuffer buf = new StringBuffer();
			int charBufLen = 1000;
			char[] charBuf = new char[charBufLen];
	
			InputStream in = fc.openInputStream();
			InputStreamReader r = new InputStreamReader(in);
			while(r.read(charBuf, 0, charBufLen) > 0){
				buf.append(charBuf);
				charBuf = new char[charBufLen];
			}
			fc.close();
			
			return buf.toString();
		} catch (Exception e){
			logger.logMessage("FileReader: " + e);
			return null;
		}
	}
	public static boolean writeFile(String fileName, String content, boolean append){
		FileConnection fc = openFile(fileName, Connector.READ_WRITE); 
		if (fc.exists() && !append){
			try {
				fc.delete();
			} catch (IOException e) {
				if (logger != null) logger.logMessage("FileReader: writeFile: " + e);
				return false;
			}
		}
		try {
			fc.create();
		} catch (IOException e) {
			if (logger != null) logger.logMessage("FileReader: writeFile: " + e);
			return false;
		}
		try {
			long size = append == true? fc.fileSize() : 0;
			if (size == -1)
				size = 0;
			OutputStream out = fc.openOutputStream(size);
			out.write(content.getBytes());
		} catch (IOException e) {
			if (logger != null) logger.logMessage("FileReader: writeFile: " + e);
			return false;
		}
		try {
			fc.close();
		} catch (IOException e) {
			if (logger != null) logger.logMessage("FileReader: writeFile: " + e);
		}
		if (logger != null) logger.logMessage("FileReader: save: " + content);
		return true;
	}
	
	public static FileConnection openFile(String fileName, int mode) {
		String url = composeFilePath(fileName); 

		// Open the file
		FileConnection fc = null;
		try {
			fc = (FileConnection) Connector.open(url, mode);
		} catch (Exception e){
			
		}
		return fc;
	}
	public static String composeFilePath(String fileName){
		// Ok... The file system root might be "/" or "C:/". Find out which one.
		Enumeration rootEnum = FileSystemRegistry.listRoots();
		String root = "";
		while (rootEnum.hasMoreElements()) {
			String r = (String) rootEnum.nextElement();
			if (r.equalsIgnoreCase("e:/")) {
				root = r;
				break;
			}
			if (r.equals("c:/"))
				root = r;

		}

		return "file:///" + root + fileName;
	}
}
