package hiit.util;


import hiit.logger.Logger;

import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.io.*;
import java.io.*;
import java.util.Enumeration;
import java.util.Hashtable;


import org.kxml2.wap.WbxmlSerializer;
import org.xmlpull.v1.*;

public class Settings {
	public static final String DISCOVERY_MODE = "discovery-mode";
	public static final String DEVICE_INQUIRY = "device-inquiry";
	public static final String GUI_REDRAW = "gui-redraw";
	public static final String LOCAL_READER = "local-reader";
	public static final String TRANSMITTER = "transmitter";
	public static final String TRANSMITTER_ADDRESS = "transmitter-address";
	
	private final String GENERAL_TAG = "general";
	
	public static final int TYPE_INT = 0;
	public static final int TYPE_STRING = 1;
	
	// XML String
	private String xmlStr = "";
	private static Hashtable values = new Hashtable();
	private String prevKey = null;
	
	private static Logger logger;
	private static String filePath = null;
	public Settings(Logger logger) {
		this.logger = logger;
	}
	
	public void readSettings(String filePath){
		this.filePath = filePath;
		try {
			String xml = FileUtil.readFile(filePath);
			if (xml != null){
				ByteArrayInputStream xmlStream = new ByteArrayInputStream(xml.getBytes());
				
				try {
					XmlPullParser parser = XmlPullParserFactory.newInstance()
							.newPullParser();
	
					parser.setInput(xmlStream, null);
					parser.nextTag();
					parser.nextTag();  // start reading from "general"
					
					processDocument(parser);
				} catch (Exception e) {
					System.out.println("e " + e);
				}
			}
		} catch (Exception e) {
			logger.logMessage("local: " + e);
		}
	}
	public static String getValue(String key){
		return (String)values.get(key);
	}
	public static void setValue(String key, Object value){
		if (values.containsKey(key))
			values.remove(key);
		
		values.put(key, value);
	}
	public static void save() {
		String s = "";
		s += "<?xml version=\"1.0\"?>\n";
		s += "<settings>\n";
		s += "<general>\n";
		
		Enumeration e = values.keys();
		while (e.hasMoreElements()) {
			String key = (String) e.nextElement();
			Object value = values.get(key);
			s += "<" + key + ">" + value + "</" + key + ">\n";
		}
		s += "</general>\n";
		s += "</settings>\n";

		FileUtil.writeFile(filePath, s.toString(), false);
	}
	
    private void processDocument(XmlPullParser parser) throws XmlPullParserException, IOException {
    	int eventType = parser.getEventType();
		do {
			if (eventType == parser.START_DOCUMENT) {
			} else if (eventType == parser.END_DOCUMENT) {
			} else if (eventType == parser.START_TAG) {
				prevKey = parser.getName();
			} else if (eventType == parser.END_TAG) {
				prevKey = null;
			} else if (eventType == parser.TEXT) {
				if (prevKey != null && !prevKey.equals(GENERAL_TAG))
					values.put(prevKey, parser.getText());
			}
			eventType = parser.next();
		} while (eventType != parser.END_DOCUMENT);
	}

}
