package utils;

import hiit.plugin.activity.data.Data;
import hiit.plugin.activity.data.Instance;
import hiit.plugin.activity.utils.AccDataSource;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;

//import javax.microedition.io.file.FileConnection;

/**
 * Read data from a resource file. Supports various window sizes and allows using sliding windows.
 * 
 * @author ptnurmi
 * @version 1.0
 */
public class AccFileReader extends AccDataSource {

    private FileReader inputFile = null; 
    private BufferedReader bufferedFile = null;
    
    /**
     * The current label (class) of the data instances.
     */
    private String _currentLabel = null;

    /**
     * Flag indicating that reading has been started. Once reading has been started, the parameters
     * (window size or sliding window) cannot be modified.
     */
    private boolean _readingStarted = false;

    /**
     * Flag indicating that the file has been closed.
     */
    private boolean _isClosed = false;

    private String _tmpLabel = "";
    
    private boolean isLabeled = false;
    
    private String [] files = null;
    private int fileIndex = 0;

    /**
     * Get the label of the current activity
     * 
     * @return The label of the current activity
     */
    public String getLabel() {
        if (this._tmpLabel != "")
            return this._tmpLabel + "-" + this._currentLabel;
        else
            return this._currentLabel;
    }

    /**
     * Create a new data reader. Uses the given filename as the resource file. The second parameter
     * determines the window size to use and the third parameter determines whether a sliding window
     * is used.
     * 
     * @param _dataFileName
     *            The name of the file to use.
     * @param isLabeled
     *            Does the file have labels?
     *            Note: if true, labels will be _mandatory_
     * @throws Exception 
     */
    
    public AccFileReader(String _dataFileName, boolean isLabeled) throws Exception {
        super(null, null);
        this.isLabeled = isLabeled;
        openFile(_dataFileName);

    }
    
    public AccFileReader(String [] files, boolean isLabeled) throws Exception {
        super(null, null);
        this.files = files;
        this.isLabeled = isLabeled;
        if (files.length > 0){
            fileIndex = 0;
            openFile(files[0]);
        }
    }

    private void openFile(String fileName) throws Exception{
        // Try to create an input stream to the appropriate resource file.
        inputFile = new FileReader(fileName);
        bufferedFile = new BufferedReader(inputFile);
        
        if (bufferedFile == null)
            throw new Exception("FileReader: File opening failed!");
    }
    
    /**
     * Determine whether the reader has been closed. Used to determine whether the end of file has
     * been reached.
     * 
     * @return True if the end of file has been reached, false otherwise.
     */
    public boolean isClosed() {
        return _isClosed;
    }

    private String convertLabel(String oldLabel, String mode){
        //if (mode.regionMatches(0, "only", 0, 4)){
        {  if (oldLabel.equals("walk-pocket") ||
                oldLabel.equals("walk-bag"))
                oldLabel = "walk";
            if (oldLabel.equals("bwalk-pocket") ||
                    oldLabel.equals("bwalk-bag"))
                oldLabel = "bwalk";
            if (oldLabel.equals("jog-pocket") ||
                    oldLabel.equals("jog-bag"))
                oldLabel = "jog";
            if (oldLabel.equals("run-pocket") ||
                    oldLabel.equals("run-bag"))
                oldLabel = "run";
        }
//        if (mode.regionMatches(0, "base", 0, 4) || mode.regionMatches(0, "only", 0, 4)){
//           if (oldLabel.equals("walk") ||
//                    oldLabel.equals("bwalk") ||
//                    oldLabel.equals("jog") ||
//                    oldLabel.equals("run"))
//                    oldLabel = "human";
//            if (oldLabel.equals("car") ||
//                    oldLabel.equals("tram") ||
//                    oldLabel.equals("bus") ||
//                    oldLabel.equals("metro"))
//                oldLabel = "motor"; 
//            if (oldLabel.equals("bus-rest") || oldLabel.equals("metro-rest")
//                    || oldLabel.equals("rest"))
//                oldLabel = "rest";
//        }
        if (mode.regionMatches(0, "only", 0, 4)){
            mode = mode.substring(4);
            if (!oldLabel.equals(mode)) oldLabel = "other";
        }
        return oldLabel;
    }
    
    /**
     * Parse a line of data into an array of double values. This method finds the number of
     * potential double entries in the data line and then calls another auxiliary method to return
     * the final results.
     * 
     * @param _line
     *             The line of data to parse.
     * @return 
     *              An Instance containing the read data. Returns an invalid Instance on EOF, and null,
     *              if the line doesn't contain appropriate data 
     * @throws Exception 
     */
    private Instance parseValues(String _line) throws Exception {

        // Check that the line that was read is valid.
        if (_line == null) //eof
            return new Instance(false); 

        if (isLabeled){
            char first = _line.charAt(0);
            if (first >= 'a' && first <= 'z' || first >= 'A' && first <= 'Z'){ //label
                this._currentLabel = convertLabel(_line, "none");
//                this._currentLabel = _line;
                System.out.println(_currentLabel);
                return new Instance(false);
            }
        }
        
        // Unify delimiters
        _line = _line.replace(' ', ',');
        _line = _line.replace('\t', ',');

        // Get the length of the string
        final int _length = _line.length();

        // Auxiliary variables, the current position within the line and
        // the number of times the delimiter has occurred within the string.
        int _curPos = 0;
        int _count = 0;

        // Go through the string
        while (_curPos < _length) {

            // Search for the next delimiter
            final int _nextDelim = _line.indexOf(',', _curPos);

            // Increase the counter
            _count++;

            // If at the end of line, stop
            if (_nextDelim == -1)
                break;

            // Update location within the string
            _curPos = _nextDelim + 1;
        }

        // If no values found, return null
        if (_count <= 0)
            return null;

        Instance result = new Instance(getEntries(_line, _count));
        
        if (isLabeled && (_currentLabel == null || _currentLabel == "") && (result != null && result.getValues().length != 0))
            throw new Exception("Unlabeled data found in file!");
        else if (isLabeled) {
            result.setLabel(_currentLabel);
            if (result.getLabel() == null)
                throw new Exception("bla");
        }
        
        return result;

    }

    /**
     * Parse a line of data into an array of double entries.
     * 
     * @param _line
     *            The line to parse.
     * @param _noValues
     *            The number of double values we expect the line to contain.
     * @return An array of double values.
     */
    private double[] getEntries(String _line, int _noValues) {

        // Storage for the values
        double[] _values = new double[_noValues];

        // Auxiliary variables, the index of the current value that we are processing and the
        // current position within the line.
        int _index = 0;
        int _curPos = 0;

        // Loop until we have read enough values
        while (_index < _noValues) {

            // Get the next index of the delimiter
            final int _nextDelim = _line.indexOf(',', _curPos);

            // Try to parse values
            try {

                // If we have reached the end of the string, use the remainder of the line
                // as the candidate for a data value
                if (_nextDelim == -1) {
                    _values[_index++] = Double.parseDouble(_line
                            .substring(_curPos));
                }

                // Otherwise use the appropriate substring
                else {
                    _values[_index++] = Double.parseDouble(_line.substring(
                            _curPos, _nextDelim));

                }

                // Update the pointer within the line
                _curPos = _nextDelim + 1;

                // If we have reached end of the line and we have not found enough values give an
                // error.
                if (_nextDelim == -1 && _index < _noValues)
                    return null;
            }

            // If an error occurs, the values are not doubles and we return null;
            catch (NumberFormatException _numberFormatException) {
                return null;
            }

        }

        // Return results
        return _values;
    }

 
    /**
     * Read the next line from the file.
     * 
     * @return A string containing the next line of data from the data file.
     * @throws Exception 
     */
    private String readNextLine() throws Exception {

        // If the stream has been closed return null
        if (_isClosed)
            return null;

        String nextLine = bufferedFile.readLine();
        
        if (nextLine == null){ //eof
            if (files != null && fileIndex < files.length - 1){
                openFile(files[++fileIndex]);
            }
            else throw new Exception("EOF"); //yeah, i know that's not the way to do it...
        }     
        
        return nextLine; 
    }

    /**
     * Close the reader. Closes the stream to the file.
     * @throws Exception 
     */
    public void close() throws Exception {
        if (bufferedFile != null) bufferedFile.close();
        // Close the stream
        if (inputFile != null) inputFile.close();

        // Update the closed flag
        _isClosed = true;
    }

    public Data getResult() throws Exception {
        Instance temp = null;
        while (temp == null) {
            temp = parseValues(readNextLine());
        }
        return temp;
    }
}