package utils;

import hiit.plugin.activity.data.Instance;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.Vector;

@SuppressWarnings("unchecked")
public class ArffWriter {
    private Vector<String> activities = new Vector<String>();
    private Vector values = new Vector();
    private Vector features = null;
    private String lastActivity = "";
    private boolean headerWritten = false;
    private int channels = 0;
    private File outFile = null;
    private FileWriter outFileWriter = null;
    private BufferedWriter outBufferedWriter = null;
    private String fileName = null;

    public ArffWriter(String path, String fileName, int channels) throws Exception {
        outFile = new File(path + "/" + fileName);
        outFileWriter = new FileWriter(outFile);
        outBufferedWriter = new BufferedWriter(outFileWriter);
        this.fileName = fileName;
        this.channels = channels;
    }

    public void addValues(Vector featureValues) throws Exception {
        if (featureValues == null || featureValues.size() == 0)
            throw new Exception("There were no featureValues to add!");
        String activity = ((Instance)featureValues.elementAt(0)).getLabel();
        if (channels == 0){
            channels = ((Instance)featureValues.elementAt(0)).getValues().length;
        }
        if(!((Instance)featureValues.elementAt(0)).valid){
            activity = "";
        }
        
        if (activity != null && !activity.equals(lastActivity)){
            lastActivity = activity;
            int index = -1;
            for (int i = 0; i < activities.size(); i++){
                if (activities.elementAt(i).equals(activity)) index = i;
            }
            if (index == -1){ 
                activities.add(activity);
            }
        }
        this.values.add(featureValues);
    }

    public void initFeatures(Vector featureNames) {
        features = featureNames;
    }

    public void writeOut() throws Exception {
        if (activities.size() == 0 || values.size() == 0)
            throw new Exception("No data to write!");
  
        //write the header only the first time - this here is a hack because of too little heap size
        //this only works, if all the activities have already been seen...
        //better to use just enough memory or give the activities by hand!
        if (!headerWritten) { 
            outBufferedWriter.write("@relation " + fileName + "\n");
            for (int i = 0; i < features.size(); i++) { // cycle through the features..
                for (int j = 0; j < channels; j++) { // cycle through the data channels
                    outBufferedWriter.write("@attribute "
                                    + (String) features.elementAt(i) + j
                                    + " numeric\n");
                }
            }
            outBufferedWriter.write("@attribute action {");
            for (int i = 0; i < activities.size(); i++) {
                outBufferedWriter.write((String) activities.elementAt(i));
                if (i < activities.size() - 1)
                    outBufferedWriter.write(", ");
            }

            outBufferedWriter.write("}\n" + "@data\n\n");
            headerWritten = true;
        }

        for (int j = 0; j < values.size(); j++) { // through the feature-vectors...
            Vector fVector = (Vector) values.elementAt(j);
            for (int r = 0; r < fVector.size(); r++) { // through the current feature-vector
                Instance currInstance = (Instance)fVector.elementAt(r);
                for (int s = 0; s < channels; s++) { // through the current feature - there's
                    // one value for every data channel!
                    outBufferedWriter.write((float)currInstance.getValues()[s] + " ");
                }
            }
            outBufferedWriter.write(((Instance)fVector.elementAt(0)).getLabel() + "\n");
        }
       
        values.clear();
    }

    public void close() throws Exception {
        outBufferedWriter.flush();
        outBufferedWriter.close();
        
        values = null;
        activities = null;
    }
}
